#------------------------------------------------
#
# testfreq.tcl --
#    This is a Tcl script which tests the freq program.
#
#------------------------------------------------

proc cequal {left right} {
   return [expr [string compare $left $right] == 0]
}

proc lempty {string} {
   return [expr [string length $string] == 0]
}

# Get command line arguments
#---------------------------
set source_dir [lindex $argv 0]
set platform   [lindex $argv 1]
set words      [lindex $argv 2]

puts stdout "Testing freq"

# Delete output files
#--------------------
puts stdout "   Deleting output files..."
file delete $words freq.tcl.out freq.ada.out

# Prepare file of words
#----------------------
puts stdout "   Preparing file of words..."
exec wordify < [file join $source_dir intro.txt] > $words

# Execute Tcl script version
#---------------------------
puts stdout "   Executing Tcl script version..."
set tcl_time [time "exec tash freq.tcl < $words > freq.tcl.out" 1]

# Execute Ada version
#--------------------
puts stdout "   Executing Ada version..."
set ada_time [time "exec ./freq < $words > freq.ada.out" 1]

# Compare outputs of both versions
#---------------------------------
puts stdout "   Comparing outputs..."
set diff [exec compare freq.tcl.out freq.ada.out]
if [lempty $diff] {
   puts stdout "Freq test PASSED"
} else {
   puts stdout "Freq test FAILED: $diff"
}

# Display timing results
#-----------------------
set tcl_time [expr [lindex $tcl_time 0] / 1000000.0]
puts stdout [format "   Elapsed time for executing Tcl script version:\
   %6.2f seconds" $tcl_time]

set ada_time [expr [lindex $ada_time 0] / 1000000.0]
puts stdout [format "   Elapsed time for executing Ada version:       \
   %6.2f seconds" $ada_time]
