
--------------------------------------------------------------------
--
-- tcl-ada.adb --
--
-- Copyright (c) 1995-1998 Terry J. Westley
--
-- See the file "license.htm" for information on usage and
-- redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
--
--------------------------------------------------------------------

with Ada.Exceptions;

package body Tcl.Ada is

   use type C.Int;

   procedure Assert (
      Interp      : Tcl_Interp;
      Return_Code : C.Int) is
   -- Raises Tcl_Error_Exception if Return_Code = TCL_ERROR
   begin -- Assert
      if Return_Code = TCL_ERROR then
         Standard.Ada.Exceptions.Raise_Exception (
            Tcl_Error_Exception'Identity, Tcl_GetResult (Interp));
      end if;
end Assert;
   package body Generic_AssocData is

      function Tcl_GetAssocData (
         interp          : in Tcl_Interp;
         name            : in String;
         procPtr         : in Tcl_InterpDeleteProc
      ) return ClientData is
         C_name : aliased C.Char_Array := C.To_C (name);
      begin -- Tcl_GetAssocData
         return Tcl_GetAssocData (
            interp,
            C.Strings.To_Chars_Ptr (C_name'unchecked_access),
            procPtr);
      end Tcl_GetAssocData;

      procedure Tcl_SetAssocData (
         interp          : in Tcl_Interp;
         name            : in String;
         proc            : in Tcl_InterpDeleteProc;
         data            : in ClientData
      ) is
         C_name : aliased C.Char_Array := C.To_C (name);
      begin -- Tcl_SetAssocData
         Tcl_SetAssocData (
            interp,
            C.Strings.To_Chars_Ptr (C_name'unchecked_access),
            proc,
            data);
      end Tcl_SetAssocData;

   end Generic_AssocData;

   package body Generic_AsyncEvents is

   end Generic_AsyncEvents;

   package body Generic_CallWhenDeleted is

   end Generic_CallWhenDeleted;

   package body Generic_Channel is

   end Generic_Channel;

   package body Generic_ChannelDriver is

      function Tcl_CreateChannel (
         typePtr         : in Tcl_ChannelType;
         chanName        : in String;
         instancedata    : in ClientData;
         mask            : in C.Int
      ) return Tcl_Channel is
         C_chanName : aliased C.Char_Array := C.To_C (chanName);
      begin -- Tcl_CreateChannel
         return Tcl_CreateChannel (
            typePtr,
            C.Strings.To_Chars_Ptr (C_chanName'unchecked_access),
            instancedata,
            mask);
      end Tcl_CreateChannel;

   end Generic_ChannelDriver;

   package body Generic_ChannelHandler is

   end Generic_ChannelHandler;

   package body Generic_CloseHandler is

   end Generic_CloseHandler;

   package body Generic_Command is

      function Tcl_CreateCommand (
         interp          : in Tcl_Interp;
         cmdName         : in String;
         proc            : in Tcl_CmdProc;
         data            : in ClientData;
         deleteProc      : in Tcl_CmdDeleteProc
      ) return Tcl_Command is
         C_cmdName : aliased C.Char_Array := C.To_C (cmdName);
      begin -- Tcl_CreateCommand
         return Tcl_CreateCommand (
            interp,
            C.Strings.To_Chars_Ptr (C_cmdName'unchecked_access),
            proc,
            data,
            deleteProc);
      end Tcl_CreateCommand;

   end Generic_Command;

   package body Generic_EventSource is

   end Generic_EventSource;

   package body Generic_ExitHandler is

   end Generic_ExitHandler;

   package body Generic_FileHandler is

   end Generic_FileHandler;

   package body Generic_GetOpenFile is

      function Tcl_GetOpenFile (
         interp          : in Tcl_Interp;
         strng           : in String;
         write           : in C.Int;
         checkUsage      : in C.Int;
         fileptr         : in ClientData
      ) return C.Int is
         C_strng : aliased C.Char_Array := C.To_C (strng);
      begin -- Tcl_GetOpenFile
         return Tcl_GetOpenFile (
            interp,
            C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
            write,
            checkUsage,
            fileptr);
      end Tcl_GetOpenFile;

      procedure Tcl_GetOpenFile (
         interp          : in Tcl_Interp;
         strng           : in String;
         write           : in C.Int;
         checkUsage      : in C.Int;
         fileptr         : in ClientData
      ) is
         C_strng : aliased C.Char_Array := C.To_C (strng);
      begin -- Tcl_GetOpenFile
         Assert (interp, Tcl_GetOpenFile (
            interp,
            C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
            write,
            checkUsage,
            fileptr));
      end Tcl_GetOpenFile;

   end Generic_GetOpenFile;

   package body Generic_Hash is

   end Generic_Hash;

   package body Generic_Idle is

   end Generic_Idle;

   package body Generic_ManageStorage is

   end Generic_ManageStorage;

   package body Generic_MathFunc is

      procedure Tcl_CreateMathFunc (
         interp          : in Tcl_Interp;
         name            : in String;
         numArgs         : in C.Int;
         argTypes        : in Tcl_ValueType;
         proc            : in Tcl_MathProc;
         data            : in ClientData
      ) is
         C_name : aliased C.Char_Array := C.To_C (name);
      begin -- Tcl_CreateMathFunc
         Tcl_CreateMathFunc (
            interp,
            C.Strings.To_Chars_Ptr (C_name'unchecked_access),
            numArgs,
            argTypes,
            proc,
            data);
      end Tcl_CreateMathFunc;

   end Generic_MathFunc;

   package body Generic_ObjCommand is

      function Tcl_CreateObjCommand (
         interp          : in Tcl_Interp;
         cmdName         : in String;
         proc            : in Tcl_ObjCmdProc;
         data            : in ClientData;
         deleteProc      : in Tcl_CmdDeleteProc
      ) return Tcl_Command is
         C_cmdName : aliased C.Char_Array := C.To_C (cmdName);
      begin -- Tcl_CreateObjCommand
         return Tcl_CreateObjCommand (
            interp,
            C.Strings.To_Chars_Ptr (C_cmdName'unchecked_access),
            proc,
            data,
            deleteProc);
      end Tcl_CreateObjCommand;

   end Generic_ObjCommand;

   package body Generic_TcpChannel is

      function Tcl_OpenTcpServer (
         interp          : in Tcl_Interp;
         port            : in C.Int;
         host            : in String;
         acceptProc      : in Tcl_TcpAcceptProc;
         callbackdata    : in ClientData
      ) return Tcl_Channel is
         C_host : aliased C.Char_Array := C.To_C (host);
      begin -- Tcl_OpenTcpServer
         return Tcl_OpenTcpServer (
            interp,
            port,
            C.Strings.To_Chars_Ptr (C_host'unchecked_access),
            acceptProc,
            callbackdata);
      end Tcl_OpenTcpServer;

   end Generic_TcpChannel;

   package body Generic_TimerHandler is

   end Generic_TimerHandler;

   package body Generic_Trace is

   end Generic_Trace;

   package body Generic_TraceVar is

      function Tcl_TraceVar (
         interp          : in Tcl_Interp;
         varName         : in String;
         flags           : in C.Int;
         proc            : in Tcl_VarTraceProc;
         data            : in ClientData
      ) return C.Int is
         C_varName : aliased C.Char_Array := C.To_C (varName);
      begin -- Tcl_TraceVar
         return Tcl_TraceVar (
            interp,
            C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
            flags,
            proc,
            data);
      end Tcl_TraceVar;

      procedure Tcl_TraceVar (
         interp          : in Tcl_Interp;
         varName         : in String;
         flags           : in C.Int;
         proc            : in Tcl_VarTraceProc;
         data            : in ClientData
      ) is
         C_varName : aliased C.Char_Array := C.To_C (varName);
      begin -- Tcl_TraceVar
         Assert (interp, Tcl_TraceVar (
            interp,
            C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
            flags,
            proc,
            data));
      end Tcl_TraceVar;

      function Tcl_TraceVar2 (
         interp          : in Tcl_Interp;
         part1           : in String;
         part2           : in String;
         flags           : in C.Int;
         proc            : in Tcl_VarTraceProc;
         data            : in ClientData
      ) return C.Int is
         C_part1 : aliased C.Char_Array := C.To_C (part1);
         C_part2 : aliased C.Char_Array := C.To_C (part2);
      begin -- Tcl_TraceVar2
         return Tcl_TraceVar2 (
            interp,
            C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
            C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
            flags,
            proc,
            data);
      end Tcl_TraceVar2;

      procedure Tcl_TraceVar2 (
         interp          : in Tcl_Interp;
         part1           : in String;
         part2           : in String;
         flags           : in C.Int;
         proc            : in Tcl_VarTraceProc;
         data            : in ClientData
      ) is
         C_part1 : aliased C.Char_Array := C.To_C (part1);
         C_part2 : aliased C.Char_Array := C.To_C (part2);
      begin -- Tcl_TraceVar2
         Assert (interp, Tcl_TraceVar2 (
            interp,
            C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
            C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
            flags,
            proc,
            data));
      end Tcl_TraceVar2;

      procedure Tcl_UntraceVar (
         interp          : in Tcl_Interp;
         varName         : in String;
         flags           : in C.Int;
         proc            : in Tcl_VarTraceProc;
         data            : in ClientData
      ) is
         C_varName : aliased C.Char_Array := C.To_C (varName);
      begin -- Tcl_UntraceVar
         Tcl_UntraceVar (
            interp,
            C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
            flags,
            proc,
            data);
      end Tcl_UntraceVar;

      procedure Tcl_UntraceVar2 (
         interp          : in Tcl_Interp;
         part1           : in String;
         part2           : in String;
         flags           : in C.Int;
         proc            : in Tcl_VarTraceProc;
         data            : in ClientData
      ) is
         C_part1 : aliased C.Char_Array := C.To_C (part1);
         C_part2 : aliased C.Char_Array := C.To_C (part2);
      begin -- Tcl_UntraceVar2
         Tcl_UntraceVar2 (
            interp,
            C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
            C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
            flags,
            proc,
            data);
      end Tcl_UntraceVar2;

      function Tcl_VarTraceInfo (
         interp          : in Tcl_Interp;
         varName         : in String;
         flags           : in C.Int;
         procPtr         : in Tcl_VarTraceProc;
         prevclientdata  : in ClientData
      ) return ClientData is
         C_varName : aliased C.Char_Array := C.To_C (varName);
      begin -- Tcl_VarTraceInfo
         return Tcl_VarTraceInfo (
            interp,
            C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
            flags,
            procPtr,
            prevclientdata);
      end Tcl_VarTraceInfo;

      function Tcl_VarTraceInfo2 (
         interp          : in Tcl_Interp;
         part1           : in String;
         part2           : in String;
         flags           : in C.Int;
         procPtr         : in Tcl_VarTraceProc;
         prevclientdata  : in ClientData
      ) return ClientData is
         C_part1 : aliased C.Char_Array := C.To_C (part1);
         C_part2 : aliased C.Char_Array := C.To_C (part2);
      begin -- Tcl_VarTraceInfo2
         return Tcl_VarTraceInfo2 (
            interp,
            C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
            C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
            flags,
            procPtr,
            prevclientdata);
      end Tcl_VarTraceInfo2;

   end Generic_TraceVar;

   function Tcl_NewStringObj (
      bytes           : in String;
      length          : in C.Int
   ) return Tcl_Obj is
      C_bytes : aliased C.Char_Array := C.To_C (bytes);
   begin -- Tcl_NewStringObj
      return Tcl.Tcl_NewStringObj (
         C.Strings.To_Chars_Ptr (C_bytes'unchecked_access),
         length);
   end Tcl_NewStringObj;

   function Tcl_DStringValue (
      dsPtr           : in Tcl_DString
   ) return String is
   begin -- Tcl_DStringValue
      return Tash_CHelper.Value (Tcl.Tcl_DStringValue (
         dsPtr));
   end Tcl_DStringValue;

   procedure Tcl_AddErrorInfo (
      interp          : in Tcl_Interp;
      message         : in String
   ) is
      C_message : aliased C.Char_Array := C.To_C (message);
   begin -- Tcl_AddErrorInfo
      Tcl.Tcl_AddErrorInfo (
         interp,
         C.Strings.To_Chars_Ptr (C_message'unchecked_access));
   end Tcl_AddErrorInfo;

   procedure Tcl_AddObjErrorInfo (
      interp          : in Tcl_Interp;
      message         : in String;
      length          : in C.Int
   ) is
      C_message : aliased C.Char_Array := C.To_C (message);
   begin -- Tcl_AddObjErrorInfo
      Tcl.Tcl_AddObjErrorInfo (
         interp,
         C.Strings.To_Chars_Ptr (C_message'unchecked_access),
         length);
   end Tcl_AddObjErrorInfo;

   procedure Tcl_AppendElement (
      interp          : in Tcl_Interp;
      strng           : in String
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_AppendElement
      Tcl.Tcl_AppendElement (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access));
   end Tcl_AppendElement;

   procedure Tcl_AppendResult (
      interp          : in Tcl_Interp;
      String1         : in String := "";
      String2         : in String := "";
      String3         : in String := "";
      String4         : in String := "";
      String5         : in String := "";
      String6         : in String := "";
      String7         : in String := "";
      String8         : in String := "";
      String9         : in String := ""
   ) is
      C_String1 : aliased C.Char_Array := C.To_C (String1);
      C_String2 : aliased C.Char_Array := C.To_C (String2);
      C_String3 : aliased C.Char_Array := C.To_C (String3);
      C_String4 : aliased C.Char_Array := C.To_C (String4);
      C_String5 : aliased C.Char_Array := C.To_C (String5);
      C_String6 : aliased C.Char_Array := C.To_C (String6);
      C_String7 : aliased C.Char_Array := C.To_C (String7);
      C_String8 : aliased C.Char_Array := C.To_C (String8);
      C_String9 : aliased C.Char_Array := C.To_C (String9);
   begin -- Tcl_AppendResult
      Tcl.Tcl_AppendResult (
         interp,
         C.Strings.To_Chars_Ptr (C_String1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String3'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String4'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String5'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String6'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String7'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String8'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String9'unchecked_access));
   end Tcl_AppendResult;

   procedure Tcl_AppendToObj (
      objPtr          : in Tcl_Obj;
      bytes           : in String;
      length          : in C.Int
   ) is
      C_bytes : aliased C.Char_Array := C.To_C (bytes);
   begin -- Tcl_AppendToObj
      Tcl.Tcl_AppendToObj (
         objPtr,
         C.Strings.To_Chars_Ptr (C_bytes'unchecked_access),
         length);
   end Tcl_AppendToObj;

   procedure Tcl_AppendStringsToObj (
      interp          : in Tcl_Obj;
      String1         : in String := "";
      String2         : in String := "";
      String3         : in String := "";
      String4         : in String := "";
      String5         : in String := "";
      String6         : in String := "";
      String7         : in String := "";
      String8         : in String := "";
      String9         : in String := ""
   ) is
      C_String1 : aliased C.Char_Array := C.To_C (String1);
      C_String2 : aliased C.Char_Array := C.To_C (String2);
      C_String3 : aliased C.Char_Array := C.To_C (String3);
      C_String4 : aliased C.Char_Array := C.To_C (String4);
      C_String5 : aliased C.Char_Array := C.To_C (String5);
      C_String6 : aliased C.Char_Array := C.To_C (String6);
      C_String7 : aliased C.Char_Array := C.To_C (String7);
      C_String8 : aliased C.Char_Array := C.To_C (String8);
      C_String9 : aliased C.Char_Array := C.To_C (String9);
   begin -- Tcl_AppendStringsToObj
      Tcl.Tcl_AppendStringsToObj (
         interp,
         C.Strings.To_Chars_Ptr (C_String1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String3'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String4'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String5'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String6'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String7'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String8'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String9'unchecked_access));
   end Tcl_AppendStringsToObj;

   procedure Tcl_Backslash (
      src             : in String;
      readPtr         : access C.Int;
      result          : out C.Char
   ) is
      C_src : aliased C.Char_Array := C.To_C (src);
   begin -- Tcl_Backslash
      result := Tcl.Tcl_Backslash (
         C.Strings.To_Chars_Ptr (C_src'unchecked_access),
         readPtr);
   end Tcl_Backslash;

   function Tcl_CommandComplete (
      cmd             : in String
   ) return C.Int is
      C_cmd : aliased C.Char_Array := C.To_C (cmd);
   begin -- Tcl_CommandComplete
      return Tcl.Tcl_CommandComplete (
         C.Strings.To_Chars_Ptr (C_cmd'unchecked_access));
   end Tcl_CommandComplete;

   function Tcl_Concat (
      argc            : in C.Int;
      argv            : in Tash_CArgv.Chars_Ptr_Ptr
   ) return String is
   begin -- Tcl_Concat
      return Tash_CHelper.Value (Tcl.Tcl_Concat (
         argc,
         argv));
   end Tcl_Concat;

   function Tcl_ConvertCountedElement (
      src             : in String;
      length          : in C.Int;
      dst             : in String;
      flags           : in C.Int
   ) return C.Int is
      C_src : aliased C.Char_Array := C.To_C (src);
      C_dst : aliased C.Char_Array := C.To_C (dst);
   begin -- Tcl_ConvertCountedElement
      return Tcl.Tcl_ConvertCountedElement (
         C.Strings.To_Chars_Ptr (C_src'unchecked_access),
         length,
         C.Strings.To_Chars_Ptr (C_dst'unchecked_access),
         flags);
   end Tcl_ConvertCountedElement;

   function Tcl_ConvertElement (
      src             : in String;
      dst             : in String;
      flags           : in C.Int
   ) return C.Int is
      C_src : aliased C.Char_Array := C.To_C (src);
      C_dst : aliased C.Char_Array := C.To_C (dst);
   begin -- Tcl_ConvertElement
      return Tcl.Tcl_ConvertElement (
         C.Strings.To_Chars_Ptr (C_src'unchecked_access),
         C.Strings.To_Chars_Ptr (C_dst'unchecked_access),
         flags);
   end Tcl_ConvertElement;

   function Tcl_CreateAlias (
      slave           : in Tcl_Interp;
      slaveCmd        : in String;
      target          : in Tcl_Interp;
      targetCmd       : in String;
      argc            : in C.Int;
      argv            : in Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int is
      C_slaveCmd : aliased C.Char_Array := C.To_C (slaveCmd);
      C_targetCmd : aliased C.Char_Array := C.To_C (targetCmd);
   begin -- Tcl_CreateAlias
      return Tcl.Tcl_CreateAlias (
         slave,
         C.Strings.To_Chars_Ptr (C_slaveCmd'unchecked_access),
         target,
         C.Strings.To_Chars_Ptr (C_targetCmd'unchecked_access),
         argc,
         argv);
   end Tcl_CreateAlias;

   function Tcl_CreateAliasObj (
      slave           : in Tcl_Interp;
      slaveCmd        : in String;
      target          : in Tcl_Interp;
      targetCmd       : in String;
      objc            : in C.Int;
      CONST_objv      : in Tcl_Obj
   ) return C.Int is
      C_slaveCmd : aliased C.Char_Array := C.To_C (slaveCmd);
      C_targetCmd : aliased C.Char_Array := C.To_C (targetCmd);
   begin -- Tcl_CreateAliasObj
      return Tcl.Tcl_CreateAliasObj (
         slave,
         C.Strings.To_Chars_Ptr (C_slaveCmd'unchecked_access),
         target,
         C.Strings.To_Chars_Ptr (C_targetCmd'unchecked_access),
         objc,
         CONST_objv);
   end Tcl_CreateAliasObj;

   function Tcl_CreateSlave (
      interp          : in Tcl_Interp;
      slaveName       : in String;
      isSafe          : in C.Int
   ) return Tcl_Interp is
      C_slaveName : aliased C.Char_Array := C.To_C (slaveName);
   begin -- Tcl_CreateSlave
      return Tcl.Tcl_CreateSlave (
         interp,
         C.Strings.To_Chars_Ptr (C_slaveName'unchecked_access),
         isSafe);
   end Tcl_CreateSlave;

   function Tcl_DbCkalloc (
      size            : in C.Unsigned;
      file            : in String;
      line            : in C.Int
   ) return String is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbCkalloc
      return Tash_CHelper.Value (Tcl.Tcl_DbCkalloc (
         size,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line));
   end Tcl_DbCkalloc;

   function Tcl_DbCkfree (
      ptr             : in String;
      file            : in String;
      line            : in C.Int
   ) return C.Int is
      C_ptr : aliased C.Char_Array := C.To_C (ptr);
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbCkfree
      return Tcl.Tcl_DbCkfree (
         C.Strings.To_Chars_Ptr (C_ptr'unchecked_access),
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbCkfree;

   function Tcl_DbCkrealloc (
      ptr             : in String;
      size            : in C.Unsigned;
      file            : in String;
      line            : in C.Int
   ) return String is
      C_ptr : aliased C.Char_Array := C.To_C (ptr);
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbCkrealloc
      return Tash_CHelper.Value (Tcl.Tcl_DbCkrealloc (
         C.Strings.To_Chars_Ptr (C_ptr'unchecked_access),
         size,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line));
   end Tcl_DbCkrealloc;

   procedure Tcl_DbDecrRefCount (
      objPtr          : in Tcl_Obj;
      file            : in String;
      line            : in C.Int
   ) is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbDecrRefCount
      Tcl.Tcl_DbDecrRefCount (
         objPtr,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbDecrRefCount;

   procedure Tcl_DbIncrRefCount (
      objPtr          : in Tcl_Obj;
      file            : in String;
      line            : in C.Int
   ) is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbIncrRefCount
      Tcl.Tcl_DbIncrRefCount (
         objPtr,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbIncrRefCount;

   function Tcl_DbIsShared (
      objPtr          : in Tcl_Obj;
      file            : in String;
      line            : in C.Int
   ) return C.Int is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbIsShared
      return Tcl.Tcl_DbIsShared (
         objPtr,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbIsShared;

   function Tcl_DbNewBooleanObj (
      boolValue       : in C.Int;
      file            : in String;
      line            : in C.Int
   ) return Tcl_Obj is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbNewBooleanObj
      return Tcl.Tcl_DbNewBooleanObj (
         boolValue,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbNewBooleanObj;

   function Tcl_DbNewDoubleObj (
      doubleValue     : in C.Double;
      file            : in String;
      line            : in C.Int
   ) return Tcl_Obj is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbNewDoubleObj
      return Tcl.Tcl_DbNewDoubleObj (
         doubleValue,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbNewDoubleObj;

   function Tcl_DbNewListObj (
      objc            : in C.Int;
      CONST_objv      : in Tcl_Obj;
      file            : in String;
      line            : in C.Int
   ) return Tcl_Obj is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbNewListObj
      return Tcl.Tcl_DbNewListObj (
         objc,
         CONST_objv,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbNewListObj;

   function Tcl_DbNewLongObj (
      longValue       : in C.Long;
      file            : in String;
      line            : in C.Int
   ) return Tcl_Obj is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbNewLongObj
      return Tcl.Tcl_DbNewLongObj (
         longValue,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbNewLongObj;

   function Tcl_DbNewObj (
      file            : in String;
      line            : in C.Int
   ) return Tcl_Obj is
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbNewObj
      return Tcl.Tcl_DbNewObj (
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbNewObj;

   function Tcl_DbNewStringObj (
      bytes           : in String;
      length          : in C.Int;
      file            : in String;
      line            : in C.Int
   ) return Tcl_Obj is
      C_bytes : aliased C.Char_Array := C.To_C (bytes);
      C_file : aliased C.Char_Array := C.To_C (file);
   begin -- Tcl_DbNewStringObj
      return Tcl.Tcl_DbNewStringObj (
         C.Strings.To_Chars_Ptr (C_bytes'unchecked_access),
         length,
         C.Strings.To_Chars_Ptr (C_file'unchecked_access),
         line);
   end Tcl_DbNewStringObj;

   function Tcl_DStringAppend (
      dsPtr           : in Tcl_DString;
      strng           : in String;
      length          : in C.Int
   ) return String is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_DStringAppend
      return Tash_CHelper.Value (Tcl.Tcl_DStringAppend (
         dsPtr,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         length));
   end Tcl_DStringAppend;

   procedure Tcl_DStringAppend (
      dsPtr           : in Tcl_DString;
      strng           : in String;
      length          : in C.Int
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      result : C.Strings.Chars_Ptr;
   begin -- Tcl_DStringAppend
      result := Tcl.Tcl_DStringAppend (
         dsPtr,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         length);
   end Tcl_DStringAppend;

   function Tcl_DStringAppendElement (
      dsPtr           : in Tcl_DString;
      strng           : in String
   ) return String is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_DStringAppendElement
      return Tash_CHelper.Value (Tcl.Tcl_DStringAppendElement (
         dsPtr,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access)));
   end Tcl_DStringAppendElement;

   procedure Tcl_DStringAppendElement (
      dsPtr           : in Tcl_DString;
      strng           : in String
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      result : C.Strings.Chars_Ptr;
   begin -- Tcl_DStringAppendElement
      result := Tcl.Tcl_DStringAppendElement (
         dsPtr,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access));
   end Tcl_DStringAppendElement;

   function Tcl_ErrnoId return String is
   begin -- Tcl_ErrnoId
      return Tash_CHelper.Value (Tcl.Tcl_ErrnoId);
   end Tcl_ErrnoId;

   function Tcl_ErrnoMsg (
      err             : in C.Int
   ) return String is
   begin -- Tcl_ErrnoMsg
      return Tash_CHelper.Value (Tcl.Tcl_ErrnoMsg (
         err));
   end Tcl_ErrnoMsg;

   function Tcl_Eval (
      interp          : in Tcl_Interp;
      strng           : in String
   ) return C.Int is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_Eval
      return Tcl.Tcl_Eval (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access));
   end Tcl_Eval;

   procedure Tcl_Eval (
      interp          : in Tcl_Interp;
      strng           : in String
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_Eval
      Assert (interp, Tcl.Tcl_Eval (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access)));
   end Tcl_Eval;

   function Tcl_EvalFile (
      interp          : in Tcl_Interp;
      fileName        : in String
   ) return C.Int is
      C_fileName : aliased C.Char_Array := C.To_C (fileName);
   begin -- Tcl_EvalFile
      return Tcl.Tcl_EvalFile (
         interp,
         C.Strings.To_Chars_Ptr (C_fileName'unchecked_access));
   end Tcl_EvalFile;

   procedure Tcl_EvalFile (
      interp          : in Tcl_Interp;
      fileName        : in String
   ) is
      C_fileName : aliased C.Char_Array := C.To_C (fileName);
   begin -- Tcl_EvalFile
      Assert (interp, Tcl.Tcl_EvalFile (
         interp,
         C.Strings.To_Chars_Ptr (C_fileName'unchecked_access)));
   end Tcl_EvalFile;

   function Tcl_ExposeCommand (
      interp          : in Tcl_Interp;
      hiddenCmdToken  : in String;
      cmdName         : in String
   ) return C.Int is
      C_hiddenCmdToken : aliased C.Char_Array := C.To_C (hiddenCmdToken);
      C_cmdName : aliased C.Char_Array := C.To_C (cmdName);
   begin -- Tcl_ExposeCommand
      return Tcl.Tcl_ExposeCommand (
         interp,
         C.Strings.To_Chars_Ptr (C_hiddenCmdToken'unchecked_access),
         C.Strings.To_Chars_Ptr (C_cmdName'unchecked_access));
   end Tcl_ExposeCommand;

   procedure Tcl_ExposeCommand (
      interp          : in Tcl_Interp;
      hiddenCmdToken  : in String;
      cmdName         : in String
   ) is
      C_hiddenCmdToken : aliased C.Char_Array := C.To_C (hiddenCmdToken);
      C_cmdName : aliased C.Char_Array := C.To_C (cmdName);
   begin -- Tcl_ExposeCommand
      Assert (interp, Tcl.Tcl_ExposeCommand (
         interp,
         C.Strings.To_Chars_Ptr (C_hiddenCmdToken'unchecked_access),
         C.Strings.To_Chars_Ptr (C_cmdName'unchecked_access)));
   end Tcl_ExposeCommand;

   function Tcl_ExprBoolean (
      interp          : in Tcl_Interp;
      strng           : in String
   ) return Boolean is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      result : C.Int;
      ptr : aliased C.Int;
   begin -- Tcl_ExprBoolean
      result := Tcl.Tcl_ExprBoolean (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         ptr'access);
      return Boolean'val (ptr);
   end Tcl_ExprBoolean;

   function Tcl_ExprDouble (
      interp          : in Tcl_Interp;
      strng           : in String
   ) return C.Double is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      result : C.Int;
      ptr : aliased C.Double;
   begin -- Tcl_ExprDouble
      result := Tcl.Tcl_ExprDouble (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         ptr'access);
      return ptr;
   end Tcl_ExprDouble;

   function Tcl_ExprLong (
      interp          : in Tcl_Interp;
      strng           : in String
   ) return C.Long is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      result : C.Int;
      ptr : aliased C.Long;
   begin -- Tcl_ExprLong
      result := Tcl.Tcl_ExprLong (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         ptr'access);
      return ptr;
   end Tcl_ExprLong;

   function Tcl_ExprString (
      interp          : in Tcl_Interp;
      strng           : in String
   ) return String is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      result : C.Int;
   begin -- Tcl_ExprString
      result := Tcl.Tcl_ExprString (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access));
      return Tash_CHelper.Value (Tcl_GetResult (interp));
   end Tcl_ExprString;

   procedure Tcl_FindExecutable (
      argv0           : in String
   ) is
      C_argv0 : aliased C.Char_Array := C.To_C (argv0);
   begin -- Tcl_FindExecutable
      Tcl.Tcl_FindExecutable (
         C.Strings.To_Chars_Ptr (C_argv0'unchecked_access));
   end Tcl_FindExecutable;

   procedure Tcl_GetAlias (
      interp          : in Tcl_Interp;
      slaveCmd        : in String;
      targetInterpPtr : in Tcl_Interp;
      targetCmdPtr    : in Tash_CArgv.Chars_Ptr_Ptr;
      argcPtr         : access C.Int;
      argvPtr         : access Tash_CArgv.Chars_Ptr_Ptr;
      result          : out C.Int
   ) is
      C_slaveCmd : aliased C.Char_Array := C.To_C (slaveCmd);
   begin -- Tcl_GetAlias
      result := Tcl.Tcl_GetAlias (
         interp,
         C.Strings.To_Chars_Ptr (C_slaveCmd'unchecked_access),
         targetInterpPtr,
         targetCmdPtr,
         argcPtr,
         argvPtr);
   end Tcl_GetAlias;

   procedure Tcl_GetAliasObj (
      interp          : in Tcl_Interp;
      slaveCmd        : in String;
      targetInterpPtr : in Tcl_Interp;
      targetCmdPtr    : in Tash_CArgv.Chars_Ptr_Ptr;
      objcPtr         : access C.Int;
      objv            : in Tcl_Obj;
      result          : out C.Int
   ) is
      C_slaveCmd : aliased C.Char_Array := C.To_C (slaveCmd);
   begin -- Tcl_GetAliasObj
      result := Tcl.Tcl_GetAliasObj (
         interp,
         C.Strings.To_Chars_Ptr (C_slaveCmd'unchecked_access),
         targetInterpPtr,
         targetCmdPtr,
         objcPtr,
         objv);
   end Tcl_GetAliasObj;

   procedure Tcl_GetBoolean (
      interp          : in Tcl_Interp;
      strng           : in String;
      boolPtr         : access C.Int;
      result          : out C.Int
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_GetBoolean
      result := Tcl.Tcl_GetBoolean (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         boolPtr);
   end Tcl_GetBoolean;

   procedure Tcl_GetDouble (
      interp          : in Tcl_Interp;
      strng           : in String;
      doublePtr       : access C.Double;
      result          : out C.Int
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_GetDouble
      result := Tcl.Tcl_GetDouble (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         doublePtr);
   end Tcl_GetDouble;

   function Tcl_GetHostName return String is
   begin -- Tcl_GetHostName
      return Tash_CHelper.Value (Tcl.Tcl_GetHostName);
   end Tcl_GetHostName;

   procedure Tcl_GetIndexFromObj (
      interp          : in Tcl_Interp;
      objPtr          : in Tcl_Obj;
      tablePtr        : in Tash_CArgv.Chars_Ptr_Ptr;
      msg             : in String;
      flags           : in C.Int;
      indexPtr        : access C.Int;
      result          : out C.Int
   ) is
      C_msg : aliased C.Char_Array := C.To_C (msg);
   begin -- Tcl_GetIndexFromObj
      result := Tcl.Tcl_GetIndexFromObj (
         interp,
         objPtr,
         tablePtr,
         C.Strings.To_Chars_Ptr (C_msg'unchecked_access),
         flags,
         indexPtr);
   end Tcl_GetIndexFromObj;

   procedure Tcl_GetInt (
      interp          : in Tcl_Interp;
      strng           : in String;
      intPtr          : access C.Int;
      result          : out C.Int
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_GetInt
      result := Tcl.Tcl_GetInt (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         intPtr);
   end Tcl_GetInt;

   function Tcl_GetObjType (
      typeName        : in String
   ) return Tcl_ObjType is
      C_typeName : aliased C.Char_Array := C.To_C (typeName);
   begin -- Tcl_GetObjType
      return Tcl.Tcl_GetObjType (
         C.Strings.To_Chars_Ptr (C_typeName'unchecked_access));
   end Tcl_GetObjType;

   function Tcl_GetPathType (
      path            : in String
   ) return Tcl_PathType is
      C_path : aliased C.Char_Array := C.To_C (path);
   begin -- Tcl_GetPathType
      return Tcl.Tcl_GetPathType (
         C.Strings.To_Chars_Ptr (C_path'unchecked_access));
   end Tcl_GetPathType;

   function Tcl_GetSlave (
      interp          : in Tcl_Interp;
      slaveName       : in String
   ) return Tcl_Interp is
      C_slaveName : aliased C.Char_Array := C.To_C (slaveName);
   begin -- Tcl_GetSlave
      return Tcl.Tcl_GetSlave (
         interp,
         C.Strings.To_Chars_Ptr (C_slaveName'unchecked_access));
   end Tcl_GetSlave;

   procedure Tcl_GetStringFromObj (
      objPtr          : in Tcl_Obj;
      lengthPtr       : access C.Int;
      result          : out String
   ) is
   begin -- Tcl_GetStringFromObj
      result := Tash_CHelper.Value (Tcl.Tcl_GetStringFromObj (
         objPtr,
         lengthPtr));
   end Tcl_GetStringFromObj;

   function Tcl_GetStringResult (
      interp          : in Tcl_Interp
   ) return String is
   begin -- Tcl_GetStringResult
      return Tash_CHelper.Value (Tcl.Tcl_GetStringResult (
         interp));
   end Tcl_GetStringResult;

   function Tcl_GetVar (
      interp          : in Tcl_Interp;
      varName         : in String;
      flags           : in C.Int
   ) return String is
      C_varName : aliased C.Char_Array := C.To_C (varName);
   begin -- Tcl_GetVar
      return Tash_CHelper.Value (Tcl.Tcl_GetVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         flags));
   end Tcl_GetVar;

   function Tcl_GetVar2 (
      interp          : in Tcl_Interp;
      part1           : in String;
      part2           : in String;
      flags           : in C.Int
   ) return String is
      C_part1 : aliased C.Char_Array := C.To_C (part1);
      C_part2 : aliased C.Char_Array := C.To_C (part2);
   begin -- Tcl_GetVar2
      return Tash_CHelper.Value (Tcl.Tcl_GetVar2 (
         interp,
         C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
         flags));
   end Tcl_GetVar2;

   function Tcl_GlobalEval (
      interp          : in Tcl_Interp;
      command         : in String
   ) return C.Int is
      C_command : aliased C.Char_Array := C.To_C (command);
   begin -- Tcl_GlobalEval
      return Tcl.Tcl_GlobalEval (
         interp,
         C.Strings.To_Chars_Ptr (C_command'unchecked_access));
   end Tcl_GlobalEval;

   procedure Tcl_GlobalEval (
      interp          : in Tcl_Interp;
      command         : in String
   ) is
      C_command : aliased C.Char_Array := C.To_C (command);
   begin -- Tcl_GlobalEval
      Assert (interp, Tcl.Tcl_GlobalEval (
         interp,
         C.Strings.To_Chars_Ptr (C_command'unchecked_access)));
   end Tcl_GlobalEval;

   function Tcl_HideCommand (
      interp          : in Tcl_Interp;
      cmdName         : in String;
      hiddenCmdToken  : in String
   ) return C.Int is
      C_cmdName : aliased C.Char_Array := C.To_C (cmdName);
      C_hiddenCmdToken : aliased C.Char_Array := C.To_C (hiddenCmdToken);
   begin -- Tcl_HideCommand
      return Tcl.Tcl_HideCommand (
         interp,
         C.Strings.To_Chars_Ptr (C_cmdName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_hiddenCmdToken'unchecked_access));
   end Tcl_HideCommand;

   procedure Tcl_HideCommand (
      interp          : in Tcl_Interp;
      cmdName         : in String;
      hiddenCmdToken  : in String
   ) is
      C_cmdName : aliased C.Char_Array := C.To_C (cmdName);
      C_hiddenCmdToken : aliased C.Char_Array := C.To_C (hiddenCmdToken);
   begin -- Tcl_HideCommand
      Assert (interp, Tcl.Tcl_HideCommand (
         interp,
         C.Strings.To_Chars_Ptr (C_cmdName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_hiddenCmdToken'unchecked_access)));
   end Tcl_HideCommand;

   function Tcl_JoinPath (
      argc            : in C.Int;
      argv            : in Tash_CArgv.Chars_Ptr_Ptr;
      resultPtr       : in Tcl_DString
   ) return String is
   begin -- Tcl_JoinPath
      return Tash_CHelper.Value (Tcl.Tcl_JoinPath (
         argc,
         argv,
         resultPtr));
   end Tcl_JoinPath;

   function Tcl_LinkVar (
      interp          : in Tcl_Interp;
      varName         : in String;
      addr            : in String;
      typ             : in C.Int
   ) return C.Int is
      C_varName : aliased C.Char_Array := C.To_C (varName);
      C_addr : aliased C.Char_Array := C.To_C (addr);
   begin -- Tcl_LinkVar
      return Tcl.Tcl_LinkVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_addr'unchecked_access),
         typ);
   end Tcl_LinkVar;

   procedure Tcl_LinkVar (
      interp          : in Tcl_Interp;
      varName         : in String;
      addr            : in String;
      typ             : in C.Int
   ) is
      C_varName : aliased C.Char_Array := C.To_C (varName);
      C_addr : aliased C.Char_Array := C.To_C (addr);
   begin -- Tcl_LinkVar
      Assert (interp, Tcl.Tcl_LinkVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_addr'unchecked_access),
         typ));
   end Tcl_LinkVar;

   function Tcl_Merge (
      argc            : in C.Int;
      argv            : in Tash_CArgv.Chars_Ptr_Ptr
   ) return String is
   begin -- Tcl_Merge
      return Tash_CHelper.Value (Tcl.Tcl_Merge (
         argc,
         argv));
   end Tcl_Merge;

   function Tcl_ParseVar (
      interp          : in Tcl_Interp;
      strng           : in String;
      termPtr         : in Tash_CArgv.Chars_Ptr_Ptr
   ) return String is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_ParseVar
      return Tash_CHelper.Value (Tcl.Tcl_ParseVar (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         termPtr));
   end Tcl_ParseVar;

   function Tcl_PkgProvide (
      interp          : in Tcl_Interp;
      name            : in String;
      version         : in String
   ) return C.Int is
      C_name : aliased C.Char_Array := C.To_C (name);
      C_version : aliased C.Char_Array := C.To_C (version);
   begin -- Tcl_PkgProvide
      return Tcl.Tcl_PkgProvide (
         interp,
         C.Strings.To_Chars_Ptr (C_name'unchecked_access),
         C.Strings.To_Chars_Ptr (C_version'unchecked_access));
   end Tcl_PkgProvide;

   procedure Tcl_PkgProvide (
      interp          : in Tcl_Interp;
      name            : in String;
      version         : in String
   ) is
      C_name : aliased C.Char_Array := C.To_C (name);
      C_version : aliased C.Char_Array := C.To_C (version);
   begin -- Tcl_PkgProvide
      Assert (interp, Tcl.Tcl_PkgProvide (
         interp,
         C.Strings.To_Chars_Ptr (C_name'unchecked_access),
         C.Strings.To_Chars_Ptr (C_version'unchecked_access)));
   end Tcl_PkgProvide;

   function Tcl_PkgRequire (
      interp          : in Tcl_Interp;
      name            : in String;
      version         : in String;
      exact           : in C.Int
   ) return String is
      C_name : aliased C.Char_Array := C.To_C (name);
      C_version : aliased C.Char_Array := C.To_C (version);
   begin -- Tcl_PkgRequire
      return Tash_CHelper.Value (Tcl.Tcl_PkgRequire (
         interp,
         C.Strings.To_Chars_Ptr (C_name'unchecked_access),
         C.Strings.To_Chars_Ptr (C_version'unchecked_access),
         exact));
   end Tcl_PkgRequire;

   function Tcl_PosixError (
      interp          : in Tcl_Interp
   ) return String is
   begin -- Tcl_PosixError
      return Tash_CHelper.Value (Tcl.Tcl_PosixError (
         interp));
   end Tcl_PosixError;

   procedure Tcl_PrintDouble (
      interp          : in Tcl_Interp;
      value           : in C.Double;
      dst             : in String
   ) is
      C_dst : aliased C.Char_Array := C.To_C (dst);
   begin -- Tcl_PrintDouble
      Tcl.Tcl_PrintDouble (
         interp,
         value,
         C.Strings.To_Chars_Ptr (C_dst'unchecked_access));
   end Tcl_PrintDouble;

   function Tcl_PutEnv (
      strng           : in String
   ) return C.Int is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_PutEnv
      return Tcl.Tcl_PutEnv (
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access));
   end Tcl_PutEnv;

   function Tcl_RecordAndEval (
      interp          : in Tcl_Interp;
      cmd             : in String;
      flags           : in C.Int
   ) return C.Int is
      C_cmd : aliased C.Char_Array := C.To_C (cmd);
   begin -- Tcl_RecordAndEval
      return Tcl.Tcl_RecordAndEval (
         interp,
         C.Strings.To_Chars_Ptr (C_cmd'unchecked_access),
         flags);
   end Tcl_RecordAndEval;

   procedure Tcl_RecordAndEval (
      interp          : in Tcl_Interp;
      cmd             : in String;
      flags           : in C.Int
   ) is
      C_cmd : aliased C.Char_Array := C.To_C (cmd);
   begin -- Tcl_RecordAndEval
      Assert (interp, Tcl.Tcl_RecordAndEval (
         interp,
         C.Strings.To_Chars_Ptr (C_cmd'unchecked_access),
         flags));
   end Tcl_RecordAndEval;

   function Tcl_RegExpCompile (
      interp          : in Tcl_Interp;
      strng           : in String
   ) return Tcl_RegExp is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_RegExpCompile
      return Tcl.Tcl_RegExpCompile (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access));
   end Tcl_RegExpCompile;

   function Tcl_RegExpExec (
      interp          : in Tcl_Interp;
      regexp          : in Tcl_RegExp;
      strng           : in String;
      start           : in String
   ) return C.Int is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      C_start : aliased C.Char_Array := C.To_C (start);
   begin -- Tcl_RegExpExec
      return Tcl.Tcl_RegExpExec (
         interp,
         regexp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         C.Strings.To_Chars_Ptr (C_start'unchecked_access));
   end Tcl_RegExpExec;

   procedure Tcl_RegExpExec (
      interp          : in Tcl_Interp;
      regexp          : in Tcl_RegExp;
      strng           : in String;
      start           : in String
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      C_start : aliased C.Char_Array := C.To_C (start);
   begin -- Tcl_RegExpExec
      Assert (interp, Tcl.Tcl_RegExpExec (
         interp,
         regexp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         C.Strings.To_Chars_Ptr (C_start'unchecked_access)));
   end Tcl_RegExpExec;

   function Tcl_RegExpMatch (
      interp          : in Tcl_Interp;
      strng           : in String;
      pattern         : in String
   ) return C.Int is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      C_pattern : aliased C.Char_Array := C.To_C (pattern);
   begin -- Tcl_RegExpMatch
      return Tcl.Tcl_RegExpMatch (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         C.Strings.To_Chars_Ptr (C_pattern'unchecked_access));
   end Tcl_RegExpMatch;

   procedure Tcl_RegExpMatch (
      interp          : in Tcl_Interp;
      strng           : in String;
      pattern         : in String
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      C_pattern : aliased C.Char_Array := C.To_C (pattern);
   begin -- Tcl_RegExpMatch
      Assert (interp, Tcl.Tcl_RegExpMatch (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         C.Strings.To_Chars_Ptr (C_pattern'unchecked_access)));
   end Tcl_RegExpMatch;

   procedure Tcl_ScanCountedElement (
      strng           : in String;
      length          : in C.Int;
      flagPtr         : access C.Int;
      result          : out C.Int
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_ScanCountedElement
      result := Tcl.Tcl_ScanCountedElement (
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         length,
         flagPtr);
   end Tcl_ScanCountedElement;

   procedure Tcl_ScanElement (
      strng           : in String;
      flagPtr         : access C.Int;
      result          : out C.Int
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_ScanElement
      result := Tcl.Tcl_ScanElement (
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         flagPtr);
   end Tcl_ScanElement;

   procedure Tcl_SetErrorCode (
      arg1            : in Tcl_Interp;
      String1         : in String := "";
      String2         : in String := "";
      String3         : in String := "";
      String4         : in String := "";
      String5         : in String := "";
      String6         : in String := "";
      String7         : in String := "";
      String8         : in String := "";
      String9         : in String := ""
   ) is
      C_String1 : aliased C.Char_Array := C.To_C (String1);
      C_String2 : aliased C.Char_Array := C.To_C (String2);
      C_String3 : aliased C.Char_Array := C.To_C (String3);
      C_String4 : aliased C.Char_Array := C.To_C (String4);
      C_String5 : aliased C.Char_Array := C.To_C (String5);
      C_String6 : aliased C.Char_Array := C.To_C (String6);
      C_String7 : aliased C.Char_Array := C.To_C (String7);
      C_String8 : aliased C.Char_Array := C.To_C (String8);
      C_String9 : aliased C.Char_Array := C.To_C (String9);
   begin -- Tcl_SetErrorCode
      Tcl.Tcl_SetErrorCode (
         arg1,
         C.Strings.To_Chars_Ptr (C_String1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String3'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String4'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String5'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String6'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String7'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String8'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String9'unchecked_access));
   end Tcl_SetErrorCode;

   procedure Tcl_SetResult (
      interp          : in Tcl_Interp;
      strng           : in String
   ) is
      C_strng : aliased C.Char_Array := C.To_C (strng);
   begin -- Tcl_SetResult
      Tcl.Tcl_SetResult (
         interp,
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         C.Int (TCL_VOLATILE));
   end Tcl_SetResult;

   procedure Tcl_SetStringObj (
      objPtr          : in Tcl_Obj;
      bytes           : in String;
      length          : in C.Int
   ) is
      C_bytes : aliased C.Char_Array := C.To_C (bytes);
   begin -- Tcl_SetStringObj
      Tcl.Tcl_SetStringObj (
         objPtr,
         C.Strings.To_Chars_Ptr (C_bytes'unchecked_access),
         length);
   end Tcl_SetStringObj;

   function Tcl_SetVar (
      interp          : in Tcl_Interp;
      varName         : in String;
      newValue        : in String;
      flags           : in C.Int
   ) return String is
      C_varName : aliased C.Char_Array := C.To_C (varName);
      C_newValue : aliased C.Char_Array := C.To_C (newValue);
   begin -- Tcl_SetVar
      return Tash_CHelper.Value (Tcl.Tcl_SetVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_newValue'unchecked_access),
         flags));
   end Tcl_SetVar;

   procedure Tcl_SetVar (
      interp          : in Tcl_Interp;
      varName         : in String;
      newValue        : in String;
      flags           : in C.Int
   ) is
      C_varName : aliased C.Char_Array := C.To_C (varName);
      C_newValue : aliased C.Char_Array := C.To_C (newValue);
      result : C.Strings.Chars_Ptr;
   begin -- Tcl_SetVar
      result := Tcl.Tcl_SetVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_newValue'unchecked_access),
         flags);
   end Tcl_SetVar;

   function Tcl_SetVar2 (
      interp          : in Tcl_Interp;
      part1           : in String;
      part2           : in String;
      newValue        : in String;
      flags           : in C.Int
   ) return String is
      C_part1 : aliased C.Char_Array := C.To_C (part1);
      C_part2 : aliased C.Char_Array := C.To_C (part2);
      C_newValue : aliased C.Char_Array := C.To_C (newValue);
   begin -- Tcl_SetVar2
      return Tash_CHelper.Value (Tcl.Tcl_SetVar2 (
         interp,
         C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_newValue'unchecked_access),
         flags));
   end Tcl_SetVar2;

   procedure Tcl_SetVar2 (
      interp          : in Tcl_Interp;
      part1           : in String;
      part2           : in String;
      newValue        : in String;
      flags           : in C.Int
   ) is
      C_part1 : aliased C.Char_Array := C.To_C (part1);
      C_part2 : aliased C.Char_Array := C.To_C (part2);
      C_newValue : aliased C.Char_Array := C.To_C (newValue);
      result : C.Strings.Chars_Ptr;
   begin -- Tcl_SetVar2
      result := Tcl.Tcl_SetVar2 (
         interp,
         C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_newValue'unchecked_access),
         flags);
   end Tcl_SetVar2;

   function Tcl_SignalId (
      sig             : in C.Int
   ) return String is
   begin -- Tcl_SignalId
      return Tash_CHelper.Value (Tcl.Tcl_SignalId (
         sig));
   end Tcl_SignalId;

   function Tcl_SignalMsg (
      sig             : in C.Int
   ) return String is
   begin -- Tcl_SignalMsg
      return Tash_CHelper.Value (Tcl.Tcl_SignalMsg (
         sig));
   end Tcl_SignalMsg;

   procedure Tcl_SplitList (
      interp          : in Tcl_Interp;
      list            : in String;
      argcPtr         : access C.Int;
      argvPtr         : access Tash_CArgv.Chars_Ptr_Ptr;
      result          : out C.Int
   ) is
      C_list : aliased C.Char_Array := C.To_C (list);
   begin -- Tcl_SplitList
      result := Tcl.Tcl_SplitList (
         interp,
         C.Strings.To_Chars_Ptr (C_list'unchecked_access),
         argcPtr,
         argvPtr);
   end Tcl_SplitList;

   procedure Tcl_SplitPath (
      path            : in String;
      argcPtr         : access C.Int;
      argvPtr         : access Tash_CArgv.Chars_Ptr_Ptr
   ) is
      C_path : aliased C.Char_Array := C.To_C (path);
   begin -- Tcl_SplitPath
      Tcl.Tcl_SplitPath (
         C.Strings.To_Chars_Ptr (C_path'unchecked_access),
         argcPtr,
         argvPtr);
   end Tcl_SplitPath;

   procedure Tcl_StaticPackage (
      interp          : in Tcl_Interp;
      pkgName         : in String;
      initProc        : in Tcl_PackageInitProc;
      safeInitProc    : in Tcl_PackageInitProc
   ) is
      C_pkgName : aliased C.Char_Array := C.To_C (pkgName);
   begin -- Tcl_StaticPackage
      Tcl.Tcl_StaticPackage (
         interp,
         C.Strings.To_Chars_Ptr (C_pkgName'unchecked_access),
         initProc,
         safeInitProc);
   end Tcl_StaticPackage;

   function Tcl_StringMatch (
      strng           : in String;
      pattern         : in String
   ) return C.Int is
      C_strng : aliased C.Char_Array := C.To_C (strng);
      C_pattern : aliased C.Char_Array := C.To_C (pattern);
   begin -- Tcl_StringMatch
      return Tcl.Tcl_StringMatch (
         C.Strings.To_Chars_Ptr (C_strng'unchecked_access),
         C.Strings.To_Chars_Ptr (C_pattern'unchecked_access));
   end Tcl_StringMatch;

   function Tcl_Ungets (
      chan            : in Tcl_Channel;
      str             : in String;
      len             : in C.Int;
      atHead          : in C.Int
   ) return C.Int is
      C_str : aliased C.Char_Array := C.To_C (str);
   begin -- Tcl_Ungets
      return Tcl.Tcl_Ungets (
         chan,
         C.Strings.To_Chars_Ptr (C_str'unchecked_access),
         len,
         atHead);
   end Tcl_Ungets;

   procedure Tcl_UnlinkVar (
      interp          : in Tcl_Interp;
      varName         : in String
   ) is
      C_varName : aliased C.Char_Array := C.To_C (varName);
   begin -- Tcl_UnlinkVar
      Tcl.Tcl_UnlinkVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access));
   end Tcl_UnlinkVar;

   function Tcl_UnsetVar (
      interp          : in Tcl_Interp;
      varName         : in String;
      flags           : in C.Int
   ) return C.Int is
      C_varName : aliased C.Char_Array := C.To_C (varName);
   begin -- Tcl_UnsetVar
      return Tcl.Tcl_UnsetVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         flags);
   end Tcl_UnsetVar;

   procedure Tcl_UnsetVar (
      interp          : in Tcl_Interp;
      varName         : in String;
      flags           : in C.Int
   ) is
      C_varName : aliased C.Char_Array := C.To_C (varName);
   begin -- Tcl_UnsetVar
      Assert (interp, Tcl.Tcl_UnsetVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         flags));
   end Tcl_UnsetVar;

   function Tcl_UnsetVar2 (
      interp          : in Tcl_Interp;
      part1           : in String;
      part2           : in String;
      flags           : in C.Int
   ) return C.Int is
      C_part1 : aliased C.Char_Array := C.To_C (part1);
      C_part2 : aliased C.Char_Array := C.To_C (part2);
   begin -- Tcl_UnsetVar2
      return Tcl.Tcl_UnsetVar2 (
         interp,
         C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
         flags);
   end Tcl_UnsetVar2;

   procedure Tcl_UnsetVar2 (
      interp          : in Tcl_Interp;
      part1           : in String;
      part2           : in String;
      flags           : in C.Int
   ) is
      C_part1 : aliased C.Char_Array := C.To_C (part1);
      C_part2 : aliased C.Char_Array := C.To_C (part2);
   begin -- Tcl_UnsetVar2
      Assert (interp, Tcl.Tcl_UnsetVar2 (
         interp,
         C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
         flags));
   end Tcl_UnsetVar2;

   procedure Tcl_UpdateLinkedVar (
      interp          : in Tcl_Interp;
      varName         : in String
   ) is
      C_varName : aliased C.Char_Array := C.To_C (varName);
   begin -- Tcl_UpdateLinkedVar
      Tcl.Tcl_UpdateLinkedVar (
         interp,
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access));
   end Tcl_UpdateLinkedVar;

   function Tcl_UpVar (
      interp          : in Tcl_Interp;
      frameName       : in String;
      varName         : in String;
      localName       : in String;
      flags           : in C.Int
   ) return C.Int is
      C_frameName : aliased C.Char_Array := C.To_C (frameName);
      C_varName : aliased C.Char_Array := C.To_C (varName);
      C_localName : aliased C.Char_Array := C.To_C (localName);
   begin -- Tcl_UpVar
      return Tcl.Tcl_UpVar (
         interp,
         C.Strings.To_Chars_Ptr (C_frameName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_localName'unchecked_access),
         flags);
   end Tcl_UpVar;

   procedure Tcl_UpVar (
      interp          : in Tcl_Interp;
      frameName       : in String;
      varName         : in String;
      localName       : in String;
      flags           : in C.Int
   ) is
      C_frameName : aliased C.Char_Array := C.To_C (frameName);
      C_varName : aliased C.Char_Array := C.To_C (varName);
      C_localName : aliased C.Char_Array := C.To_C (localName);
   begin -- Tcl_UpVar
      Assert (interp, Tcl.Tcl_UpVar (
         interp,
         C.Strings.To_Chars_Ptr (C_frameName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_varName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_localName'unchecked_access),
         flags));
   end Tcl_UpVar;

   function Tcl_UpVar2 (
      interp          : in Tcl_Interp;
      frameName       : in String;
      part1           : in String;
      part2           : in String;
      localName       : in String;
      flags           : in C.Int
   ) return C.Int is
      C_frameName : aliased C.Char_Array := C.To_C (frameName);
      C_part1 : aliased C.Char_Array := C.To_C (part1);
      C_part2 : aliased C.Char_Array := C.To_C (part2);
      C_localName : aliased C.Char_Array := C.To_C (localName);
   begin -- Tcl_UpVar2
      return Tcl.Tcl_UpVar2 (
         interp,
         C.Strings.To_Chars_Ptr (C_frameName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_localName'unchecked_access),
         flags);
   end Tcl_UpVar2;

   procedure Tcl_UpVar2 (
      interp          : in Tcl_Interp;
      frameName       : in String;
      part1           : in String;
      part2           : in String;
      localName       : in String;
      flags           : in C.Int
   ) is
      C_frameName : aliased C.Char_Array := C.To_C (frameName);
      C_part1 : aliased C.Char_Array := C.To_C (part1);
      C_part2 : aliased C.Char_Array := C.To_C (part2);
      C_localName : aliased C.Char_Array := C.To_C (localName);
   begin -- Tcl_UpVar2
      Assert (interp, Tcl.Tcl_UpVar2 (
         interp,
         C.Strings.To_Chars_Ptr (C_frameName'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_part2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_localName'unchecked_access),
         flags));
   end Tcl_UpVar2;

   function Tcl_VarEval (
      interp          : in Tcl_Interp;
      String1         : in String := "";
      String2         : in String := "";
      String3         : in String := "";
      String4         : in String := "";
      String5         : in String := "";
      String6         : in String := "";
      String7         : in String := "";
      String8         : in String := "";
      String9         : in String := ""
   ) return C.Int is
      C_String1 : aliased C.Char_Array := C.To_C (String1);
      C_String2 : aliased C.Char_Array := C.To_C (String2);
      C_String3 : aliased C.Char_Array := C.To_C (String3);
      C_String4 : aliased C.Char_Array := C.To_C (String4);
      C_String5 : aliased C.Char_Array := C.To_C (String5);
      C_String6 : aliased C.Char_Array := C.To_C (String6);
      C_String7 : aliased C.Char_Array := C.To_C (String7);
      C_String8 : aliased C.Char_Array := C.To_C (String8);
      C_String9 : aliased C.Char_Array := C.To_C (String9);
   begin -- Tcl_VarEval
      return Tcl.Tcl_VarEval (
         interp,
         C.Strings.To_Chars_Ptr (C_String1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String3'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String4'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String5'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String6'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String7'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String8'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String9'unchecked_access));
   end Tcl_VarEval;

   procedure Tcl_VarEval (
      interp          : in Tcl_Interp;
      String1         : in String := "";
      String2         : in String := "";
      String3         : in String := "";
      String4         : in String := "";
      String5         : in String := "";
      String6         : in String := "";
      String7         : in String := "";
      String8         : in String := "";
      String9         : in String := ""
   ) is
      C_String1 : aliased C.Char_Array := C.To_C (String1);
      C_String2 : aliased C.Char_Array := C.To_C (String2);
      C_String3 : aliased C.Char_Array := C.To_C (String3);
      C_String4 : aliased C.Char_Array := C.To_C (String4);
      C_String5 : aliased C.Char_Array := C.To_C (String5);
      C_String6 : aliased C.Char_Array := C.To_C (String6);
      C_String7 : aliased C.Char_Array := C.To_C (String7);
      C_String8 : aliased C.Char_Array := C.To_C (String8);
      C_String9 : aliased C.Char_Array := C.To_C (String9);
   begin -- Tcl_VarEval
      Assert (interp, Tcl.Tcl_VarEval (
         interp,
         C.Strings.To_Chars_Ptr (C_String1'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String2'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String3'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String4'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String5'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String6'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String7'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String8'unchecked_access),
         C.Strings.To_Chars_Ptr (C_String9'unchecked_access)));
   end Tcl_VarEval;

   procedure Tcl_WrongNumArgs (
      interp          : in Tcl_Interp;
      objc            : in C.Int;
      CONST_objv      : in Tcl_Obj;
      message         : in String
   ) is
      C_message : aliased C.Char_Array := C.To_C (message);
   begin -- Tcl_WrongNumArgs
      Tcl.Tcl_WrongNumArgs (
         interp,
         objc,
         CONST_objv,
         C.Strings.To_Chars_Ptr (C_message'unchecked_access));
   end Tcl_WrongNumArgs;

   function Tcl_GetResult (
      interp          : in Tcl_Interp
   ) return String is
   begin -- Tcl_GetResult
      return Tash_CHelper.Value (Tcl.Tcl_GetResult (
         interp));
   end Tcl_GetResult;


end Tcl.Ada;
