
--------------------------------------------------------------------
--
-- tcl-tk.ads -- This package is the "thin" binding to Tcl.Tk.
--
-- Copyright (c) 1995-1998 Terry J. Westley
--
-- See the file "license.htm" for information on usage and
-- redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
--
-- This package is automatically generated from tk.h.  Note that some
-- of the comments below, preserved from tk.h, do not apply to the
-- Ada version.  Someday, these comments may be customized better.
--
--------------------------------------------------------------------

with Tash_CArgv;
with Tash_CHelper;
with Interfaces.C.Strings;
with System;
with Tcl;

package Tcl.Tk is

   pragma Linker_Options ("-ltk");
   pragma Linker_Options ("-L/usr/X11R6/lib");
   pragma Linker_Options ("-lX11");

   package C renames Interfaces.C;

   use type C.Size_t;
   --
   --  tk.h --
   --
   --   Declarations for Tk-related things that are visible
   --   outside of the Tk module itself.
   --
   --  Copyright {c} 1989-1994 The Regents of the University of California.
   --  Copyright {c} 1994 The Australian National University.
   --  Copyright {c} 1994-1997 Sun Microsystems, Inc.
   --
   --  See the file "license.terms" for information on usage and redistribution
   --  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
   --
   --  SCCS: @{#} tk.h 1.211 97/11/20 12:44:45
   --

   --
   --  When version numbers change here, you must also go into the following files
   --  and update the version numbers:
   --
   --  unix/configure.in
   --  win/makefile.bc
   --  win/makefile.vc
   --  library/tk.tcl
   --
   --  The release level should be  0 for alpha, 1 for beta, and 2 for
   --  final/patch.  The release serial value is the number that follows the
   --  "a", "b", or "p" in the patch level; for example, if the patch level
   --  is 4.3b2, TK_RELEASE_SERIAL is 2.  It restarts at 1 whenever the
   --  release level is changed, except for the final release, which should
   --  be 0.
   --
   --  You may also need to update some of these files when the numbers change
   --  for the version of Tcl that this release of Tk is compiled against.
   --

   TK_MAJOR_VERSION               : constant := 8;
   TK_MINOR_VERSION               : constant := 0;
   TK_RELEASE_LEVEL               : constant := 2;
   TK_RELEASE_SERIAL              : constant := 2;
   TK_VERSION                     : constant String := "8.0";
   TK_PATCH_LEVEL                 : constant String := "8.0p2";
   --
   --  A special definition used to allow this header file to be included
   --  in resource files.
   --

   --
   --  The following definitions set up the proper options for Macintosh
   --  compilers.  We use this method because there is no autoconf equivalent.
   --

   --
   --  Decide whether or not to use input methods.
   --

   --
   --  Dummy types that are used by clients:
   --

   type Tk_BindingTable is private;

   function Is_Null (Ptr : Tk_BindingTable) return Boolean;

   type Tk_Canvas is private;

   function Is_Null (Ptr : Tk_Canvas) return Boolean;

   type Tk_Cursor is private;

   function Is_Null (Ptr : Tk_Cursor) return Boolean;

   type Tk_ErrorHandler is private;

   function Is_Null (Ptr : Tk_ErrorHandler) return Boolean;

   type Tk_Font is private;

   function Is_Null (Ptr : Tk_Font) return Boolean;

   type Tk_Image is private;

   function Is_Null (Ptr : Tk_Image) return Boolean;

   type Tk_ImageMaster is private;

   function Is_Null (Ptr : Tk_ImageMaster) return Boolean;

   type Tk_TextLayout is private;

   function Is_Null (Ptr : Tk_TextLayout) return Boolean;

   type Tk_Window is private;

   function Is_Null (Ptr : Tk_Window) return Boolean;

   type Tk_3DBorder is private;

   function Is_Null (Ptr : Tk_3DBorder) return Boolean;

   --
   --  Additional types exported to clients.
   --

   subtype Tk_Uid is C.Strings.Chars_Ptr;

   --
   --  Structure used to specify how to handle argv options.
   --

   type Tk_ArgvInfo_rec is private;
   type Tk_ArgvInfo is access all Tk_ArgvInfo_rec;
   pragma Convention (C, Tk_ArgvInfo);

   type Tk_ArgvInfo_array is array (C.size_t range <>) of Tk_ArgvInfo;

   function Is_Null (Ptr : Tk_ArgvInfo) return Boolean;

   --
   --  Legal values for the type field of a Tk_ArgvInfo: see the user
   --  documentation for details.
   --

   TK_ARGV_CONSTANT               : constant := 15;
   TK_ARGV_INT                    : constant := 16;
   TK_ARGV_STRING                 : constant := 17;
   TK_ARGV_UID                    : constant := 18;
   TK_ARGV_REST                   : constant := 19;
   TK_ARGV_FLOAT                  : constant := 20;
   TK_ARGV_FUNC                   : constant := 21;
   TK_ARGV_GENFUNC                : constant := 22;
   TK_ARGV_HELP                   : constant := 23;
   TK_ARGV_CONST_OPTION           : constant := 24;
   TK_ARGV_OPTION_VALUE           : constant := 25;
   TK_ARGV_OPTION_NAME_VALUE      : constant := 26;
   TK_ARGV_END                    : constant := 27;
   --
   --  Flag bits for passing to Tk_ParseArgv:
   --

   TK_ARGV_NO_DEFAULTS            : constant := 1;
   TK_ARGV_NO_LEFTOVERS           : constant := 2;
   TK_ARGV_NO_ABBREV              : constant := 4;
   TK_ARGV_DONT_SKIP_FIRST_ARG    : constant := 8;
   --
   --  Structure used to describe application-specific configuration
   --  options:  indicates procedures to call to parse an option and
   --  to return a text string describing an option.
   --

   type Tk_OptionParseProc is access function (
      data            : ClientData;
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      value           : C.Strings.Chars_Ptr;
      widgRec         : C.Strings.Chars_Ptr;
      offset          : C.Int
   ) return C.Int;
   pragma Convention (C, Tk_OptionParseProc);

   type Tk_OptionPrintProc is access function (
      data            : ClientData;
      tkwin           : Tk_Window;
      widgRec         : C.Strings.Chars_Ptr;
      offset          : C.Int;
      freeProcPtr     : Tcl_FreeProc
   ) return C.Strings.Chars_Ptr;
   pragma Convention (C, Tk_OptionPrintProc);

   type Tk_CustomOption_rec is private;
   type Tk_CustomOption is access all Tk_CustomOption_rec;
   pragma Convention (C, Tk_CustomOption);

   type Tk_CustomOption_array is array (C.size_t range <>) of Tk_CustomOption;

   function Is_Null (Ptr : Tk_CustomOption) return Boolean;

   --
   --  Structure used to specify information for Tk_ConfigureWidget.  Each
   --  structure gives complete information for one option, including
   --  how the option is specified on the command line, where it appears
   --  in the option database, etc.
   --

   type Tk_ConfigSpec_rec is private;
   type Tk_ConfigSpec is access all Tk_ConfigSpec_rec;
   pragma Convention (C, Tk_ConfigSpec);

   type Tk_ConfigSpec_array is array (C.size_t range <>) of Tk_ConfigSpec;

   function Is_Null (Ptr : Tk_ConfigSpec) return Boolean;

   --
   --  Type values for Tk_ConfigSpec structures.  See the user
   --  documentation for details.
   --

   TK_CONFIG_BOOLEAN              : constant := 1;
   TK_CONFIG_INT                  : constant := 2;
   TK_CONFIG_DOUBLE               : constant := 3;
   TK_CONFIG_STRING               : constant := 4;
   TK_CONFIG_UID                  : constant := 5;
   TK_CONFIG_COLOR                : constant := 6;
   TK_CONFIG_FONT                 : constant := 7;
   TK_CONFIG_BITMAP               : constant := 8;
   TK_CONFIG_BORDER               : constant := 9;
   TK_CONFIG_RELIEF               : constant := 10;
   TK_CONFIG_CURSOR               : constant := 11;
   TK_CONFIG_ACTIVE_CURSOR        : constant := 12;
   TK_CONFIG_JUSTIFY              : constant := 13;
   TK_CONFIG_ANCHOR               : constant := 14;
   TK_CONFIG_SYNONYM              : constant := 15;
   TK_CONFIG_CAP_STYLE            : constant := 16;
   TK_CONFIG_JOIN_STYLE           : constant := 17;
   TK_CONFIG_PIXELS               : constant := 18;
   TK_CONFIG_MM                   : constant := 19;
   TK_CONFIG_WINDOW               : constant := 20;
   TK_CONFIG_CUSTOM               : constant := 21;
   TK_CONFIG_END                  : constant := 22;
   --
   --  Macro to use to fill in "offset" fields of Tk_ConfigInfos.
   --  Computes number of bytes from beginning of structure to a
   --  given field.
   --

   --
   --  Possible values for flags argument to Tk_ConfigureWidget:
   --

   TK_CONFIG_ARGV_ONLY            : constant := 1;
   --
   --  Possible flag values for Tk_ConfigInfo structures.  Any bits at
   --  or above TK_CONFIG_USER_BIT may be used by clients for selecting
   --  certain entries.  Before changing any values here, coordinate with
   --  tkConfig.c {internal-use-only flags are defined there}.
   --

   TK_CONFIG_COLOR_ONLY           : constant := 1;
   TK_CONFIG_MONO_ONLY            : constant := 2;
   TK_CONFIG_NULL_OK              : constant := 4;
   TK_CONFIG_DONT_SET_DEFAULT     : constant := 8;
   TK_CONFIG_OPTION_SPECIFIED     : constant := 16;
   TK_CONFIG_USER_BIT             : constant := 256;
   --
   --  Enumerated type for describing actions to be taken in response
   --  to a restrictProc established by Tk_RestrictEvents.
   --

   type Tk_RestrictAction is (
      TK_DEFER_EVENT,
      TK_PROCESS_EVENT,
      TK_DISCARD_EVENT
   );
   for Tk_RestrictAction'size use 32;

   --
   --  Priority levels to pass to Tk_AddOption:
   --

   TK_WIDGET_DEFAULT_PRIO         : constant := 20;
   TK_STARTUP_FILE_PRIO           : constant := 40;
   TK_USER_DEFAULT_PRIO           : constant := 60;
   TK_INTERACTIVE_PRIO            : constant := 80;
   TK_MAX_PRIO                    : constant := 100;
   --
   --  Relief values returned by Tk_GetRelief:
   --

   TK_RELIEF_RAISED               : constant := 1;
   TK_RELIEF_FLAT                 : constant := 2;
   TK_RELIEF_SUNKEN               : constant := 4;
   TK_RELIEF_GROOVE               : constant := 8;
   TK_RELIEF_RIDGE                : constant := 16;
   TK_RELIEF_SOLID                : constant := 32;
   --
   --  "Which" argument values for Tk_3DBorderGC:
   --

   TK_3D_FLAT_GC                  : constant := 1;
   TK_3D_LIGHT_GC                 : constant := 2;
   TK_3D_DARK_GC                  : constant := 3;
   --
   --  Special EnterNotify/LeaveNotify "mode" for use in events
   --  generated by tkShare.c.  Pick a high enough value that it's
   --  unlikely to conflict with existing values {like NotifyNormal}
   --  or any new values defined in the future.
   --

   TK_NOTIFY_SHARE                : constant := 20;
   --
   --  Enumerated type for describing a point by which to anchor something:
   --

   type Tk_Anchor is (
      TK_ANCHOR_N,
      TK_ANCHOR_NE,
      TK_ANCHOR_E,
      TK_ANCHOR_SE,
      TK_ANCHOR_S,
      TK_ANCHOR_SW,
      TK_ANCHOR_W,
      TK_ANCHOR_NW,
      TK_ANCHOR_CENTER
   );
   for Tk_Anchor'size use 32;

   --
   --  Enumerated type for describing a style of justification:
   --

   type Tk_Justify is (
      TK_JUSTIFY_LEFT,
      TK_JUSTIFY_RIGHT,
      TK_JUSTIFY_CENTER
   );
   for Tk_Justify'size use 32;

   --
   --  The following structure is used by Tk_GetFontMetrics {} to return
   --  information about the properties of a Tk_Font.
   --

   type Tk_FontMetrics_rec is private;
   type Tk_FontMetrics is access all Tk_FontMetrics_rec;
   pragma Convention (C, Tk_FontMetrics);

   type Tk_FontMetrics_array is array (C.size_t range <>) of Tk_FontMetrics;

   function Is_Null (Ptr : Tk_FontMetrics) return Boolean;

   --
   --  Flags passed to Tk_MeasureChars:
   --

   TK_WHOLE_WORDS                 : constant := 1;
   TK_AT_LEAST_ONE                : constant := 2;
   TK_PARTIAL_OK                  : constant := 4;
   --
   --  Flags passed to Tk_ComputeTextLayout:
   --

   TK_IGNORE_TABS                 : constant := 8;
   TK_IGNORE_NEWLINES             : constant := 16;
   --
   --  Each geometry manager {the packer, the placer, etc.} is represented
   --  by a structure of the following form, which indicates procedures
   --  to invoke in the geometry manager to carry out certain functions.
   --

   type Tk_GeomRequestProc is access procedure (
      data            : ClientData;
      tkwin           : Tk_Window
   );
   pragma Convention (C, Tk_GeomRequestProc);

   type Tk_GeomLostSlaveProc is access procedure (
      data            : ClientData;
      tkwin           : Tk_Window
   );
   pragma Convention (C, Tk_GeomLostSlaveProc);

   type Tk_GeomMgr_rec is private;
   type Tk_GeomMgr is access all Tk_GeomMgr_rec;
   pragma Convention (C, Tk_GeomMgr);

   type Tk_GeomMgr_array is array (C.size_t range <>) of Tk_GeomMgr;

   function Is_Null (Ptr : Tk_GeomMgr) return Boolean;

   --
   --  Result values returned by Tk_GetScrollInfo:
   --

   TK_SCROLL_MOVETO               : constant := 1;
   TK_SCROLL_PAGES                : constant := 2;
   TK_SCROLL_UNITS                : constant := 3;
   TK_SCROLL_ERROR                : constant := 4;
   --
   -- ---------------------------------------------------------------------------
   --
   --  Extensions to the X event set
   --
   -- ---------------------------------------------------------------------------
   --

   VirtualEventMask               : constant := 1073741824;
   ActivateMask                   : constant := 536870912;
   --
   --  A virtual event shares most of its fields with the XKeyEvent and
   --  XButtonEvent structures.  99% of the time a virtual event will be
   --  an abstraction of a key or button event, so this structure provides
   --  the most information to the user.  The only difference is the changing
   --  of the detail field for a virtual event so that it holds the name of the
   --  virtual event being triggered.
   --

   type XVirtualEvent_rec is private;
   type XVirtualEvent is access all XVirtualEvent_rec;
   pragma Convention (C, XVirtualEvent);

   type XVirtualEvent_array is array (C.size_t range <>) of XVirtualEvent;

   function Is_Null (Ptr : XVirtualEvent) return Boolean;

   type XActivateDeactivateEvent_rec is private;
   type XActivateDeactivateEvent is access all XActivateDeactivateEvent_rec;
   pragma Convention (C, XActivateDeactivateEvent);

   type XActivateDeactivateEvent_array is array (C.size_t range <>) of XActivateDeactivateEvent;

   function Is_Null (Ptr : XActivateDeactivateEvent) return Boolean;

   subtype XActivateEvent is XActivateDeactivateEvent;

   subtype XDeactivateEvent is XActivateDeactivateEvent;

   --
   -- --------------------------------------------------------------
   --
   --  Macros for querying Tk_Window structures.  See the
   --  manual entries for documentation.
   --
   -- --------------------------------------------------------------
   --

   function Tk_ScreenNumber (
      tkwin           : Tk_Window
   ) return C.Int;
   pragma Import (C, Tk_ScreenNumber, "Tk_CallScreenNumber");

   function Tk_Depth (
      tkwin           : Tk_Window
   ) return C.Int;
   pragma Import (C, Tk_Depth, "Tk_CallDepth");

   function Tk_PathName (
      tkwin           : Tk_Window
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_PathName, "Tk_CallPathName");

   function Tk_Name (
      tkwin           : Tk_Window
   ) return Tk_Uid;
   pragma Import (C, Tk_Name, "Tk_CallName");

   function Tk_Class (
      tkwin           : Tk_Window
   ) return Tk_Uid;
   pragma Import (C, Tk_Class, "Tk_CallClass");

   function Tk_Parent (
      tkwin           : Tk_Window
   ) return Tk_Window;
   pragma Import (C, Tk_Parent, "Tk_CallParent");

   --
   --  The structure below is needed by the macros above so that they can
   --  access the fields of a Tk_Window.  The fields not needed by the macros
   --  are declared as "dummyX".  The structure has its own type in order to
   --  prevent applications from accessing Tk_Window fields except using
   --  official macros.  WARNING!! The structure definition must be kept
   --  consistent with the TkWindow structure in tkInt.h.  If you change one,
   --  then change the other.  See the declaration in tkInt.h for
   --  documentation on what the fields are used for internally.
   --

   type Tk_FakeWin is private;

   function Is_Null (Ptr : Tk_FakeWin) return Boolean;

   --
   --  Flag values for TkWindow {and Tk_FakeWin} structures are:
   --
   --  TK_MAPPED:                       1 means window is currently mapped,
   --                           0 means unmapped.
   --  TK_TOP_LEVEL:            1 means this is a top-level window {it
   --                           was or will be created as a child of
   --                           a root window}.
   --  TK_ALREADY_DEAD:         1 means the window is in the process of
   --                           being destroyed already.
   --  TK_NEED_CONFIG_NOTIFY:   1 means that the window has been reconfigured
   --                           before it was made to exist.  At the time of
   --                           making it exist a ConfigureNotify event needs
   --                           to be generated.
   --  TK_GRAB_FLAG:            Used to manage grabs.  See tkGrab.c for
   --                           details.
   --  TK_CHECKED_IC:           1 means we've already tried to get an input
   --                           context for this window;  if the ic field
   --                           is NULL it means that there isn't a context
   --                           for the field.
   --  TK_DONT_DESTROY_WINDOW:  1 means that Tk_DestroyWindow should not
   --                           invoke XDestroyWindow to destroy this widget's
   --                           X window.  The flag is set when the window
   --                           has already been destroyed elsewhere {e.g.
   --                           by another application} or when it will be
   --                           destroyed later {e.g. by destroying its
   --                           parent}.
   --  TK_WM_COLORMAP_WINDOW:   1 means that this window has at some time
   --                           appeared in the WM_COLORMAP_WINDOWS property
   --                           for its toplevel, so we have to remove it
   --                           from that property if the window is
   --                           deleted and the toplevel isn't.
   --  TK_EMBEDDED:                     1 means that this window {which must be a
   --                           toplevel} is not a free-standing window but
   --                           rather is embedded in some other application.
   --  TK_CONTAINER:            1 means that this window is a container, and
   --                           that some other application {either in
   --                           this process or elsewhere} may be
   --                           embedding itself inside the window.
   --  TK_BOTH_HALVES:          1 means that this window is used for
   --                           application embedding {either as
   --                           container or embedded application}, and
   --                           both the containing and embedded halves
   --                           are associated with windows in this
   --                           particular process.
   --  TK_DEFER_MODAL:          1 means that this window has deferred a modal
   --                           loop until all of the bindings for the current
   --                           event have been invoked.
   --  TK_WRAPPER:                      1 means that this window is the extra
   --                           wrapper window created around a toplevel
   --                           to hold the menubar under Unix.  See
   --                           tkUnixWm.c for more information.
   --  TK_REPARENTED:           1 means that this window has been reparented
   --                           so that as far as the window system is
   --                           concerned it isn't a child of its Tk
   --                           parent.  Initially this is used only for
   --                           special Unix menubar windows.
   --

   TK_MAPPED                      : constant := 1;
   TK_TOP_LEVEL                   : constant := 2;
   TK_ALREADY_DEAD                : constant := 4;
   TK_NEED_CONFIG_NOTIFY          : constant := 8;
   TK_GRAB_FLAG                   : constant := 16;
   TK_CHECKED_IC                  : constant := 32;
   TK_DONT_DESTROY_WINDOW         : constant := 64;
   TK_WM_COLORMAP_WINDOW          : constant := 128;
   TK_EMBEDDED                    : constant := 256;
   TK_CONTAINER                   : constant := 512;
   TK_BOTH_HALVES                 : constant := 1024;
   TK_DEFER_MODAL                 : constant := 2048;
   TK_WRAPPER                     : constant := 4096;
   TK_REPARENTED                  : constant := 8192;
   --
   -- --------------------------------------------------------------
   --
   --  Procedure prototypes and structures used for defining new canvas
   --  items:
   --
   -- --------------------------------------------------------------
   --

   --
   --  For each item in a canvas widget there exists one record with
   --  the following structure.  Each actual item is represented by
   --  a record with the following stuff at its beginning, plus additional
   --  type-specific stuff after that.
   --

   TK_TAG_SPACE                   : constant := 3;
   type Tk_Item_rec is private;
   type Tk_Item is access all Tk_Item_rec;
   pragma Convention (C, Tk_Item);

   type Tk_Item_array is array (C.size_t range <>) of Tk_Item;

   function Is_Null (Ptr : Tk_Item) return Boolean;

   --
   --  Records of the following type are used to describe a type of
   --  item {e.g.  lines, circles, etc.} that can form part of a
   --  canvas widget.
   --

   type Tk_ItemCreateProc is access function (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Convention (C, Tk_ItemCreateProc);

   type Tk_ItemConfigureProc is access function (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ItemConfigureProc);

   type Tk_ItemCoordProc is access function (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Convention (C, Tk_ItemCoordProc);

   type Tk_ItemPointProc is access function (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      pointPtr        : access C.Double
   ) return C.Double;
   pragma Convention (C, Tk_ItemPointProc);

   type Tk_ItemAreaProc is access function (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      rectPtr         : access C.Double
   ) return C.Int;
   pragma Convention (C, Tk_ItemAreaProc);

   type Tk_ItemPostscriptProc is access function (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      prepass         : C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ItemPostscriptProc);

   type Tk_ItemScaleProc is access procedure (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      originX         : C.Double;
      originY         : C.Double;
      scaleX          : C.Double;
      scaleY          : C.Double
   );
   pragma Convention (C, Tk_ItemScaleProc);

   type Tk_ItemTranslateProc is access procedure (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      deltaX          : C.Double;
      deltaY          : C.Double
   );
   pragma Convention (C, Tk_ItemTranslateProc);

   type Tk_ItemIndexProc is access function (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      indexString     : C.Strings.Chars_Ptr;
      indexPtr        : access C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ItemIndexProc);

   type Tk_ItemCursorProc is access procedure (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      index           : C.Int
   );
   pragma Convention (C, Tk_ItemCursorProc);

   type Tk_ItemSelectionProc is access function (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      offset          : C.Int;
      buffer          : C.Strings.Chars_Ptr;
      maxBytes        : C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ItemSelectionProc);

   type Tk_ItemInsertProc is access procedure (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      beforeThis      : C.Int;
      strng           : C.Strings.Chars_Ptr
   );
   pragma Convention (C, Tk_ItemInsertProc);

   type Tk_ItemDCharsProc is access procedure (
      canvas          : Tk_Canvas;
      itemPtr         : Tk_Item;
      first           : C.Int;
      last            : C.Int
   );
   pragma Convention (C, Tk_ItemDCharsProc);

   type Tk_ItemType_rec is private;
   type Tk_ItemType is access all Tk_ItemType_rec;
   pragma Convention (C, Tk_ItemType);

   type Tk_ItemType_array is array (C.size_t range <>) of Tk_ItemType;

   function Is_Null (Ptr : Tk_ItemType) return Boolean;

   --
   --  The following structure provides information about the selection and
   --  the insertion cursor.  It is needed by only a few items, such as
   --  those that display text.  It is shared by the generic canvas code
   --  and the item-specific code, but most of the fields should be written
   --  only by the canvas generic code.
   --

   type Tk_CanvasTextInfo_rec is private;
   type Tk_CanvasTextInfo is access all Tk_CanvasTextInfo_rec;
   pragma Convention (C, Tk_CanvasTextInfo);

   type Tk_CanvasTextInfo_array is array (C.size_t range <>) of Tk_CanvasTextInfo;

   function Is_Null (Ptr : Tk_CanvasTextInfo) return Boolean;

   --
   -- --------------------------------------------------------------
   --
   --  Procedure prototypes and structures used for managing images:
   --
   -- --------------------------------------------------------------
   --

   type Tk_ImageType is private;

   function Is_Null (Ptr : Tk_ImageType) return Boolean;

   type Tk_ImageCreateProc is access function (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      typePtr         : Tk_ImageType;
      master          : Tk_ImageMaster;
      masterdataptr   : ClientData
   ) return C.Int;
   pragma Convention (C, Tk_ImageCreateProc);

   type Tk_ImageGetProc is access function (
      tkwin           : Tk_Window;
      masterdata      : ClientData
   ) return ClientData;
   pragma Convention (C, Tk_ImageGetProc);

   type Tk_ImageDeleteProc is access procedure (
      masterdata      : ClientData
   );
   pragma Convention (C, Tk_ImageDeleteProc);

   type Tk_ImageChangedProc is access procedure (
      data            : ClientData;
      x               : C.Int;
      y               : C.Int;
      width           : C.Int;
      height          : C.Int;
      imageWidth      : C.Int;
      imageHeight     : C.Int
   );
   pragma Convention (C, Tk_ImageChangedProc);

   --
   --  The following structure represents a particular type of image
   --  {bitmap, xpm image, etc.}.  It provides information common to
   --  all images of that type, such as the type name and a collection
   --  of procedures in the image manager that respond to various
   --  events.  Each image manager is represented by one of these
   --  structures.
   --

   --
   -- --------------------------------------------------------------
   --
   --  Additional definitions used to manage images of type "photo".
   --
   -- --------------------------------------------------------------
   --

   --
   --  The following type is used to identify a particular photo image
   --  to be manipulated:
   --

   type Tk_PhotoHandle is private;

   function Is_Null (Ptr : Tk_PhotoHandle) return Boolean;

   --
   --  The following structure describes a block of pixels in memory:
   --

   type Tk_PhotoImageBlock_rec is private;
   type Tk_PhotoImageBlock is access all Tk_PhotoImageBlock_rec;
   pragma Convention (C, Tk_PhotoImageBlock);

   type Tk_PhotoImageBlock_array is array (C.size_t range <>) of Tk_PhotoImageBlock;

   function Is_Null (Ptr : Tk_PhotoImageBlock) return Boolean;

   --
   --  Procedure prototypes and structures used in reading and
   --  writing photo images:
   --

   type Tk_PhotoImageFormat is private;

   function Is_Null (Ptr : Tk_PhotoImageFormat) return Boolean;

   type Tk_ImageFileMatchProc is access function (
      chan            : Tcl_Channel;
      fileName        : C.Strings.Chars_Ptr;
      formatString    : C.Strings.Chars_Ptr;
      widthPtr        : access C.Int;
      heightPtr       : access C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ImageFileMatchProc);

   type Tk_ImageStringMatchProc is access function (
      strng           : C.Strings.Chars_Ptr;
      formatString    : C.Strings.Chars_Ptr;
      widthPtr        : access C.Int;
      heightPtr       : access C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ImageStringMatchProc);

   type Tk_ImageFileReadProc is access function (
      interp          : Tcl_Interp;
      chan            : Tcl_Channel;
      fileName        : C.Strings.Chars_Ptr;
      formatString    : C.Strings.Chars_Ptr;
      imageHandle     : Tk_PhotoHandle;
      destX           : C.Int;
      destY           : C.Int;
      width           : C.Int;
      height          : C.Int;
      srcX            : C.Int;
      srcY            : C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ImageFileReadProc);

   type Tk_ImageStringReadProc is access function (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      formatString    : C.Strings.Chars_Ptr;
      imageHandle     : Tk_PhotoHandle;
      destX           : C.Int;
      destY           : C.Int;
      width           : C.Int;
      height          : C.Int;
      srcX            : C.Int;
      srcY            : C.Int
   ) return C.Int;
   pragma Convention (C, Tk_ImageStringReadProc);

   type Tk_ImageFileWriteProc is access function (
      interp          : Tcl_Interp;
      fileName        : C.Strings.Chars_Ptr;
      formatString    : C.Strings.Chars_Ptr;
      blockPtr        : Tk_PhotoImageBlock
   ) return C.Int;
   pragma Convention (C, Tk_ImageFileWriteProc);

   type Tk_ImageStringWriteProc is access function (
      interp          : Tcl_Interp;
      dataPtr         : Tcl_DString;
      formatString    : C.Strings.Chars_Ptr;
      blockPtr        : Tk_PhotoImageBlock
   ) return C.Int;
   pragma Convention (C, Tk_ImageStringWriteProc);

   --
   --  The following structure represents a particular file format for
   --  storing images {e.g., PPM, GIF, JPEG, etc.}.  It provides information
   --  to allow image files of that format to be recognized and read into
   --  a photo image.
   --

   --
   -- --------------------------------------------------------------
   --
   --  The definitions below provide backward compatibility for
   --  functions and types related to event handling that used to
   --  be in Tk but have moved to Tcl.
   --
   -- --------------------------------------------------------------
   --

   TK_READABLE                    : constant := TCL_READABLE;
   TK_WRITABLE                    : constant := TCL_WRITABLE;
   TK_EXCEPTION                   : constant := TCL_EXCEPTION;
   TK_DONT_WAIT                   : constant := TCL_DONT_WAIT;
   TK_X_EVENTS                    : constant := TCL_WINDOW_EVENTS;
   TK_WINDOW_EVENTS               : constant := TCL_WINDOW_EVENTS;
   TK_FILE_EVENTS                 : constant := TCL_FILE_EVENTS;
   TK_TIMER_EVENTS                : constant := TCL_TIMER_EVENTS;
   TK_IDLE_EVENTS                 : constant := TCL_IDLE_EVENTS;
   TK_ALL_EVENTS                  : constant := TCL_ALL_EVENTS;
   -- Additional stuff that has moved to Tcl:

   --
   -- --------------------------------------------------------------
   --
   --  Additional procedure types defined by Tk.
   --
   -- --------------------------------------------------------------
   --

   type Tk_GetSelProc is access function (
      data            : ClientData;
      interp          : Tcl_Interp;
      portion         : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Convention (C, Tk_GetSelProc);

   type Tk_LostSelProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tk_LostSelProc);

   type Tk_SelectionProc is access function (
      data            : ClientData;
      offset          : C.Int;
      buffer          : C.Strings.Chars_Ptr;
      maxBytes        : C.Int
   ) return C.Int;
   pragma Convention (C, Tk_SelectionProc);

   --
   -- --------------------------------------------------------------
   --
   --  Exported procedures and variables.
   --
   -- --------------------------------------------------------------
   --

   procedure Tk_AddOption (
      tkwin           : Tk_Window;
      name            : C.Strings.Chars_Ptr;
      value           : C.Strings.Chars_Ptr;
      priority        : C.Int
   );
   pragma Import (C, Tk_AddOption, "Tk_AddOption");

   procedure Tk_CanvasDrawableCoords (
      canvas          : Tk_Canvas;
      x               : C.Double;
      y               : C.Double;
      drawableXPtr    : access C.Short;
      drawableYPtr    : access C.Short
   );
   pragma Import (C, Tk_CanvasDrawableCoords, "Tk_CanvasDrawableCoords");

   procedure Tk_CanvasEventuallyRedraw (
      canvas          : Tk_Canvas;
      x1              : C.Int;
      y1              : C.Int;
      x2              : C.Int;
      y2              : C.Int
   );
   pragma Import (C, Tk_CanvasEventuallyRedraw, "Tk_CanvasEventuallyRedraw");

   function Tk_CanvasGetCoord (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      strng           : C.Strings.Chars_Ptr;
      doublePtr       : access C.Double
   ) return C.Int;
   pragma Import (C, Tk_CanvasGetCoord, "Tk_CanvasGetCoord");

   function Tk_CanvasGetTextInfo (
      canvas          : Tk_Canvas
   ) return Tk_CanvasTextInfo;
   pragma Import (C, Tk_CanvasGetTextInfo, "Tk_CanvasGetTextInfo");

   function Tk_CanvasPsFont (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      font            : Tk_Font
   ) return C.Int;
   pragma Import (C, Tk_CanvasPsFont, "Tk_CanvasPsFont");

   procedure Tk_CanvasPsPath (
      interp          : Tcl_Interp;
      canvas          : Tk_Canvas;
      coordPtr        : access C.Double;
      numPoints       : C.Int
   );
   pragma Import (C, Tk_CanvasPsPath, "Tk_CanvasPsPath");

   function Tk_CanvasPsY (
      canvas          : Tk_Canvas;
      y               : C.Double
   ) return C.Double;
   pragma Import (C, Tk_CanvasPsY, "Tk_CanvasPsY");

   function Tk_CanvasTagsParseProc (
      data            : ClientData;
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      value           : C.Strings.Chars_Ptr;
      widgRec         : C.Strings.Chars_Ptr;
      offset          : C.Int
   ) return C.Int;
   pragma Import (C, Tk_CanvasTagsParseProc, "Tk_CanvasTagsParseProc");

   function Tk_CanvasTagsPrintProc (
      data            : ClientData;
      tkwin           : Tk_Window;
      widgRec         : C.Strings.Chars_Ptr;
      offset          : C.Int;
      freeProcPtr     : Tcl_FreeProc
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_CanvasTagsPrintProc, "Tk_CanvasTagsPrintProc");

   function Tk_CanvasTkwin (
      canvas          : Tk_Canvas
   ) return Tk_Window;
   pragma Import (C, Tk_CanvasTkwin, "Tk_CanvasTkwin");

   procedure Tk_CanvasWindowCoords (
      canvas          : Tk_Canvas;
      x               : C.Double;
      y               : C.Double;
      screenXPtr      : access C.Short;
      screenYPtr      : access C.Short
   );
   pragma Import (C, Tk_CanvasWindowCoords, "Tk_CanvasWindowCoords");

   function Tk_CharBbox (
      layout          : Tk_TextLayout;
      index           : C.Int;
      xPtr            : access C.Int;
      yPtr            : access C.Int;
      widthPtr        : access C.Int;
      heightPtr       : access C.Int
   ) return C.Int;
   pragma Import (C, Tk_CharBbox, "Tk_CharBbox");

   function Tk_ClipboardClear (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window
   ) return C.Int;
   pragma Import (C, Tk_ClipboardClear, "Tk_ClipboardClear");

   function Tk_ConfigureInfo (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      specs           : Tk_ConfigSpec;
      widgRec         : C.Strings.Chars_Ptr;
      argvName        : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tk_ConfigureInfo, "Tk_ConfigureInfo");

   function Tk_ConfigureValue (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      specs           : Tk_ConfigSpec;
      widgRec         : C.Strings.Chars_Ptr;
      argvName        : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tk_ConfigureValue, "Tk_ConfigureValue");

   function Tk_ConfigureWidget (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      specs           : Tk_ConfigSpec;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      widgRec         : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tk_ConfigureWidget, "Tk_ConfigureWidget");

   function Tk_ComputeTextLayout (
      font            : Tk_Font;
      strng           : C.Strings.Chars_Ptr;
      numChars        : C.Int;
      wrapLength      : C.Int;
      justify         : Tk_Justify;
      flags           : C.Int;
      widthPtr        : access C.Int;
      heightPtr       : access C.Int
   ) return Tk_TextLayout;
   pragma Import (C, Tk_ComputeTextLayout, "Tk_ComputeTextLayout");

   function Tk_CoordsToWindow (
      rootX           : C.Int;
      rootY           : C.Int;
      tkwin           : Tk_Window
   ) return Tk_Window;
   pragma Import (C, Tk_CoordsToWindow, "Tk_CoordsToWindow");

   function Tk_CreateBinding (
      interp          : Tcl_Interp;
      bindingTable    : Tk_BindingTable;
      object          : ClientData;
      eventString     : C.Strings.Chars_Ptr;
      command         : C.Strings.Chars_Ptr;
      append          : C.Int
   ) return C.Unsigned_Long;
   pragma Import (C, Tk_CreateBinding, "Tk_CreateBinding");

   function Tk_CreateBindingTable (
      interp          : Tcl_Interp
   ) return Tk_BindingTable;
   pragma Import (C, Tk_CreateBindingTable, "Tk_CreateBindingTable");

   procedure Tk_CreateImageType (
      typePtr         : Tk_ImageType
   );
   pragma Import (C, Tk_CreateImageType, "Tk_CreateImageType");

   procedure Tk_CreateItemType (
      typePtr         : Tk_ItemType
   );
   pragma Import (C, Tk_CreateItemType, "Tk_CreateItemType");

   procedure Tk_CreatePhotoImageFormat (
      formatPtr       : Tk_PhotoImageFormat
   );
   pragma Import (C, Tk_CreatePhotoImageFormat, "Tk_CreatePhotoImageFormat");

   function Tk_CreateWindow (
      interp          : Tcl_Interp;
      parent          : Tk_Window;
      name            : C.Strings.Chars_Ptr;
      screenName      : C.Strings.Chars_Ptr
   ) return Tk_Window;
   pragma Import (C, Tk_CreateWindow, "Tk_CreateWindow");

   function Tk_CreateWindowFromPath (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      pathName        : C.Strings.Chars_Ptr;
      screenName      : C.Strings.Chars_Ptr
   ) return Tk_Window;
   pragma Import (C, Tk_CreateWindowFromPath, "Tk_CreateWindowFromPath");

   function Tk_DefineBitmap (
      interp          : Tcl_Interp;
      name            : Tk_Uid;
      source          : C.Strings.Chars_Ptr;
      width           : C.Int;
      height          : C.Int
   ) return C.Int;
   pragma Import (C, Tk_DefineBitmap, "Tk_DefineBitmap");

   procedure Tk_DefineCursor (
      window          : Tk_Window;
      cursor          : Tk_Cursor
   );
   pragma Import (C, Tk_DefineCursor, "Tk_DefineCursor");

   procedure Tk_DeleteAllBindings (
      bindingTable    : Tk_BindingTable;
      object          : ClientData
   );
   pragma Import (C, Tk_DeleteAllBindings, "Tk_DeleteAllBindings");

   function Tk_DeleteBinding (
      interp          : Tcl_Interp;
      bindingTable    : Tk_BindingTable;
      object          : ClientData;
      eventString     : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tk_DeleteBinding, "Tk_DeleteBinding");

   procedure Tk_DeleteBindingTable (
      bindingTable    : Tk_BindingTable
   );
   pragma Import (C, Tk_DeleteBindingTable, "Tk_DeleteBindingTable");

   procedure Tk_DeleteErrorHandler (
      handler         : Tk_ErrorHandler
   );
   pragma Import (C, Tk_DeleteErrorHandler, "Tk_DeleteErrorHandler");

   procedure Tk_DeleteImage (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tk_DeleteImage, "Tk_DeleteImage");

   procedure Tk_DestroyWindow (
      tkwin           : Tk_Window
   );
   pragma Import (C, Tk_DestroyWindow, "Tk_DestroyWindow");

   function Tk_DisplayName (
      tkwin           : Tk_Window
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_DisplayName, "Tk_DisplayName");

   function Tk_DistanceToTextLayout (
      layout          : Tk_TextLayout;
      x               : C.Int;
      y               : C.Int
   ) return C.Int;
   pragma Import (C, Tk_DistanceToTextLayout, "Tk_DistanceToTextLayout");

   function Tk_FindPhoto (
      interp          : Tcl_Interp;
      imageName       : C.Strings.Chars_Ptr
   ) return Tk_PhotoHandle;
   pragma Import (C, Tk_FindPhoto, "Tk_FindPhoto");

   procedure Tk_Free3DBorder (
      border          : Tk_3DBorder
   );
   pragma Import (C, Tk_Free3DBorder, "Tk_Free3DBorder");

   procedure Tk_FreeFont (
      tkfont          : Tk_Font
   );
   pragma Import (C, Tk_FreeFont, "Tk_FreeFont");

   procedure Tk_FreeImage (
      image           : Tk_Image
   );
   pragma Import (C, Tk_FreeImage, "Tk_FreeImage");

   procedure Tk_FreeTextLayout (
      textLayout      : Tk_TextLayout
   );
   pragma Import (C, Tk_FreeTextLayout, "Tk_FreeTextLayout");

   procedure Tk_GeometryRequest (
      tkwin           : Tk_Window;
      reqWidth        : C.Int;
      reqHeight       : C.Int
   );
   pragma Import (C, Tk_GeometryRequest, "Tk_GeometryRequest");

   function Tk_Get3DBorder (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      colorName       : Tk_Uid
   ) return Tk_3DBorder;
   pragma Import (C, Tk_Get3DBorder, "Tk_Get3DBorder");

   procedure Tk_GetAllBindings (
      interp          : Tcl_Interp;
      bindingTable    : Tk_BindingTable;
      object          : ClientData
   );
   pragma Import (C, Tk_GetAllBindings, "Tk_GetAllBindings");

   function Tk_GetAnchor (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      anchorPtr       : Tk_Anchor
   ) return C.Int;
   pragma Import (C, Tk_GetAnchor, "Tk_GetAnchor");

   function Tk_GetBinding (
      interp          : Tcl_Interp;
      bindingTable    : Tk_BindingTable;
      object          : ClientData;
      eventString     : C.Strings.Chars_Ptr
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_GetBinding, "Tk_GetBinding");

   function Tk_GetCapStyle (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      capPtr          : access C.Int
   ) return C.Int;
   pragma Import (C, Tk_GetCapStyle, "Tk_GetCapStyle");

   function Tk_GetCursor (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      strng           : Tk_Uid
   ) return Tk_Cursor;
   pragma Import (C, Tk_GetCursor, "Tk_GetCursor");

   function Tk_GetCursorFromData (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      source          : C.Strings.Chars_Ptr;
      mask            : C.Strings.Chars_Ptr;
      width           : C.Int;
      height          : C.Int;
      xHot            : C.Int;
      yHot            : C.Int;
      fg              : Tk_Uid;
      bg              : Tk_Uid
   ) return Tk_Cursor;
   pragma Import (C, Tk_GetCursorFromData, "Tk_GetCursorFromData");

   function Tk_GetFont (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      strng           : C.Strings.Chars_Ptr
   ) return Tk_Font;
   pragma Import (C, Tk_GetFont, "Tk_GetFont");

   function Tk_GetFontFromObj (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      objPtr          : Tcl_Obj
   ) return Tk_Font;
   pragma Import (C, Tk_GetFontFromObj, "Tk_GetFontFromObj");

   procedure Tk_GetFontMetrics (
      font            : Tk_Font;
      fmPtr           : Tk_FontMetrics
   );
   pragma Import (C, Tk_GetFontMetrics, "Tk_GetFontMetrics");

   function Tk_GetImage (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      name            : C.Strings.Chars_Ptr;
      changeProc      : Tk_ImageChangedProc;
      data            : ClientData
   ) return Tk_Image;
   pragma Import (C, Tk_GetImage, "Tk_GetImage");

   function Tk_GetImageMasterData (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      typePtrPtr      : Tk_ImageType
   ) return ClientData;
   pragma Import (C, Tk_GetImageMasterData, "Tk_GetImageMasterData");

   function Tk_GetItemTypes return Tk_ItemType;
   pragma Import (C, Tk_GetItemTypes, "Tk_GetItemTypes");

   function Tk_GetJoinStyle (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      joinPtr         : access C.Int
   ) return C.Int;
   pragma Import (C, Tk_GetJoinStyle, "Tk_GetJoinStyle");

   function Tk_GetJustify (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      justifyPtr      : Tk_Justify
   ) return C.Int;
   pragma Import (C, Tk_GetJustify, "Tk_GetJustify");

   function Tk_GetNumMainWindows return C.Int;
   pragma Import (C, Tk_GetNumMainWindows, "Tk_GetNumMainWindows");

   function Tk_GetOption (
      tkwin           : Tk_Window;
      name            : C.Strings.Chars_Ptr;
      className       : C.Strings.Chars_Ptr
   ) return Tk_Uid;
   pragma Import (C, Tk_GetOption, "Tk_GetOption");

   function Tk_GetPixels (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      strng           : C.Strings.Chars_Ptr;
      intPtr          : access C.Int
   ) return C.Int;
   pragma Import (C, Tk_GetPixels, "Tk_GetPixels");

   function Tk_GetRelief (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      reliefPtr       : access C.Int
   ) return C.Int;
   pragma Import (C, Tk_GetRelief, "Tk_GetRelief");

   procedure Tk_GetRootCoords (
      tkwin           : Tk_Window;
      xPtr            : access C.Int;
      yPtr            : access C.Int
   );
   pragma Import (C, Tk_GetRootCoords, "Tk_GetRootCoords");

   function Tk_GetScrollInfo (
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      dblPtr          : access C.Double;
      intPtr          : access C.Int
   ) return C.Int;
   pragma Import (C, Tk_GetScrollInfo, "Tk_GetScrollInfo");

   function Tk_GetScreenMM (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      strng           : C.Strings.Chars_Ptr;
      doublePtr       : access C.Double
   ) return C.Int;
   pragma Import (C, Tk_GetScreenMM, "Tk_GetScreenMM");

   function Tk_GetUid (
      strng           : C.Strings.Chars_Ptr
   ) return Tk_Uid;
   pragma Import (C, Tk_GetUid, "Tk_GetUid");

   procedure Tk_GetVRootGeometry (
      tkwin           : Tk_Window;
      xPtr            : access C.Int;
      yPtr            : access C.Int;
      widthPtr        : access C.Int;
      heightPtr       : access C.Int
   );
   pragma Import (C, Tk_GetVRootGeometry, "Tk_GetVRootGeometry");

   function Tk_Grab (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      grabGlobal      : C.Int
   ) return C.Int;
   pragma Import (C, Tk_Grab, "Tk_Grab");

   procedure Tk_ImageChanged (
      master          : Tk_ImageMaster;
      x               : C.Int;
      y               : C.Int;
      width           : C.Int;
      height          : C.Int;
      imageWidth      : C.Int;
      imageHeight     : C.Int
   );
   pragma Import (C, Tk_ImageChanged, "Tk_ImageChanged");

   function Tk_Init (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tk_Init, "Tk_Init");

   function Tk_IntersectTextLayout (
      layout          : Tk_TextLayout;
      x               : C.Int;
      y               : C.Int;
      width           : C.Int;
      height          : C.Int
   ) return C.Int;
   pragma Import (C, Tk_IntersectTextLayout, "Tk_IntersectTextLayout");

   procedure Tk_Main (
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      appInitProc     : Tcl_AppInitProc
   );
   pragma Import (C, Tk_Main, "Tk_Main");

   procedure Tk_MainLoop;
   pragma Import (C, Tk_MainLoop, "Tk_MainLoop");

   procedure Tk_MaintainGeometry (
      slave           : Tk_Window;
      master          : Tk_Window;
      x               : C.Int;
      y               : C.Int;
      width           : C.Int;
      height          : C.Int
   );
   pragma Import (C, Tk_MaintainGeometry, "Tk_MaintainGeometry");

   function Tk_MainWindow (
      interp          : Tcl_Interp
   ) return Tk_Window;
   pragma Import (C, Tk_MainWindow, "Tk_MainWindow");

   procedure Tk_MakeWindowExist (
      tkwin           : Tk_Window
   );
   pragma Import (C, Tk_MakeWindowExist, "Tk_MakeWindowExist");

   procedure Tk_ManageGeometry (
      tkwin           : Tk_Window;
      mgrPtr          : Tk_GeomMgr;
      data            : ClientData
   );
   pragma Import (C, Tk_ManageGeometry, "Tk_ManageGeometry");

   procedure Tk_MapWindow (
      tkwin           : Tk_Window
   );
   pragma Import (C, Tk_MapWindow, "Tk_MapWindow");

   function Tk_MeasureChars (
      tkfont          : Tk_Font;
      source          : C.Strings.Chars_Ptr;
      maxChars        : C.Int;
      maxPixels       : C.Int;
      flags           : C.Int;
      lengthPtr       : access C.Int
   ) return C.Int;
   pragma Import (C, Tk_MeasureChars, "Tk_MeasureChars");

   procedure Tk_MoveResizeWindow (
      tkwin           : Tk_Window;
      x               : C.Int;
      y               : C.Int;
      width           : C.Int;
      height          : C.Int
   );
   pragma Import (C, Tk_MoveResizeWindow, "Tk_MoveResizeWindow");

   procedure Tk_MoveWindow (
      tkwin           : Tk_Window;
      x               : C.Int;
      y               : C.Int
   );
   pragma Import (C, Tk_MoveWindow, "Tk_MoveWindow");

   procedure Tk_MoveToplevelWindow (
      tkwin           : Tk_Window;
      x               : C.Int;
      y               : C.Int
   );
   pragma Import (C, Tk_MoveToplevelWindow, "Tk_MoveToplevelWindow");

   function Tk_NameOf3DBorder (
      border          : Tk_3DBorder
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOf3DBorder, "Tk_NameOf3DBorder");

   function Tk_NameOfAnchor (
      anchor          : Tk_Anchor
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOfAnchor, "Tk_NameOfAnchor");

   function Tk_NameOfCapStyle (
      cap             : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOfCapStyle, "Tk_NameOfCapStyle");

   function Tk_NameOfFont (
      font            : Tk_Font
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOfFont, "Tk_NameOfFont");

   function Tk_NameOfImage (
      imageMaster     : Tk_ImageMaster
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOfImage, "Tk_NameOfImage");

   function Tk_NameOfJoinStyle (
      join            : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOfJoinStyle, "Tk_NameOfJoinStyle");

   function Tk_NameOfJustify (
      justify         : Tk_Justify
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOfJustify, "Tk_NameOfJustify");

   function Tk_NameOfRelief (
      relief          : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_NameOfRelief, "Tk_NameOfRelief");

   function Tk_NameToWindow (
      interp          : Tcl_Interp;
      pathName        : C.Strings.Chars_Ptr;
      tkwin           : Tk_Window
   ) return Tk_Window;
   pragma Import (C, Tk_NameToWindow, "Tk_NameToWindow");

   function Tk_ParseArgv (
      interp          : Tcl_Interp;
      tkwin           : Tk_Window;
      argcPtr         : access C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      argTable        : Tk_ArgvInfo;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tk_ParseArgv, "Tk_ParseArgv");

   procedure Tk_PhotoPutBlock (
      handle          : Tk_PhotoHandle;
      blockPtr        : Tk_PhotoImageBlock;
      x               : C.Int;
      y               : C.Int;
      width           : C.Int;
      height          : C.Int
   );
   pragma Import (C, Tk_PhotoPutBlock, "Tk_PhotoPutBlock");

   procedure Tk_PhotoPutZoomedBlock (
      handle          : Tk_PhotoHandle;
      blockPtr        : Tk_PhotoImageBlock;
      x               : C.Int;
      y               : C.Int;
      width           : C.Int;
      height          : C.Int;
      zoomX           : C.Int;
      zoomY           : C.Int;
      subsampleX      : C.Int;
      subsampleY      : C.Int
   );
   pragma Import (C, Tk_PhotoPutZoomedBlock, "Tk_PhotoPutZoomedBlock");

   function Tk_PhotoGetImage (
      handle          : Tk_PhotoHandle;
      blockPtr        : Tk_PhotoImageBlock
   ) return C.Int;
   pragma Import (C, Tk_PhotoGetImage, "Tk_PhotoGetImage");

   procedure Tk_PhotoBlank (
      handle          : Tk_PhotoHandle
   );
   pragma Import (C, Tk_PhotoBlank, "Tk_PhotoBlank");

   procedure Tk_PhotoExpand (
      handle          : Tk_PhotoHandle;
      width           : C.Int;
      height          : C.Int
   );
   pragma Import (C, Tk_PhotoExpand, "Tk_PhotoExpand");

   procedure Tk_PhotoGetSize (
      handle          : Tk_PhotoHandle;
      widthPtr        : access C.Int;
      heightPtr       : access C.Int
   );
   pragma Import (C, Tk_PhotoGetSize, "Tk_PhotoGetSize");

   procedure Tk_PhotoSetSize (
      handle          : Tk_PhotoHandle;
      width           : C.Int;
      height          : C.Int
   );
   pragma Import (C, Tk_PhotoSetSize, "Tk_PhotoSetSize");

   function Tk_PointToChar (
      layout          : Tk_TextLayout;
      x               : C.Int;
      y               : C.Int
   ) return C.Int;
   pragma Import (C, Tk_PointToChar, "Tk_PointToChar");

   function Tk_PostscriptFontName (
      tkfont          : Tk_Font;
      dsPtr           : Tcl_DString
   ) return C.Int;
   pragma Import (C, Tk_PostscriptFontName, "Tk_PostscriptFontName");

   procedure Tk_ResizeWindow (
      tkwin           : Tk_Window;
      width           : C.Int;
      height          : C.Int
   );
   pragma Import (C, Tk_ResizeWindow, "Tk_ResizeWindow");

   function Tk_RestackWindow (
      tkwin           : Tk_Window;
      aboveBelow      : C.Int;
      other           : Tk_Window
   ) return C.Int;
   pragma Import (C, Tk_RestackWindow, "Tk_RestackWindow");

   function Tk_SafeInit (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tk_SafeInit, "Tk_SafeInit");

   function Tk_SetAppName (
      tkwin           : Tk_Window;
      name            : C.Strings.Chars_Ptr
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tk_SetAppName, "Tk_SetAppName");

   procedure Tk_SetBackgroundFromBorder (
      tkwin           : Tk_Window;
      border          : Tk_3DBorder
   );
   pragma Import (C, Tk_SetBackgroundFromBorder, "Tk_SetBackgroundFromBorder");

   procedure Tk_SetClass (
      tkwin           : Tk_Window;
      className       : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tk_SetClass, "Tk_SetClass");

   procedure Tk_SetGrid (
      tkwin           : Tk_Window;
      reqWidth        : C.Int;
      reqHeight       : C.Int;
      gridWidth       : C.Int;
      gridHeight      : C.Int
   );
   pragma Import (C, Tk_SetGrid, "Tk_SetGrid");

   procedure Tk_SetInternalBorder (
      tkwin           : Tk_Window;
      width           : C.Int
   );
   pragma Import (C, Tk_SetInternalBorder, "Tk_SetInternalBorder");

   procedure Tk_SetWindowBackground (
      tkwin           : Tk_Window;
      pixel           : C.Unsigned_Long
   );
   pragma Import (C, Tk_SetWindowBackground, "Tk_SetWindowBackground");

   procedure Tk_SetWindowBorder (
      tkwin           : Tk_Window;
      pixel           : C.Unsigned_Long
   );
   pragma Import (C, Tk_SetWindowBorder, "Tk_SetWindowBorder");

   procedure Tk_SetWindowBorderWidth (
      tkwin           : Tk_Window;
      width           : C.Int
   );
   pragma Import (C, Tk_SetWindowBorderWidth, "Tk_SetWindowBorderWidth");

   procedure Tk_SizeOfImage (
      image           : Tk_Image;
      widthPtr        : access C.Int;
      heightPtr       : access C.Int
   );
   pragma Import (C, Tk_SizeOfImage, "Tk_SizeOfImage");

   function Tk_StrictMotif (
      tkwin           : Tk_Window
   ) return C.Int;
   pragma Import (C, Tk_StrictMotif, "Tk_StrictMotif");

   procedure Tk_TextLayoutToPostscript (
      interp          : Tcl_Interp;
      layout          : Tk_TextLayout
   );
   pragma Import (C, Tk_TextLayoutToPostscript, "Tk_TextLayoutToPostscript");

   function Tk_TextWidth (
      font            : Tk_Font;
      strng           : C.Strings.Chars_Ptr;
      numChars        : C.Int
   ) return C.Int;
   pragma Import (C, Tk_TextWidth, "Tk_TextWidth");

   procedure Tk_UndefineCursor (
      window          : Tk_Window
   );
   pragma Import (C, Tk_UndefineCursor, "Tk_UndefineCursor");

   procedure Tk_Ungrab (
      tkwin           : Tk_Window
   );
   pragma Import (C, Tk_Ungrab, "Tk_Ungrab");

   procedure Tk_UnmaintainGeometry (
      slave           : Tk_Window;
      master          : Tk_Window
   );
   pragma Import (C, Tk_UnmaintainGeometry, "Tk_UnmaintainGeometry");

   procedure Tk_UnmapWindow (
      tkwin           : Tk_Window
   );
   pragma Import (C, Tk_UnmapWindow, "Tk_UnmapWindow");

   procedure Tk_UnsetGrid (
      tkwin           : Tk_Window
   );
   pragma Import (C, Tk_UnsetGrid, "Tk_UnsetGrid");

   procedure Tk_UpdatePointer (
      tkwin           : Tk_Window;
      x               : C.Int;
      y               : C.Int;
      state           : C.Int
   );
   pragma Import (C, Tk_UpdatePointer, "Tk_UpdatePointer");

   --
   --  Tcl commands exported by Tk:
   --

   function Tk_AfterCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_AfterCmd, "Tk_AfterCmd");

   function Tk_BellCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_BellCmd, "Tk_BellCmd");

   function Tk_BindCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_BindCmd, "Tk_BindCmd");

   function Tk_BindtagsCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_BindtagsCmd, "Tk_BindtagsCmd");

   function Tk_ButtonCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ButtonCmd, "Tk_ButtonCmd");

   function Tk_CanvasCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_CanvasCmd, "Tk_CanvasCmd");

   function Tk_CheckbuttonCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_CheckbuttonCmd, "Tk_CheckbuttonCmd");

   function Tk_ClipboardCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ClipboardCmd, "Tk_ClipboardCmd");

   function Tk_ChooseColorCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ChooseColorCmd, "Tk_ChooseColorCmd");

   function Tk_ChooseFontCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ChooseFontCmd, "Tk_ChooseFontCmd");

   function Tk_DestroyCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_DestroyCmd, "Tk_DestroyCmd");

   function Tk_EntryCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_EntryCmd, "Tk_EntryCmd");

   function Tk_EventCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_EventCmd, "Tk_EventCmd");

   function Tk_FileeventCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_FileeventCmd, "Tk_FileeventCmd");

   function Tk_FrameCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_FrameCmd, "Tk_FrameCmd");

   function Tk_FocusCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_FocusCmd, "Tk_FocusCmd");

   function Tk_FontObjCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tk_FontObjCmd, "Tk_FontObjCmd");

   function Tk_GetOpenFileCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_GetOpenFileCmd, "Tk_GetOpenFileCmd");

   function Tk_GetSaveFileCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_GetSaveFileCmd, "Tk_GetSaveFileCmd");

   function Tk_GrabCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_GrabCmd, "Tk_GrabCmd");

   function Tk_GridCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_GridCmd, "Tk_GridCmd");

   function Tk_ImageCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ImageCmd, "Tk_ImageCmd");

   function Tk_LabelCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_LabelCmd, "Tk_LabelCmd");

   function Tk_ListboxCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ListboxCmd, "Tk_ListboxCmd");

   function Tk_LowerCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_LowerCmd, "Tk_LowerCmd");

   function Tk_MenuCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_MenuCmd, "Tk_MenuCmd");

   function Tk_MenubuttonCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_MenubuttonCmd, "Tk_MenubuttonCmd");

   function Tk_MessageBoxCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_MessageBoxCmd, "Tk_MessageBoxCmd");

   function Tk_MessageCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_MessageCmd, "Tk_MessageCmd");

   function Tk_OptionCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_OptionCmd, "Tk_OptionCmd");

   function Tk_PackCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_PackCmd, "Tk_PackCmd");

   function Tk_PlaceCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_PlaceCmd, "Tk_PlaceCmd");

   function Tk_RadiobuttonCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_RadiobuttonCmd, "Tk_RadiobuttonCmd");

   function Tk_RaiseCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_RaiseCmd, "Tk_RaiseCmd");

   function Tk_ScaleCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ScaleCmd, "Tk_ScaleCmd");

   function Tk_ScrollbarCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ScrollbarCmd, "Tk_ScrollbarCmd");

   function Tk_SelectionCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_SelectionCmd, "Tk_SelectionCmd");

   function Tk_SendCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_SendCmd, "Tk_SendCmd");

   function Tk_TextCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_TextCmd, "Tk_TextCmd");

   function Tk_TkObjCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tk_TkObjCmd, "Tk_TkObjCmd");

   function Tk_TkwaitCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_TkwaitCmd, "Tk_TkwaitCmd");

   function Tk_ToplevelCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_ToplevelCmd, "Tk_ToplevelCmd");

   function Tk_UpdateCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_UpdateCmd, "Tk_UpdateCmd");

   function Tk_WinfoObjCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tk_WinfoObjCmd, "Tk_WinfoObjCmd");

   function Tk_WmCmd (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tk_WmCmd, "Tk_WmCmd");

   function Tcl_GetResult (
      interp          : Tcl_Interp
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetResult, "Tcl_GetResult");


private
   type Tk_3DBorder_rec is null record;
   type Tk_3DBorder is access all Tk_3DBorder_rec;
   pragma Convention (C, Tk_3DBorder);

   type Tk_3DBorder_array is array (C.size_t range <>) of Tk_3DBorder;

   type Tk_ArgvInfo_rec is
      record
         key             : C.Strings.Chars_Ptr;
         -- The key string that flags the option in the
         -- argv array.
         typ             : C.Int;
         -- Indicates option type;  see below.
         src             : C.Strings.Chars_Ptr;
         -- Value to be used in setting dst;  usage
         -- depends on type.
         dst             : C.Strings.Chars_Ptr;
         -- Address of value to be modified;  usage
         -- depends on type.
         help            : C.Strings.Chars_Ptr;
         -- Documentation message describing this option.
      end record;

   type Tk_BindingTable_rec is null record;
   type Tk_BindingTable is access all Tk_BindingTable_rec;
   pragma Convention (C, Tk_BindingTable);

   type Tk_BindingTable_array is array (C.size_t range <>) of Tk_BindingTable;

   type Tk_Canvas_rec is null record;
   type Tk_Canvas is access all Tk_Canvas_rec;
   pragma Convention (C, Tk_Canvas);

   type Tk_Canvas_array is array (C.size_t range <>) of Tk_Canvas;

   type Tk_CanvasTextInfo_rec is
      record
         selBorder       : Tk_3DBorder;
         -- Border and background for selected
         -- characters.  Read-only to items.
         selBorderWidth  : C.Int;
         -- Width of border around selection.
         -- Read-only to items.
      end record;

   type Tk_ConfigSpec_rec is
      record
         typ             : C.Int;
         -- Type of option, such as TK_CONFIG_COLOR;
         -- see definitions below.  Last option in
         -- table must have type TK_CONFIG_END.
         argvName        : C.Strings.Chars_Ptr;
         -- Switch used to specify option in argv.
         -- NULL means this spec is part of a group.
         dbName          : C.Strings.Chars_Ptr;
         -- Name for option in option database.
         dbClass         : C.Strings.Chars_Ptr;
         -- Class for option in database.
         defValue        : C.Strings.Chars_Ptr;
         -- Default value for option if not
         -- specified in command line or database.
         offset          : C.Int;
         -- Where in widget record to store value;
         -- use Tk_Offset macro to generate values
         -- for this.
         specFlags       : C.Int;
         -- Any combination of the values defined
         -- below;  other bits are used internally
         -- by tkConfig.c.
         customPtr       : Tk_CustomOption;
         -- If type is TK_CONFIG_CUSTOM then this is
         -- a pointer to info about how to parse and
         -- print the option.  Otherwise it is
         -- irrelevant.
      end record;

   type Tk_Cursor_rec is null record;
   type Tk_Cursor is access all Tk_Cursor_rec;
   pragma Convention (C, Tk_Cursor);

   type Tk_Cursor_array is array (C.size_t range <>) of Tk_Cursor;

   type Tk_CustomOption_rec is
      record
         parseProc       : Tk_OptionParseProc;
         -- Procedure to call to parse an
         -- option and store it in converted
         -- form.
         printProc       : Tk_OptionPrintProc;
         -- Procedure to return a printable
         -- string describing an existing
         -- option.
         data            : ClientData;
         -- Arbitrary one-word value used by
         -- option parser:  passed to
         -- parseProc and printProc.
      end record;

   type Tk_ErrorHandler_rec is null record;
   type Tk_ErrorHandler is access all Tk_ErrorHandler_rec;
   pragma Convention (C, Tk_ErrorHandler);

   type Tk_ErrorHandler_array is array (C.size_t range <>) of Tk_ErrorHandler;

   type Tk_FakeWin_rec is null record;
   type Tk_FakeWin is access all Tk_FakeWin_rec;
   pragma Convention (C, Tk_FakeWin);

   type Tk_FakeWin_array is array (C.size_t range <>) of Tk_FakeWin;

   type Tk_Font_rec is null record;
   type Tk_Font is access all Tk_Font_rec;
   pragma Convention (C, Tk_Font);

   type Tk_Font_array is array (C.size_t range <>) of Tk_Font;

   type Tk_FontMetrics_rec is
      record
         ascent          : C.Int;
         -- The amount in pixels that the tallest
         -- letter sticks up above the baseline, plus
         -- any extra blank space added by the designer
         -- of the font.
         descent         : C.Int;
         -- The largest amount in pixels that any
         -- letter sticks below the baseline, plus any
         -- extra blank space added by the designer of
         -- the font.
         linespace       : C.Int;
         -- The sum of the ascent and descent.  How
         -- far apart two lines of text in the same
         -- font should be placed so that none of the
         -- characters in one line overlap any of the
         -- characters in the other line.
      end record;

   type Tk_GeomMgr_rec is
      record
         name            : C.Strings.Chars_Ptr;
         -- Name of the geometry manager {command
         -- used to invoke it, or name of widget
         -- class that allows embedded widgets}.
         requestProc     : Tk_GeomRequestProc;
         -- Procedure to invoke when a slave's
         -- requested geometry changes.
         lostSlaveProc   : Tk_GeomLostSlaveProc;
         -- Procedure to invoke when a slave is
         -- taken away from one geometry manager
         -- by another.  NULL means geometry manager
         -- doesn't care when slaves are lost.
      end record;

   type Tk_Image_rec is null record;
   type Tk_Image is access all Tk_Image_rec;
   pragma Convention (C, Tk_Image);

   type Tk_Image_array is array (C.size_t range <>) of Tk_Image;

   type Tk_ImageMaster_rec is null record;
   type Tk_ImageMaster is access all Tk_ImageMaster_rec;
   pragma Convention (C, Tk_ImageMaster);

   type Tk_ImageMaster_array is array (C.size_t range <>) of Tk_ImageMaster;

   type Tk_ImageType_rec is null record;
   type Tk_ImageType is access all Tk_ImageType_rec;
   pragma Convention (C, Tk_ImageType);

   type Tk_ImageType_array is array (C.size_t range <>) of Tk_ImageType;

   type Tk_Item_rec is
      record
         id              : C.Int;
         -- Unique identifier for this item
         -- {also serves as first tag for
         -- item}.
         nextPtr         : Tk_Item;
         -- Next in display list of all
         -- items in this canvas.  Later items
         -- in list are drawn on top of earlier
         -- ones.
      end record;

   type Tk_ItemType_rec is
      record
         name            : C.Strings.Chars_Ptr;
         -- The name of this type of item, such
         -- as "line".
         itemSize        : C.Int;
         -- Total amount of space needed for
         -- item's record.
         createProc      : Tk_ItemCreateProc;
         -- Procedure to create a new item of
         -- this type.
         configSpecs     : Tk_ConfigSpec;
         -- Pointer to array of configuration
         -- specs for this type.  Used for
         -- returning configuration info.
         configProc      : Tk_ItemConfigureProc;
         -- Procedure to call to change
         -- configuration options.
         coordProc       : Tk_ItemCoordProc;
         -- Procedure to call to get and set
         -- the item's coordinates.
      end record;

   type Tk_PhotoHandle_rec is null record;
   type Tk_PhotoHandle is access all Tk_PhotoHandle_rec;
   pragma Convention (C, Tk_PhotoHandle);

   type Tk_PhotoHandle_array is array (C.size_t range <>) of Tk_PhotoHandle;

   type Tk_PhotoImageBlock_rec is
      record
         pixelPtr        : C.Strings.Chars_Ptr;
         -- Pointer to the first pixel.
         width           : C.Int;
         -- Width of block, in pixels.
         height          : C.Int;
         -- Height of block, in pixels.
         pitch           : C.Int;
         -- Address difference between corresponding
         -- pixels in successive lines.
         pixelSize       : C.Int;
         -- Address difference between successive
         -- pixels in the same line.
         offset          : Tash_CHelper.Int_Array(0..2);
         -- Address differences between the red, green
         -- and blue components of the pixel and the
         -- pixel as a whole.
      end record;

   type Tk_PhotoImageFormat_rec is null record;
   type Tk_PhotoImageFormat is access all Tk_PhotoImageFormat_rec;
   pragma Convention (C, Tk_PhotoImageFormat);

   type Tk_PhotoImageFormat_array is array (C.size_t range <>) of Tk_PhotoImageFormat;

   type Tk_TextLayout_rec is null record;
   type Tk_TextLayout is access all Tk_TextLayout_rec;
   pragma Convention (C, Tk_TextLayout);

   type Tk_TextLayout_array is array (C.size_t range <>) of Tk_TextLayout;

   type Tk_Window_rec is null record;
   type Tk_Window is access all Tk_Window_rec;
   pragma Convention (C, Tk_Window);

   type Tk_Window_array is array (C.size_t range <>) of Tk_Window;

   type XActivateDeactivateEvent_rec is
      record
         typ             : C.Int;
         serial          : C.Unsigned_Long;
         -- # of last request processed by server
         send_event      : Boolean;
         -- True if this came from a SendEvent request
      end record;

   type XVirtualEvent_rec is
      record
         typ             : C.Int;
         serial          : C.Unsigned_Long;
         -- # of last request processed by server
         send_event      : Boolean;
         -- True if this came from a SendEvent request
      end record;


end Tcl.Tk;
