
--------------------------------------------------------------------
--
-- tcl.ads -- This package is the "thin" binding to Tcl.
--
-- Copyright (c) 1995-1998 Terry J. Westley
--
-- See the file "license.htm" for information on usage and
-- redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
--
-- This package is automatically generated from tcl.h.  Note that some
-- of the comments below, preserved from tcl.h, do not apply to the
-- Ada version.  Someday, these comments may be customized better.
--
--------------------------------------------------------------------

with Tash_CArgv;
with Tash_CHelper;
with Interfaces.C.Strings;
with System;

package Tcl is

   pragma Linker_Options ("-ltash");
   pragma Linker_Options ("-ltcl");
   pragma Linker_Options ("-ldl");
   pragma Linker_Options ("-lm");

   package C renames Interfaces.C;

   use type C.Size_t;
   --
   --  tcl.h --
   --
   --   This header file describes the externally-visible facilities
   --   of the Tcl interpreter.
   --
   --  Copyright {c} 1987-1994 The Regents of the University of California.
   --  Copyright {c} 1994-1997 Sun Microsystems, Inc.
   --  Copyright {c} 1993-1996 Lucent Technologies.
   --
   --  See the file "license.terms" for information on usage and redistribution
   --  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
   --
   --  SCCS: @{#} tcl.h 1.326 97/11/20 12:40:43
   --

   --
   --  When version numbers change here, must also go into the following files
   --  and update the version numbers:
   --
   --  library/init.tcl
   --  unix/configure.in
   --  unix/pkginfo
   --  win/makefile.bc
   --  win/makefile.vc
   --
   --  The release level should be  0 for alpha, 1 for beta, and 2 for
   --  final/patch.  The release serial value is the number that follows the
   --  "a", "b", or "p" in the patch level; for example, if the patch level
   --  is 7.6b2, TCL_RELEASE_SERIAL is 2.  It restarts at 1 whenever the
   --  release level is changed, except for the final release which is 0
   --  {the first patch will start at 1}.
   --

   TCL_MAJOR_VERSION              : constant := 8;
   TCL_MINOR_VERSION              : constant := 0;
   TCL_RELEASE_LEVEL              : constant := 2;
   TCL_RELEASE_SERIAL             : constant := 2;
   TCL_VERSION                    : constant String := "8.0";
   TCL_PATCH_LEVEL                : constant String := "8.0p2";
   --
   --  The following definitions set up the proper options for Windows
   --  compilers.  We use this method because there is no autoconf equivalent.
   --

   --
   --  The following definitions set up the proper options for Macintosh
   --  compilers.  We use this method because there is no autoconf equivalent.
   --

   --
   --  A special definition used to allow this header file to be included
   --  in resource files so that they can get obtain version information from
   --  this file.  Resource compilers don't like all the C stuff, like typedefs
   --  and procedure declarations, that occur below.
   --

   --
   --  Definitions that allow Tcl functions with variable numbers of
   --  arguments to be used with either varargs.h or stdarg.h.  TCL_VARARGS
   --  is used in procedure prototypes.  TCL_VARARGS_DEF is used to declare
   --  the arguments in a function definiton: it takes the type and name of
   --  the first argument and supplies the appropriate argument declaration
   --  string for use in the function definition.  TCL_VARARGS_START
   --  initializes the va_list data structure and returns the first argument.
   --

   --
   --  Definitions that allow this header file to be used either with or
   --  without ANSI C features like function prototypes.
   --

   --
   --  Macro to use instead of "void" for arguments that must have
   --  type "void *" in ANSI C;  maps them to type "char *" in
   --  non-ANSI systems.
   --

   --
   --  Miscellaneous declarations.
   --

   subtype ClientData is C.Int;

   --
   --  Data structures defined opaquely in this module. The definitions below
   --  just provide dummy types. A few fields are made visible in Tcl_Interp
   --  structures, namely those used for returning a string result from
   --  commands. Direct access to the result field is discouraged in Tcl 8.0.
   --  The interpreter result is either an object or a string, and the two
   --  values are kept consistent unless some C code sets interp->result
   --  directly. Programmers should use either the procedure Tcl_GetObjResult {}
   --  or Tcl_GetStringResult {} to read the interpreter's result. See the
   --  SetResult man page for details.
   --
   --  Note: any change to the Tcl_Interp definition below must be mirrored
   --  in the "real" definition in tclInt.h.
   --
   --  Note: Tcl_ObjCmdProc procedures do not directly set result and freeProc.
   --  Instead, they set a Tcl_Obj member in the "real" structure that can be
   --  accessed with Tcl_GetObjResult {} and Tcl_SetObjResult {}.
   --

   type Tcl_Interp is private;

   function Is_Null (Ptr : Tcl_Interp) return Boolean;

   type Tcl_AsyncHandler is private;

   function Is_Null (Ptr : Tcl_AsyncHandler) return Boolean;

   type Tcl_Channel is private;

   function Is_Null (Ptr : Tcl_Channel) return Boolean;

   type Tcl_Command is private;

   function Is_Null (Ptr : Tcl_Command) return Boolean;

   type Tcl_Event is private;

   function Is_Null (Ptr : Tcl_Event) return Boolean;

   type Tcl_Pid is private;

   function Is_Null (Ptr : Tcl_Pid) return Boolean;

   type Tcl_RegExp is private;

   function Is_Null (Ptr : Tcl_RegExp) return Boolean;

   type Tcl_TimerToken is private;

   function Is_Null (Ptr : Tcl_TimerToken) return Boolean;

   type Tcl_Trace is private;

   function Is_Null (Ptr : Tcl_Trace) return Boolean;

   type Tcl_Var is private;

   function Is_Null (Ptr : Tcl_Var) return Boolean;

   --
   --  When a TCL command returns, the interpreter contains a result from the
   --  command. Programmers are strongly encouraged to use one of the
   --  procedures Tcl_GetObjResult {} or Tcl_GetStringResult {} to read the
   --  interpreter's result. See the SetResult man page for details. Besides
   --  this result, the command procedure returns an integer code, which is
   --  one of the following:
   --
   --  TCL_OK           Command completed normally; the interpreter's
   --                   result contains the command's result.
   --  TCL_ERROR                The command couldn't be completed successfully;
   --                   the interpreter's result describes what went wrong.
   --  TCL_RETURN               The command requests that the current procedure
   --                   return; the interpreter's result contains the
   --                   procedure's return value.
   --  TCL_BREAK                The command requests that the innermost loop
   --                   be exited; the interpreter's result is meaningless.
   --  TCL_CONTINUE             Go on to the next iteration of the current loop;
   --                   the interpreter's result is meaningless.
   --

   TCL_OK                         : constant := 0;
   TCL_ERROR                      : constant := 1;
   TCL_RETURN                     : constant := 2;
   TCL_BREAK                      : constant := 3;
   TCL_CONTINUE                   : constant := 4;
   TCL_RESULT_SIZE                : constant := 200;
   --
   --  Argument descriptors for math function callbacks in expressions:
   --

   type Tcl_ValueType is (
      TCL_INT,
      TCL_DOUBLE,
      TCL_EITHER
   );
   for Tcl_ValueType'size use 32;

   type Tcl_Value_rec is private;
   type Tcl_Value is access all Tcl_Value_rec;
   pragma Convention (C, Tcl_Value);

   type Tcl_Value_array is array (C.size_t range <>) of Tcl_Value;

   function Is_Null (Ptr : Tcl_Value) return Boolean;

   --
   --  Forward declaration of Tcl_Obj to prevent an error when the forward
   --  reference to Tcl_Obj is encountered in the procedure types declared
   --  below.
   --

   type Tcl_Obj is private;

   function Is_Null (Ptr : Tcl_Obj) return Boolean;

   --
   --  Procedure types defined by Tcl:
   --

   type Tcl_AppInitProc is access function (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Convention (C, Tcl_AppInitProc);

   type Tcl_AsyncProc is access function (
      data            : ClientData;
      interp          : Tcl_Interp;
      code            : C.Int
   ) return C.Int;
   pragma Convention (C, Tcl_AsyncProc);

   type Tcl_ChannelProc is access procedure (
      data            : ClientData;
      mask            : C.Int
   );
   pragma Convention (C, Tcl_ChannelProc);

   type Tcl_CloseProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tcl_CloseProc);

   type Tcl_CmdDeleteProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tcl_CmdDeleteProc);

   type Tcl_CmdProc is access function (
      data            : ClientData;
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Convention (C, Tcl_CmdProc);

   type Tcl_CmdTraceProc is access procedure (
      data            : ClientData;
      interp          : Tcl_Interp;
      level           : C.Int;
      command         : C.Strings.Chars_Ptr;
      proc            : Tcl_CmdProc;
      cmdclientdata   : ClientData;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   );
   pragma Convention (C, Tcl_CmdTraceProc);

   type Tcl_DupInternalRepProc is access procedure;
   pragma Convention (C, Tcl_DupInternalRepProc);

   type Tcl_EventProc is access function (
      evPtr           : Tcl_Event;
      flags           : C.Int
   ) return C.Int;
   pragma Convention (C, Tcl_EventProc);

   type Tcl_EventCheckProc is access procedure (
      data            : ClientData;
      flags           : C.Int
   );
   pragma Convention (C, Tcl_EventCheckProc);

   type Tcl_EventDeleteProc is access function (
      evPtr           : Tcl_Event;
      data            : ClientData
   ) return C.Int;
   pragma Convention (C, Tcl_EventDeleteProc);

   type Tcl_EventSetupProc is access procedure (
      data            : ClientData;
      flags           : C.Int
   );
   pragma Convention (C, Tcl_EventSetupProc);

   type Tcl_ExitProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tcl_ExitProc);

   type Tcl_FileProc is access procedure (
      data            : ClientData;
      mask            : C.Int
   );
   pragma Convention (C, Tcl_FileProc);

   type Tcl_FileFreeProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tcl_FileFreeProc);

   type Tcl_FreeInternalRepProc is access procedure;
   pragma Convention (C, Tcl_FreeInternalRepProc);

   type Tcl_FreeProc is access procedure (
      blockPtr        : C.Strings.Chars_Ptr
   );
   pragma Convention (C, Tcl_FreeProc);

   type Tcl_IdleProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tcl_IdleProc);

   type Tcl_InterpDeleteProc is access procedure (
      data            : ClientData;
      interp          : Tcl_Interp
   );
   pragma Convention (C, Tcl_InterpDeleteProc);

   type Tcl_MathProc is access function (
      data            : ClientData;
      interp          : Tcl_Interp;
      args            : Tcl_Value;
      resultPtr       : Tcl_Value
   ) return C.Int;
   pragma Convention (C, Tcl_MathProc);

   type Tcl_NamespaceDeleteProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tcl_NamespaceDeleteProc);

   type Tcl_ObjCmdProc is access function (
      data            : ClientData;
      interp          : Tcl_Interp;
      objc            : C.Int
   ) return C.Int;
   pragma Convention (C, Tcl_ObjCmdProc);

   type Tcl_PackageInitProc is access function (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Convention (C, Tcl_PackageInitProc);

   type Tcl_TcpAcceptProc is access procedure (
      callbackdata    : ClientData;
      chan            : Tcl_Channel;
      address         : C.Strings.Chars_Ptr;
      port            : C.Int
   );
   pragma Convention (C, Tcl_TcpAcceptProc);

   type Tcl_TimerProc is access procedure (
      data            : ClientData
   );
   pragma Convention (C, Tcl_TimerProc);

   type Tcl_SetFromAnyProc is access function (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Convention (C, Tcl_SetFromAnyProc);

   type Tcl_UpdateStringProc is access procedure;
   pragma Convention (C, Tcl_UpdateStringProc);

   type Tcl_VarTraceProc is access function (
      data            : ClientData;
      interp          : Tcl_Interp;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Convention (C, Tcl_VarTraceProc);

   --
   --  The following structure represents a type of object, which is a
   --  particular internal representation for an object plus a set of
   --  procedures that provide standard operations on objects of that type.
   --

   type Tcl_ObjType_rec is private;
   type Tcl_ObjType is access all Tcl_ObjType_rec;
   pragma Convention (C, Tcl_ObjType);

   type Tcl_ObjType_array is array (C.size_t range <>) of Tcl_ObjType;

   function Is_Null (Ptr : Tcl_ObjType) return Boolean;

   --
   --  One of the following structures exists for each object in the Tcl
   --  system. An object stores a value as either a string, some internal
   --  representation, or both.
   --

   --
   --  Macros to increment and decrement a Tcl_Obj's reference count, and to
   --  test whether an object is shared {i.e. has reference count > 1}.
   --  Note: clients should use Tcl_DecrRefCount {} when they are finished using
   --  an object, and should never call TclFreeObj {} directly. TclFreeObj {} is
   --  only defined and made public in tcl.h to support Tcl_DecrRefCount's macro
   --  definition. Note also that Tcl_DecrRefCount {} refers to the parameter
   --  "obj" twice. This means that you should avoid calling it with an
   --  expression that is expensive to compute or has side effects.
   --

   procedure Tcl_IncrRefCount (
      objPtr          : Tcl_Obj
   );
   pragma Import (C, Tcl_IncrRefCount, "Tcl_IncrRefCount");

   procedure Tcl_DecrRefCount (
      objPtr          : Tcl_Obj
   );
   pragma Import (C, Tcl_DecrRefCount, "Tcl_DecrRefCount");

   function Tcl_IsShared (
      objPtr          : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_IsShared, "Tcl_IsShared");

   --
   --  Macros and definitions that help to debug the use of Tcl objects.
   --  When TCL_MEM_DEBUG is defined, the Tcl_New * declarations are
   --  overridden to call debugging versions of the object creation procedures.
   --

   function Tcl_NewBooleanObj (
      boolValue       : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_NewBooleanObj, "Tcl_NewBooleanObj");

   function Tcl_NewDoubleObj (
      doubleValue     : C.Double
   ) return Tcl_Obj;
   pragma Import (C, Tcl_NewDoubleObj, "Tcl_NewDoubleObj");

   function Tcl_NewIntObj (
      intValue        : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_NewIntObj, "Tcl_NewIntObj");

   function Tcl_NewListObj (
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   ) return Tcl_Obj;
   pragma Import (C, Tcl_NewListObj, "Tcl_NewListObj");

   function Tcl_NewLongObj (
      longValue       : C.Long
   ) return Tcl_Obj;
   pragma Import (C, Tcl_NewLongObj, "Tcl_NewLongObj");

   function Tcl_NewObj return Tcl_Obj;
   pragma Import (C, Tcl_NewObj, "Tcl_NewObj");

   function Tcl_NewStringObj (
      bytes           : C.Strings.Chars_Ptr;
      length          : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_NewStringObj, "Tcl_NewStringObj");

   --
   --  The following definitions support Tcl's namespace facility.
   --  Note: the first five fields must match exactly the fields in a
   --  Namespace structure {see tcl.h}.
   --

   type Tcl_Namespace_rec is private;
   type Tcl_Namespace is access all Tcl_Namespace_rec;
   pragma Convention (C, Tcl_Namespace);

   type Tcl_Namespace_array is array (C.size_t range <>) of Tcl_Namespace;

   function Is_Null (Ptr : Tcl_Namespace) return Boolean;

   --
   --  The following structure represents a call frame, or activation record.
   --  A call frame defines a naming context for a procedure call: its local
   --  scope {for local variables} and its namespace scope {used for non-local
   --  variables; often the global :: namespace}. A call frame can also define
   --  the naming context for a namespace eval or namespace inscope command:
   --  the namespace in which the command's code should execute. The
   --  Tcl_CallFrame structures exist only while procedures or namespace
   --  eval/inscope's are being executed, and provide a Tcl call stack.
   --
   --  A call frame is initialized and pushed using Tcl_PushCallFrame and
   --  popped using Tcl_PopCallFrame. Storage for a Tcl_CallFrame must be
   --  provided by the Tcl_PushCallFrame caller, and callers typically allocate
   --  them on the C call stack for efficiency. For this reason, Tcl_CallFrame
   --  is defined as a structure and not as an opaque token. However, most
   --  Tcl_CallFrame fields are hidden since applications should not access
   --  them directly; others are declared as "dummyX".
   --
   --  WARNING!! The structure definition must be kept consistent with the
   --  CallFrame structure in tclInt.h. If you change one, change the other.
   --

   type Tcl_CallFrame_rec is private;
   type Tcl_CallFrame is access all Tcl_CallFrame_rec;
   pragma Convention (C, Tcl_CallFrame);

   type Tcl_CallFrame_array is array (C.size_t range <>) of Tcl_CallFrame;

   function Is_Null (Ptr : Tcl_CallFrame) return Boolean;

   --
   --  Information about commands that is returned by Tcl_GetCommandInfo and
   --  passed to Tcl_SetCommandInfo. objProc is an objc/objv object-based
   --  command procedure while proc is a traditional Tcl argc/argv
   --  string-based procedure. Tcl_CreateObjCommand and Tcl_CreateCommand
   --  ensure that both objProc and proc are non-NULL and can be called to
   --  execute the command. However, it may be faster to call one instead of
   --  the other. The member isNativeObjectProc is set to 1 if an
   --  object-based procedure was registered by Tcl_CreateObjCommand, and to
   --  0 if a string-based procedure was registered by Tcl_CreateCommand.
   --  The other procedure is typically set to a compatibility wrapper that
   --  does string-to-object or object-to-string argument conversions then
   --  calls the other procedure.
   --

   type Tcl_CmdInfo_rec is private;
   type Tcl_CmdInfo is access all Tcl_CmdInfo_rec;
   pragma Convention (C, Tcl_CmdInfo);

   type Tcl_CmdInfo_array is array (C.size_t range <>) of Tcl_CmdInfo;

   function Is_Null (Ptr : Tcl_CmdInfo) return Boolean;

   --
   --  The structure defined below is used to hold dynamic strings.  The only
   --  field that clients should use is the string field, and they should
   --  never modify it.
   --

   TCL_DSTRING_STATIC_SIZE        : constant := 200;
   type Tcl_DString_rec is private;
   type Tcl_DString is access all Tcl_DString_rec;
   pragma Convention (C, Tcl_DString);

   type Tcl_DString_array is array (C.size_t range <>) of Tcl_DString;

   function Is_Null (Ptr : Tcl_DString) return Boolean;

   function Tcl_DStringLength (
      dsPtr           : Tcl_DString
   ) return C.Int;
   pragma Import (C, Tcl_DStringLength, "Tcl_CallDStringLength");

   function Tcl_DStringValue (
      dsPtr           : Tcl_DString
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_DStringValue, "Tcl_CallDStringValue");

   procedure Tcl_DStringSetLength (
      dsPtr           : Tcl_DString;
      length          : C.Int
   );
   pragma Import (C, Tcl_DStringSetLength, "Tcl_DStringSetLength");

   procedure Tcl_DStringTrunc (
      dsPtr           : Tcl_DString;
      length          : C.Int
   ) renames Tcl_DStringSetLength;

   --
   --  Definitions for the maximum number of digits of precision that may
   --  be specified in the "tcl_precision" variable, and the number of
   --  characters of buffer space required by Tcl_PrintDouble.
   --

   TCL_MAX_PREC                   : constant := 17;
   TCL_DOUBLE_SPACE               : constant := TCL_MAX_PREC+10;
   --
   --  Flag that may be passed to Tcl_ConvertElement to force it not to
   --  output braces {careful!  if you change this flag be sure to change
   --  the definitions at the front of tclUtil.c}.
   --

   TCL_DONT_USE_BRACES            : constant := 1;
   --
   --  Flag that may be passed to Tcl_GetIndexFromObj to force it to disallow
   --  abbreviated strings.
   --

   TCL_EXACT                      : constant := 1;
   --
   --  Flag values passed to Tcl_RecordAndEval.
   --  WARNING: these bit choices must not conflict with the bit choices
   --  for evalFlag bits in tclInt.h!!
   --

   TCL_NO_EVAL                    : constant := 65536;
   TCL_EVAL_GLOBAL                : constant := 131072;
   --
   --  Special freeProc values that may be passed to Tcl_SetResult {see
   --  the man page for details}:
   --

   TCL_VOLATILE                   : constant :=  1;
   TCL_STATIC                     : constant :=  0;
   TCL_DYNAMIC                    : constant :=  3;
   --
   --  Flag values passed to variable-related procedures.
   --

   TCL_GLOBAL_ONLY                : constant := 1;
   TCL_NAMESPACE_ONLY             : constant := 2;
   TCL_APPEND_VALUE               : constant := 4;
   TCL_LIST_ELEMENT               : constant := 8;
   TCL_TRACE_READS                : constant := 16;
   TCL_TRACE_WRITES               : constant := 32;
   TCL_TRACE_UNSETS               : constant := 64;
   TCL_TRACE_DESTROYED            : constant := 128;
   TCL_INTERP_DESTROYED           : constant := 256;
   TCL_LEAVE_ERR_MSG              : constant := 512;
   TCL_PARSE_PART1                : constant := 1024;
   --
   --  Types for linked variables:
   --

   TCL_LINK_INT                   : constant := 1;
   TCL_LINK_DOUBLE                : constant := 2;
   TCL_LINK_BOOLEAN               : constant := 3;
   TCL_LINK_STRING                : constant := 4;
   TCL_LINK_READ_ONLY             : constant := 128;
   --
   --  malloc and free, or they map them to procedures with all sorts
   --  of debugging hooks defined in tclCkalloc.c.
   --

   --
   --  Forward declaration of Tcl_HashTable.  Needed by some C++ compilers
   --  to prevent errors when the forward reference to Tcl_HashTable is
   --  encountered in the Tcl_HashEntry structure.
   --

   --
   --  Structure definition for an entry in a hash table.  No-one outside
   --  Tcl should access any of these fields directly;  use the macros
   --  defined below.
   --

   type Tcl_HashEntry is private;

   function Is_Null (Ptr : Tcl_HashEntry) return Boolean;

   --
   --  Structure definition for a hash table.  Must be in tcl.h so clients
   --  can allocate space for these structures, but clients should never
   --  access any fields in this structure.
   --

   TCL_SMALL_HASH_TABLE           : constant := 4;
   type Tcl_HashTable_rec is private;
   type Tcl_HashTable is access all Tcl_HashTable_rec;
   pragma Convention (C, Tcl_HashTable);

   type Tcl_HashTable_array is array (C.size_t range <>) of Tcl_HashTable;

   function Is_Null (Ptr : Tcl_HashTable) return Boolean;

   --
   --  Structure definition for information used to keep track of searches
   --  through hash tables:
   --

   type Tcl_HashSearch_rec is private;
   type Tcl_HashSearch is access all Tcl_HashSearch_rec;
   pragma Convention (C, Tcl_HashSearch);

   type Tcl_HashSearch_array is array (C.size_t range <>) of Tcl_HashSearch;

   function Is_Null (Ptr : Tcl_HashSearch) return Boolean;

   --
   --  Acceptable key types for hash tables:
   --

   TCL_STRING_KEYS                : constant := 0;
   TCL_ONE_WORD_KEYS              : constant := 1;
   --
   --  Macros for clients to use to access fields of hash entries:
   --

   function Tcl_GetHashValue (
      HashEntry       : Tcl_HashEntry
   ) return ClientData;
   pragma Import (C, Tcl_GetHashValue, "Tcl_CallGetHashValue");

   procedure Tcl_SetHashValue (
      HashEntry       : Tcl_HashEntry;
      value           : ClientData
   );
   pragma Import (C, Tcl_SetHashValue, "Tcl_CallSetHashValue");

   function Tcl_GetHashKey (
      HashTable       : Tcl_HashTable;
      HashEntry       : Tcl_HashEntry
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetHashKey, "Tcl_CallGetHashKey");

   --
   --  Macros to use for clients to use to invoke find and create procedures
   --  for hash tables:
   --

   function Tcl_FindHashEntry (
      HashTable       : Tcl_HashTable;
      key             : C.Strings.Chars_Ptr
   ) return Tcl_HashEntry;
   pragma Import (C, Tcl_FindHashEntry, "Tcl_CallFindHashEntry");

   function Tcl_CreateHashEntry (
      HashTable       : Tcl_HashTable;
      key             : C.Strings.Chars_Ptr;
      newPtr          : access C.Int
   ) return Tcl_HashEntry;
   pragma Import (C, Tcl_CreateHashEntry, "Tcl_CallCreateHashEntry");

   --
   --  Flag values to pass to Tcl_DoOneEvent to disable searches
   --  for some kinds of events:
   --

   TCL_DONT_WAIT                  : constant := 2;
   TCL_WINDOW_EVENTS              : constant := 4;
   TCL_FILE_EVENTS                : constant := 8;
   TCL_TIMER_EVENTS               : constant := 16;
   TCL_IDLE_EVENTS                : constant := 32;
   -- WAS 0x10 ????

   TCL_ALL_EVENTS                 : constant := -3;
   --
   --  The following structure defines a generic event for the Tcl event
   --  system.  These are the things that are queued in calls to Tcl_QueueEvent
   --  and serviced later by Tcl_DoOneEvent.  There can be many different
   --  kinds of events with different fields, corresponding to window events,
   --  timer events, etc.  The structure for a particular event consists of
   --  a Tcl_Event header followed by additional information specific to that
   --  event.
   --

   --
   --  Positions to pass to Tcl_QueueEvent:
   --

   type Tcl_QueuePosition is (
      TCL_QUEUE_TAIL,
      TCL_QUEUE_HEAD,
      TCL_QUEUE_MARK
   );
   for Tcl_QueuePosition'size use 32;

   --
   --  Values to pass to Tcl_SetServiceMode to specify the behavior of notifier
   --  event routines.
   --

   TCL_SERVICE_NONE               : constant := 0;
   TCL_SERVICE_ALL                : constant := 1;
   --
   --  The following structure keeps is used to hold a time value, either as
   --  an absolute time {the number of seconds from the epoch} or as an
   --  elapsed time. On Unix systems the epoch is Midnight Jan 1, 1970 GMT.
   --  On Macintosh systems the epoch is Midnight Jan 1, 1904 GMT.
   --

   type Tcl_Time_rec is private;
   type Tcl_Time is access all Tcl_Time_rec;
   pragma Convention (C, Tcl_Time);

   type Tcl_Time_array is array (C.size_t range <>) of Tcl_Time;

   function Is_Null (Ptr : Tcl_Time) return Boolean;

   --
   --  Bits to pass to Tcl_CreateFileHandler and Tcl_CreateChannelHandler
   --  to indicate what sorts of events are of interest:
   --

   TCL_READABLE                   : constant := 2;
   TCL_WRITABLE                   : constant := 4;
   TCL_EXCEPTION                  : constant := 8;
   --
   --  Flag values to pass to Tcl_OpenCommandChannel to indicate the
   --  disposition of the stdio handles.  TCL_STDIN, TCL_STDOUT, TCL_STDERR,
   --  are also used in Tcl_GetStdChannel.
   --

   TCL_STDIN                      : constant := 2;
   TCL_STDOUT                     : constant := 4;
   TCL_STDERR                     : constant := 8;
   TCL_ENFORCE_MODE               : constant := 16;
   --
   --  Typedefs for the various operations in a channel type:
   --

   type Tcl_DriverBlockModeProc is access function (
      instancedata    : ClientData;
      mode            : C.Int
   ) return C.Int;
   pragma Convention (C, Tcl_DriverBlockModeProc);

   type Tcl_DriverCloseProc is access function (
      instancedata    : ClientData;
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Convention (C, Tcl_DriverCloseProc);

   type Tcl_DriverInputProc is access function (
      instancedata    : ClientData;
      buf             : C.Strings.Chars_Ptr;
      toRead          : C.Int;
      errorCodePtr    : access C.Int
   ) return C.Int;
   pragma Convention (C, Tcl_DriverInputProc);

   type Tcl_DriverOutputProc is access function (
      instancedata    : ClientData;
      buf             : C.Strings.Chars_Ptr;
      toWrite         : C.Int;
      errorCodePtr    : access C.Int
   ) return C.Int;
   pragma Convention (C, Tcl_DriverOutputProc);

   type Tcl_DriverSeekProc is access function (
      instancedata    : ClientData;
      offset          : C.Long;
      mode            : C.Int;
      errorCodePtr    : access C.Int
   ) return C.Int;
   pragma Convention (C, Tcl_DriverSeekProc);

   type Tcl_DriverSetOptionProc is access function (
      instancedata    : ClientData;
      interp          : Tcl_Interp;
      optionName      : C.Strings.Chars_Ptr;
      value           : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Convention (C, Tcl_DriverSetOptionProc);

   type Tcl_DriverGetOptionProc is access function (
      instancedata    : ClientData;
      interp          : Tcl_Interp;
      optionName      : C.Strings.Chars_Ptr;
      dsPtr           : Tcl_DString
   ) return C.Int;
   pragma Convention (C, Tcl_DriverGetOptionProc);

   type Tcl_DriverWatchProc is access procedure (
      instancedata    : ClientData;
      mask            : C.Int
   );
   pragma Convention (C, Tcl_DriverWatchProc);

   type Tcl_DriverGetHandleProc is access function (
      instancedata    : ClientData;
      direction       : C.Int;
      handleptr       : ClientData
   ) return C.Int;
   pragma Convention (C, Tcl_DriverGetHandleProc);

   --
   --  Enum for different end of line translation and recognition modes.
   --

   type Tcl_EolTranslation is (
      TCL_TRANSLATE_AUTO,
      -- Eol == \r, \n and \r\n.
      TCL_TRANSLATE_CR,
      -- Eol == \r.
      TCL_TRANSLATE_LF,
      -- Eol == \n.
      TCL_TRANSLATE_CRLF
      -- Eol == \r\n.
   );
   for Tcl_EolTranslation'size use 32;

   --
   --  struct Tcl_ChannelType:
   --
   --  One such structure exists for each type {kind} of channel.
   --  It collects together in one place all the functions that are
   --  part of the specific channel type.
   --

   type Tcl_ChannelType_rec is private;
   type Tcl_ChannelType is access all Tcl_ChannelType_rec;
   pragma Convention (C, Tcl_ChannelType);

   type Tcl_ChannelType_array is array (C.size_t range <>) of Tcl_ChannelType;

   function Is_Null (Ptr : Tcl_ChannelType) return Boolean;

   --
   --  The following flags determine whether the blockModeProc above should
   --  set the channel into blocking or nonblocking mode. They are passed
   --  as arguments to the blockModeProc procedure in the above structure.
   --

   TCL_MODE_BLOCKING              : constant := 0;
   -- Put channel into blocking mode.

   TCL_MODE_NONBLOCKING           : constant := 1;
   -- Put channel into nonblocking
   --  mode.

   --
   --  Enum for different types of file paths.
   --

   type Tcl_PathType is (
      TCL_PATH_ABSOLUTE,
      TCL_PATH_RELATIVE,
      TCL_PATH_VOLUME_RELATIVE
   );
   for Tcl_PathType'size use 32;

   --
   --  Exported Tcl procedures:
   --

   procedure Tcl_AddErrorInfo (
      interp          : Tcl_Interp;
      message         : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_AddErrorInfo, "Tcl_AddErrorInfo");

   procedure Tcl_AddObjErrorInfo (
      interp          : Tcl_Interp;
      message         : C.Strings.Chars_Ptr;
      length          : C.Int
   );
   pragma Import (C, Tcl_AddObjErrorInfo, "Tcl_AddObjErrorInfo");

   procedure Tcl_AllowExceptions (
      interp          : Tcl_Interp
   );
   pragma Import (C, Tcl_AllowExceptions, "Tcl_AllowExceptions");

   function Tcl_AppendAllObjTypes (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_AppendAllObjTypes, "Tcl_AppendAllObjTypes");

   procedure Tcl_AppendElement (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_AppendElement, "Tcl_AppendElement");

   procedure Tcl_AppendResult (
      interp          : Tcl_Interp;
      String1         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String2         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String3         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String4         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String5         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String6         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String7         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String8         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String9         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr
   );
   pragma Import (C, Tcl_AppendResult, "Tcl_AppendResult");

   procedure Tcl_AppendToObj (
      objPtr          : Tcl_Obj;
      bytes           : C.Strings.Chars_Ptr;
      length          : C.Int
   );
   pragma Import (C, Tcl_AppendToObj, "Tcl_AppendToObj");

   procedure Tcl_AppendStringsToObj (
      interp          : Tcl_Obj;
      String1         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String2         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String3         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String4         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String5         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String6         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String7         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String8         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String9         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr
   );
   pragma Import (C, Tcl_AppendStringsToObj, "Tcl_AppendStringsToObj");

   function Tcl_AppInit (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tcl_AppInit, "Tcl_AppInit");

   function Tcl_AsyncCreate (
      proc            : Tcl_AsyncProc;
      data            : ClientData
   ) return Tcl_AsyncHandler;
   pragma Import (C, Tcl_AsyncCreate, "Tcl_AsyncCreate");

   procedure Tcl_AsyncDelete (
      async           : Tcl_AsyncHandler
   );
   pragma Import (C, Tcl_AsyncDelete, "Tcl_AsyncDelete");

   function Tcl_AsyncInvoke (
      interp          : Tcl_Interp;
      code            : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_AsyncInvoke, "Tcl_AsyncInvoke");

   procedure Tcl_AsyncMark (
      async           : Tcl_AsyncHandler
   );
   pragma Import (C, Tcl_AsyncMark, "Tcl_AsyncMark");

   function Tcl_AsyncReady return C.Int;
   pragma Import (C, Tcl_AsyncReady, "Tcl_AsyncReady");

   procedure Tcl_BackgroundError (
      interp          : Tcl_Interp
   );
   pragma Import (C, Tcl_BackgroundError, "Tcl_BackgroundError");

   function Tcl_Backslash (
      src             : C.Strings.Chars_Ptr;
      readPtr         : access C.Int
   ) return C.Char;
   pragma Import (C, Tcl_Backslash, "Tcl_Backslash");

   function Tcl_BadChannelOption (
      interp          : Tcl_Interp;
      optionName      : C.Strings.Chars_Ptr;
      optionList      : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_BadChannelOption, "Tcl_BadChannelOption");

   procedure Tcl_CallWhenDeleted (
      interp          : Tcl_Interp;
      proc            : Tcl_InterpDeleteProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CallWhenDeleted, "Tcl_CallWhenDeleted");

   procedure Tcl_CancelIdleCall (
      idleProc        : Tcl_IdleProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CancelIdleCall, "Tcl_CancelIdleCall");

   function Tcl_Close (
      interp          : Tcl_Interp;
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_Close, "Tcl_Close");

   function Tcl_CommandComplete (
      cmd             : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_CommandComplete, "Tcl_CommandComplete");

   function Tcl_Concat (
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_Concat, "Tcl_Concat");

   function Tcl_ConcatObj (
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   ) return Tcl_Obj;
   pragma Import (C, Tcl_ConcatObj, "Tcl_ConcatObj");

   function Tcl_ConvertCountedElement (
      src             : C.Strings.Chars_Ptr;
      length          : C.Int;
      dst             : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ConvertCountedElement, "Tcl_ConvertCountedElement");

   function Tcl_ConvertElement (
      src             : C.Strings.Chars_Ptr;
      dst             : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ConvertElement, "Tcl_ConvertElement");

   function Tcl_ConvertToType (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      typePtr         : Tcl_ObjType
   ) return C.Int;
   pragma Import (C, Tcl_ConvertToType, "Tcl_ConvertToType");

   function Tcl_CreateAlias (
      slave           : Tcl_Interp;
      slaveCmd        : C.Strings.Chars_Ptr;
      target          : Tcl_Interp;
      targetCmd       : C.Strings.Chars_Ptr;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_CreateAlias, "Tcl_CreateAlias");

   function Tcl_CreateAliasObj (
      slave           : Tcl_Interp;
      slaveCmd        : C.Strings.Chars_Ptr;
      target          : Tcl_Interp;
      targetCmd       : C.Strings.Chars_Ptr;
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_CreateAliasObj, "Tcl_CreateAliasObj");

   function Tcl_CreateChannel (
      typePtr         : Tcl_ChannelType;
      chanName        : C.Strings.Chars_Ptr;
      instancedata    : ClientData;
      mask            : C.Int
   ) return Tcl_Channel;
   pragma Import (C, Tcl_CreateChannel, "Tcl_CreateChannel");

   procedure Tcl_CreateChannelHandler (
      chan            : Tcl_Channel;
      mask            : C.Int;
      proc            : Tcl_ChannelProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CreateChannelHandler, "Tcl_CreateChannelHandler");

   procedure Tcl_CreateCloseHandler (
      chan            : Tcl_Channel;
      proc            : Tcl_CloseProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CreateCloseHandler, "Tcl_CreateCloseHandler");

   function Tcl_CreateCommand (
      interp          : Tcl_Interp;
      cmdName         : C.Strings.Chars_Ptr;
      proc            : Tcl_CmdProc;
      data            : ClientData;
      deleteProc      : Tcl_CmdDeleteProc
   ) return Tcl_Command;
   pragma Import (C, Tcl_CreateCommand, "Tcl_CreateCommand");

   procedure Tcl_CreateEventSource (
      setupProc       : Tcl_EventSetupProc;
      checkProc       : Tcl_EventCheckProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CreateEventSource, "Tcl_CreateEventSource");

   procedure Tcl_CreateExitHandler (
      proc            : Tcl_ExitProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CreateExitHandler, "Tcl_CreateExitHandler");

   procedure Tcl_CreateFileHandler (
      fd              : C.Int;
      mask            : C.Int;
      proc            : Tcl_FileProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CreateFileHandler, "Tcl_CreateFileHandler");

   function Tcl_CreateInterp return Tcl_Interp;
   pragma Import (C, Tcl_CreateInterp, "Tcl_CreateInterp");

   procedure Tcl_CreateMathFunc (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      numArgs         : C.Int;
      argTypes        : Tcl_ValueType;
      proc            : Tcl_MathProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_CreateMathFunc, "Tcl_CreateMathFunc");

   function Tcl_CreateObjCommand (
      interp          : Tcl_Interp;
      cmdName         : C.Strings.Chars_Ptr;
      proc            : Tcl_ObjCmdProc;
      data            : ClientData;
      deleteProc      : Tcl_CmdDeleteProc
   ) return Tcl_Command;
   pragma Import (C, Tcl_CreateObjCommand, "Tcl_CreateObjCommand");

   function Tcl_CreateSlave (
      interp          : Tcl_Interp;
      slaveName       : C.Strings.Chars_Ptr;
      isSafe          : C.Int
   ) return Tcl_Interp;
   pragma Import (C, Tcl_CreateSlave, "Tcl_CreateSlave");

   function Tcl_CreateTimerHandler (
      milliseconds    : C.Int;
      proc            : Tcl_TimerProc;
      data            : ClientData
   ) return Tcl_TimerToken;
   pragma Import (C, Tcl_CreateTimerHandler, "Tcl_CreateTimerHandler");

   function Tcl_CreateTrace (
      interp          : Tcl_Interp;
      level           : C.Int;
      proc            : Tcl_CmdTraceProc;
      data            : ClientData
   ) return Tcl_Trace;
   pragma Import (C, Tcl_CreateTrace, "Tcl_CreateTrace");

   function Tcl_DbCkalloc (
      size            : C.Unsigned;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_DbCkalloc, "Tcl_DbCkalloc");

   function Tcl_DbCkfree (
      ptr             : C.Strings.Chars_Ptr;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_DbCkfree, "Tcl_DbCkfree");

   function Tcl_DbCkrealloc (
      ptr             : C.Strings.Chars_Ptr;
      size            : C.Unsigned;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_DbCkrealloc, "Tcl_DbCkrealloc");

   procedure Tcl_DbDecrRefCount (
      objPtr          : Tcl_Obj;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   );
   pragma Import (C, Tcl_DbDecrRefCount, "Tcl_DbDecrRefCount");

   procedure Tcl_DbIncrRefCount (
      objPtr          : Tcl_Obj;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   );
   pragma Import (C, Tcl_DbIncrRefCount, "Tcl_DbIncrRefCount");

   function Tcl_DbIsShared (
      objPtr          : Tcl_Obj;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_DbIsShared, "Tcl_DbIsShared");

   function Tcl_DbNewBooleanObj (
      boolValue       : C.Int;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_DbNewBooleanObj, "Tcl_DbNewBooleanObj");

   function Tcl_DbNewDoubleObj (
      doubleValue     : C.Double;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_DbNewDoubleObj, "Tcl_DbNewDoubleObj");

   function Tcl_DbNewListObj (
      objc            : C.Int;
      CONST_objv      : Tcl_Obj;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_DbNewListObj, "Tcl_DbNewListObj");

   function Tcl_DbNewLongObj (
      longValue       : C.Long;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_DbNewLongObj, "Tcl_DbNewLongObj");

   function Tcl_DbNewObj (
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_DbNewObj, "Tcl_DbNewObj");

   function Tcl_DbNewStringObj (
      bytes           : C.Strings.Chars_Ptr;
      length          : C.Int;
      file            : C.Strings.Chars_Ptr;
      line            : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_DbNewStringObj, "Tcl_DbNewStringObj");

   procedure Tcl_DeleteAssocData (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_DeleteAssocData, "Tcl_DeleteAssocData");

   function Tcl_DeleteCommand (
      interp          : Tcl_Interp;
      cmdName         : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_DeleteCommand, "Tcl_DeleteCommand");

   function Tcl_DeleteCommandFromToken (
      interp          : Tcl_Interp;
      command         : Tcl_Command
   ) return C.Int;
   pragma Import (C, Tcl_DeleteCommandFromToken, "Tcl_DeleteCommandFromToken");

   procedure Tcl_DeleteChannelHandler (
      chan            : Tcl_Channel;
      proc            : Tcl_ChannelProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_DeleteChannelHandler, "Tcl_DeleteChannelHandler");

   procedure Tcl_DeleteCloseHandler (
      chan            : Tcl_Channel;
      proc            : Tcl_CloseProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_DeleteCloseHandler, "Tcl_DeleteCloseHandler");

   procedure Tcl_DeleteEvents (
      proc            : Tcl_EventDeleteProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_DeleteEvents, "Tcl_DeleteEvents");

   procedure Tcl_DeleteEventSource (
      setupProc       : Tcl_EventSetupProc;
      checkProc       : Tcl_EventCheckProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_DeleteEventSource, "Tcl_DeleteEventSource");

   procedure Tcl_DeleteExitHandler (
      proc            : Tcl_ExitProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_DeleteExitHandler, "Tcl_DeleteExitHandler");

   procedure Tcl_DeleteFileHandler (
      fd              : C.Int
   );
   pragma Import (C, Tcl_DeleteFileHandler, "Tcl_DeleteFileHandler");

   procedure Tcl_DeleteHashEntry (
      entryPtr        : Tcl_HashEntry
   );
   pragma Import (C, Tcl_DeleteHashEntry, "Tcl_DeleteHashEntry");

   procedure Tcl_DeleteHashTable (
      tablePtr        : Tcl_HashTable
   );
   pragma Import (C, Tcl_DeleteHashTable, "Tcl_DeleteHashTable");

   procedure Tcl_DeleteInterp (
      interp          : Tcl_Interp
   );
   pragma Import (C, Tcl_DeleteInterp, "Tcl_DeleteInterp");

   procedure Tcl_DeleteTimerHandler (
      token           : Tcl_TimerToken
   );
   pragma Import (C, Tcl_DeleteTimerHandler, "Tcl_DeleteTimerHandler");

   procedure Tcl_DeleteTrace (
      interp          : Tcl_Interp;
      trace           : Tcl_Trace
   );
   pragma Import (C, Tcl_DeleteTrace, "Tcl_DeleteTrace");

   procedure Tcl_DetachPids (
      numPids         : C.Int;
      pidPtr          : Tcl_Pid
   );
   pragma Import (C, Tcl_DetachPids, "Tcl_DetachPids");

   procedure Tcl_DontCallWhenDeleted (
      interp          : Tcl_Interp;
      proc            : Tcl_InterpDeleteProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_DontCallWhenDeleted, "Tcl_DontCallWhenDeleted");

   function Tcl_DoOneEvent (
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_DoOneEvent, "Tcl_DoOneEvent");

   procedure Tcl_DoWhenIdle (
      proc            : Tcl_IdleProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_DoWhenIdle, "Tcl_DoWhenIdle");

   function Tcl_DStringAppend (
      dsPtr           : Tcl_DString;
      strng           : C.Strings.Chars_Ptr;
      length          : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_DStringAppend, "Tcl_DStringAppend");

   function Tcl_DStringAppendElement (
      dsPtr           : Tcl_DString;
      strng           : C.Strings.Chars_Ptr
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_DStringAppendElement, "Tcl_DStringAppendElement");

   procedure Tcl_DStringEndSublist (
      dsPtr           : Tcl_DString
   );
   pragma Import (C, Tcl_DStringEndSublist, "Tcl_DStringEndSublist");

   procedure Tcl_DStringFree (
      dsPtr           : Tcl_DString
   );
   pragma Import (C, Tcl_DStringFree, "Tcl_DStringFree");

   procedure Tcl_DStringGetResult (
      interp          : Tcl_Interp;
      dsPtr           : Tcl_DString
   );
   pragma Import (C, Tcl_DStringGetResult, "Tcl_DStringGetResult");

   procedure Tcl_DStringInit (
      dsPtr           : Tcl_DString
   );
   pragma Import (C, Tcl_DStringInit, "Tcl_DStringInit");

   procedure Tcl_DStringResult (
      interp          : Tcl_Interp;
      dsPtr           : Tcl_DString
   );
   pragma Import (C, Tcl_DStringResult, "Tcl_DStringResult");

   procedure Tcl_DStringStartSublist (
      dsPtr           : Tcl_DString
   );
   pragma Import (C, Tcl_DStringStartSublist, "Tcl_DStringStartSublist");

   function Tcl_DuplicateObj (
      objPtr          : Tcl_Obj
   ) return Tcl_Obj;
   pragma Import (C, Tcl_DuplicateObj, "Tcl_DuplicateObj");

   function Tcl_Eof (
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_Eof, "Tcl_Eof");

   function Tcl_ErrnoId return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_ErrnoId, "Tcl_ErrnoId");

   function Tcl_ErrnoMsg (
      err             : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_ErrnoMsg, "Tcl_ErrnoMsg");

   function Tcl_Eval (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_Eval, "Tcl_Eval");

   function Tcl_EvalFile (
      interp          : Tcl_Interp;
      fileName        : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_EvalFile, "Tcl_EvalFile");

   procedure Tcl_EventuallyFree (
      data            : ClientData;
      freeProc        : Tcl_FreeProc
   );
   pragma Import (C, Tcl_EventuallyFree, "Tcl_EventuallyFree");

   function Tcl_EvalObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_EvalObj, "Tcl_EvalObj");

   procedure Tcl_Exit (
      status          : C.Int
   );
   pragma Import (C, Tcl_Exit, "Tcl_Exit");

   function Tcl_ExposeCommand (
      interp          : Tcl_Interp;
      hiddenCmdToken  : C.Strings.Chars_Ptr;
      cmdName         : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_ExposeCommand, "Tcl_ExposeCommand");

   function Tcl_ExprBoolean (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      ptr             : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ExprBoolean, "Tcl_ExprBoolean");

   function Tcl_ExprBooleanObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      ptr             : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ExprBooleanObj, "Tcl_ExprBooleanObj");

   function Tcl_ExprDouble (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      ptr             : access C.Double
   ) return C.Int;
   pragma Import (C, Tcl_ExprDouble, "Tcl_ExprDouble");

   function Tcl_ExprDoubleObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      ptr             : access C.Double
   ) return C.Int;
   pragma Import (C, Tcl_ExprDoubleObj, "Tcl_ExprDoubleObj");

   function Tcl_ExprLong (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      ptr             : access C.Long
   ) return C.Int;
   pragma Import (C, Tcl_ExprLong, "Tcl_ExprLong");

   function Tcl_ExprLongObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      ptr             : access C.Long
   ) return C.Int;
   pragma Import (C, Tcl_ExprLongObj, "Tcl_ExprLongObj");

   function Tcl_ExprObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      resultPtrPtr    : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_ExprObj, "Tcl_ExprObj");

   function Tcl_ExprString (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_ExprString, "Tcl_ExprString");

   procedure Tcl_Finalize;
   pragma Import (C, Tcl_Finalize, "Tcl_Finalize");

   procedure Tcl_FindExecutable (
      argv0           : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_FindExecutable, "Tcl_FindExecutable");

   function Tcl_FirstHashEntry (
      tablePtr        : Tcl_HashTable;
      searchPtr       : Tcl_HashSearch
   ) return Tcl_HashEntry;
   pragma Import (C, Tcl_FirstHashEntry, "Tcl_FirstHashEntry");

   function Tcl_Flush (
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_Flush, "Tcl_Flush");

   procedure TclFreeObj (
      objPtr          : Tcl_Obj
   );
   pragma Import (C, TclFreeObj, "TclFreeObj");

   procedure Tcl_FreeResult (
      interp          : Tcl_Interp
   );
   pragma Import (C, Tcl_FreeResult, "Tcl_FreeResult");

   function Tcl_GetAlias (
      interp          : Tcl_Interp;
      slaveCmd        : C.Strings.Chars_Ptr;
      targetInterpPtr : Tcl_Interp;
      targetCmdPtr    : Tash_CArgv.Chars_Ptr_Ptr;
      argcPtr         : access C.Int;
      argvPtr         : access Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_GetAlias, "Tcl_GetAlias");

   function Tcl_GetAliasObj (
      interp          : Tcl_Interp;
      slaveCmd        : C.Strings.Chars_Ptr;
      targetInterpPtr : Tcl_Interp;
      targetCmdPtr    : Tash_CArgv.Chars_Ptr_Ptr;
      objcPtr         : access C.Int;
      objv            : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_GetAliasObj, "Tcl_GetAliasObj");

   function Tcl_GetAssocData (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      procPtr         : Tcl_InterpDeleteProc
   ) return ClientData;
   pragma Import (C, Tcl_GetAssocData, "Tcl_GetAssocData");

   function Tcl_GetBoolean (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      boolPtr         : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_GetBoolean, "Tcl_GetBoolean");

   function Tcl_GetBooleanFromObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      boolPtr         : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_GetBooleanFromObj, "Tcl_GetBooleanFromObj");

   function Tcl_GetChannel (
      interp          : Tcl_Interp;
      chanName        : C.Strings.Chars_Ptr;
      modePtr         : access C.Int
   ) return Tcl_Channel;
   pragma Import (C, Tcl_GetChannel, "Tcl_GetChannel");

   function Tcl_GetChannelBufferSize (
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_GetChannelBufferSize, "Tcl_GetChannelBufferSize");

   function Tcl_GetChannelHandle (
      chan            : Tcl_Channel;
      direction       : C.Int;
      handleptr       : ClientData
   ) return C.Int;
   pragma Import (C, Tcl_GetChannelHandle, "Tcl_GetChannelHandle");

   function Tcl_GetChannelInstanceData (
      chan            : Tcl_Channel
   ) return ClientData;
   pragma Import (C, Tcl_GetChannelInstanceData, "Tcl_GetChannelInstanceData");

   function Tcl_GetChannelMode (
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_GetChannelMode, "Tcl_GetChannelMode");

   function Tcl_GetChannelName (
      chan            : Tcl_Channel
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetChannelName, "Tcl_GetChannelName");

   function Tcl_GetChannelOption (
      interp          : Tcl_Interp;
      chan            : Tcl_Channel;
      optionName      : C.Strings.Chars_Ptr;
      dsPtr           : Tcl_DString
   ) return C.Int;
   pragma Import (C, Tcl_GetChannelOption, "Tcl_GetChannelOption");

   function Tcl_GetChannelType (
      chan            : Tcl_Channel
   ) return Tcl_ChannelType;
   pragma Import (C, Tcl_GetChannelType, "Tcl_GetChannelType");

   function Tcl_GetCommandInfo (
      interp          : Tcl_Interp;
      cmdName         : C.Strings.Chars_Ptr;
      infoPtr         : Tcl_CmdInfo
   ) return C.Int;
   pragma Import (C, Tcl_GetCommandInfo, "Tcl_GetCommandInfo");

   function Tcl_GetCommandName (
      interp          : Tcl_Interp;
      command         : Tcl_Command
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetCommandName, "Tcl_GetCommandName");

   function Tcl_GetDouble (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      doublePtr       : access C.Double
   ) return C.Int;
   pragma Import (C, Tcl_GetDouble, "Tcl_GetDouble");

   function Tcl_GetDoubleFromObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      doublePtr       : access C.Double
   ) return C.Int;
   pragma Import (C, Tcl_GetDoubleFromObj, "Tcl_GetDoubleFromObj");

   function Tcl_GetErrno return C.Int;
   pragma Import (C, Tcl_GetErrno, "Tcl_GetErrno");

   function Tcl_GetErrorLine (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tcl_GetErrorLine, "Tcl_GetErrorLine");

   function Tcl_GetHostName return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetHostName, "Tcl_GetHostName");

   function Tcl_GetIndexFromObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      tablePtr        : Tash_CArgv.Chars_Ptr_Ptr;
      msg             : C.Strings.Chars_Ptr;
      flags           : C.Int;
      indexPtr        : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_GetIndexFromObj, "Tcl_GetIndexFromObj");

   function Tcl_GetInt (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      intPtr          : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_GetInt, "Tcl_GetInt");

   function Tcl_GetInterpPath (
      askInterp       : Tcl_Interp;
      slaveInterp     : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tcl_GetInterpPath, "Tcl_GetInterpPath");

   function Tcl_GetIntFromObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      intPtr          : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_GetIntFromObj, "Tcl_GetIntFromObj");

   function Tcl_GetLongFromObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj;
      longPtr         : access C.Long
   ) return C.Int;
   pragma Import (C, Tcl_GetLongFromObj, "Tcl_GetLongFromObj");

   function Tcl_GetMaster (
      interp          : Tcl_Interp
   ) return Tcl_Interp;
   pragma Import (C, Tcl_GetMaster, "Tcl_GetMaster");

   function Tcl_GetObjResult (
      interp          : Tcl_Interp
   ) return Tcl_Obj;
   pragma Import (C, Tcl_GetObjResult, "Tcl_GetObjResult");

   function Tcl_GetObjType (
      typeName        : C.Strings.Chars_Ptr
   ) return Tcl_ObjType;
   pragma Import (C, Tcl_GetObjType, "Tcl_GetObjType");

   procedure Tcl_GetOpenFile (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      write           : C.Int;
      checkUsage      : C.Int;
      fileptr         : ClientData
   );
   pragma Import (C, Tcl_GetOpenFile, "Tcl_GetOpenFile");

   function Tcl_GetOriginalCommand (
      command         : Tcl_Command
   ) return Tcl_Command;
   pragma Import (C, Tcl_GetOriginalCommand, "Tcl_GetOriginalCommand");

   function Tcl_GetPathType (
      path            : C.Strings.Chars_Ptr
   ) return Tcl_PathType;
   pragma Import (C, Tcl_GetPathType, "Tcl_GetPathType");

   function Tcl_Gets (
      chan            : Tcl_Channel;
      dsPtr           : Tcl_DString
   ) return C.Int;
   pragma Import (C, Tcl_Gets, "Tcl_Gets");

   function Tcl_GetsObj (
      chan            : Tcl_Channel;
      objPtr          : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_GetsObj, "Tcl_GetsObj");

   function Tcl_GetServiceMode return C.Int;
   pragma Import (C, Tcl_GetServiceMode, "Tcl_GetServiceMode");

   function Tcl_GetSlave (
      interp          : Tcl_Interp;
      slaveName       : C.Strings.Chars_Ptr
   ) return Tcl_Interp;
   pragma Import (C, Tcl_GetSlave, "Tcl_GetSlave");

   function Tcl_GetStdChannel (
      typ             : C.Int
   ) return Tcl_Channel;
   pragma Import (C, Tcl_GetStdChannel, "Tcl_GetStdChannel");

   function Tcl_GetStringFromObj (
      objPtr          : Tcl_Obj;
      lengthPtr       : access C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetStringFromObj, "Tcl_GetStringFromObj");

   function Tcl_GetStringResult (
      interp          : Tcl_Interp
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetStringResult, "Tcl_GetStringResult");

   function Tcl_GetVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetVar, "Tcl_GetVar");

   function Tcl_GetVar2 (
      interp          : Tcl_Interp;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetVar2, "Tcl_GetVar2");

   function Tcl_GlobalEval (
      interp          : Tcl_Interp;
      command         : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_GlobalEval, "Tcl_GlobalEval");

   function Tcl_GlobalEvalObj (
      interp          : Tcl_Interp;
      objPtr          : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_GlobalEvalObj, "Tcl_GlobalEvalObj");

   function Tcl_HashStats (
      tablePtr        : Tcl_HashTable
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_HashStats, "Tcl_HashStats");

   function Tcl_HideCommand (
      interp          : Tcl_Interp;
      cmdName         : C.Strings.Chars_Ptr;
      hiddenCmdToken  : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_HideCommand, "Tcl_HideCommand");

   function Tcl_Init (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tcl_Init, "Tcl_Init");

   procedure Tcl_InitHashTable (
      tablePtr        : Tcl_HashTable;
      keyType         : C.Int
   );
   pragma Import (C, Tcl_InitHashTable, "Tcl_InitHashTable");

   procedure Tcl_InitMemory (
      interp          : Tcl_Interp
   );
   pragma Import (C, Tcl_InitMemory, "Tcl_InitMemory");

   function Tcl_InputBlocked (
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_InputBlocked, "Tcl_InputBlocked");

   function Tcl_InputBuffered (
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_InputBuffered, "Tcl_InputBuffered");

   function Tcl_InterpDeleted (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tcl_InterpDeleted, "Tcl_InterpDeleted");

   function Tcl_IsSafe (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tcl_IsSafe, "Tcl_IsSafe");

   procedure Tcl_InvalidateStringRep (
      objPtr          : Tcl_Obj
   );
   pragma Import (C, Tcl_InvalidateStringRep, "Tcl_InvalidateStringRep");

   function Tcl_JoinPath (
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      resultPtr       : Tcl_DString
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_JoinPath, "Tcl_JoinPath");

   function Tcl_LinkVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr;
      addr            : C.Strings.Chars_Ptr;
      typ             : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_LinkVar, "Tcl_LinkVar");

   function Tcl_ListObjAppendList (
      interp          : Tcl_Interp;
      listPtr         : Tcl_Obj;
      elemListPtr     : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_ListObjAppendList, "Tcl_ListObjAppendList");

   function Tcl_ListObjAppendElement (
      interp          : Tcl_Interp;
      listPtr         : Tcl_Obj;
      objPtr          : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_ListObjAppendElement, "Tcl_ListObjAppendElement");

   function Tcl_ListObjGetElements (
      interp          : Tcl_Interp;
      listPtr         : Tcl_Obj;
      objcPtr         : access C.Int;
      objvPtr         : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_ListObjGetElements, "Tcl_ListObjGetElements");

   function Tcl_ListObjIndex (
      interp          : Tcl_Interp;
      listPtr         : Tcl_Obj;
      index           : C.Int;
      objPtrPtr       : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_ListObjIndex, "Tcl_ListObjIndex");

   function Tcl_ListObjLength (
      interp          : Tcl_Interp;
      listPtr         : Tcl_Obj;
      intPtr          : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ListObjLength, "Tcl_ListObjLength");

   function Tcl_ListObjReplace (
      interp          : Tcl_Interp;
      listPtr         : Tcl_Obj;
      first           : C.Int;
      count           : C.Int;
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   ) return C.Int;
   pragma Import (C, Tcl_ListObjReplace, "Tcl_ListObjReplace");

   procedure Tcl_Main (
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      appInitProc     : Tcl_AppInitProc
   );
   pragma Import (C, Tcl_Main, "Tcl_Main");

   function Tcl_MakeFileChannel (
      handle          : ClientData;
      mode            : C.Int
   ) return Tcl_Channel;
   pragma Import (C, Tcl_MakeFileChannel, "Tcl_MakeFileChannel");

   function Tcl_MakeSafe (
      interp          : Tcl_Interp
   ) return C.Int;
   pragma Import (C, Tcl_MakeSafe, "Tcl_MakeSafe");

   function Tcl_MakeTcpClientChannel (
      tcpsocket       : ClientData
   ) return Tcl_Channel;
   pragma Import (C, Tcl_MakeTcpClientChannel, "Tcl_MakeTcpClientChannel");

   function Tcl_Merge (
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_Merge, "Tcl_Merge");

   function Tcl_NextHashEntry (
      searchPtr       : Tcl_HashSearch
   ) return Tcl_HashEntry;
   pragma Import (C, Tcl_NextHashEntry, "Tcl_NextHashEntry");

   procedure Tcl_NotifyChannel (
      channel         : Tcl_Channel;
      mask            : C.Int
   );
   pragma Import (C, Tcl_NotifyChannel, "Tcl_NotifyChannel");

   function Tcl_ObjGetVar2 (
      interp          : Tcl_Interp;
      part1Ptr        : Tcl_Obj;
      part2Ptr        : Tcl_Obj;
      flags           : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_ObjGetVar2, "Tcl_ObjGetVar2");

   function Tcl_ObjSetVar2 (
      interp          : Tcl_Interp;
      part1Ptr        : Tcl_Obj;
      part2Ptr        : Tcl_Obj;
      newValuePtr     : Tcl_Obj;
      flags           : C.Int
   ) return Tcl_Obj;
   pragma Import (C, Tcl_ObjSetVar2, "Tcl_ObjSetVar2");

   function Tcl_OpenCommandChannel (
      interp          : Tcl_Interp;
      argc            : C.Int;
      argv            : Tash_CArgv.Chars_Ptr_Ptr;
      flags           : C.Int
   ) return Tcl_Channel;
   pragma Import (C, Tcl_OpenCommandChannel, "Tcl_OpenCommandChannel");

   function Tcl_OpenFileChannel (
      interp          : Tcl_Interp;
      fileName        : C.Strings.Chars_Ptr;
      modeString      : C.Strings.Chars_Ptr;
      permissions     : C.Int
   ) return Tcl_Channel;
   pragma Import (C, Tcl_OpenFileChannel, "Tcl_OpenFileChannel");

   function Tcl_OpenTcpClient (
      interp          : Tcl_Interp;
      port            : C.Int;
      address         : C.Strings.Chars_Ptr;
      myaddr          : C.Strings.Chars_Ptr;
      myport          : C.Int;
      async           : C.Int
   ) return Tcl_Channel;
   pragma Import (C, Tcl_OpenTcpClient, "Tcl_OpenTcpClient");

   function Tcl_OpenTcpServer (
      interp          : Tcl_Interp;
      port            : C.Int;
      host            : C.Strings.Chars_Ptr;
      acceptProc      : Tcl_TcpAcceptProc;
      callbackdata    : ClientData
   ) return Tcl_Channel;
   pragma Import (C, Tcl_OpenTcpServer, "Tcl_OpenTcpServer");

   function Tcl_ParseVar (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      termPtr         : Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_ParseVar, "Tcl_ParseVar");

   function Tcl_PkgProvide (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      version         : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_PkgProvide, "Tcl_PkgProvide");

   function Tcl_PkgRequire (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      version         : C.Strings.Chars_Ptr;
      exact           : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_PkgRequire, "Tcl_PkgRequire");

   function Tcl_PosixError (
      interp          : Tcl_Interp
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_PosixError, "Tcl_PosixError");

   procedure Tcl_Preserve (
      data            : ClientData
   );
   pragma Import (C, Tcl_Preserve, "Tcl_Preserve");

   procedure Tcl_PrintDouble (
      interp          : Tcl_Interp;
      value           : C.Double;
      dst             : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_PrintDouble, "Tcl_PrintDouble");

   function Tcl_PutEnv (
      strng           : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_PutEnv, "Tcl_PutEnv");

   procedure Tcl_QueueEvent (
      evPtr           : Tcl_Event;
      position        : Tcl_QueuePosition
   );
   pragma Import (C, Tcl_QueueEvent, "Tcl_QueueEvent");

   function Tcl_Read (
      chan            : Tcl_Channel;
      bufPtr          : C.Strings.Chars_Ptr;
      toRead          : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_Read, "Tcl_Read");

   procedure Tcl_ReapDetachedProcs;
   pragma Import (C, Tcl_ReapDetachedProcs, "Tcl_ReapDetachedProcs");

   function Tcl_RecordAndEval (
      interp          : Tcl_Interp;
      cmd             : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_RecordAndEval, "Tcl_RecordAndEval");

   function Tcl_RecordAndEvalObj (
      interp          : Tcl_Interp;
      cmdPtr          : Tcl_Obj;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_RecordAndEvalObj, "Tcl_RecordAndEvalObj");

   function Tcl_RegExpCompile (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr
   ) return Tcl_RegExp;
   pragma Import (C, Tcl_RegExpCompile, "Tcl_RegExpCompile");

   function Tcl_RegExpExec (
      interp          : Tcl_Interp;
      regexp          : Tcl_RegExp;
      strng           : C.Strings.Chars_Ptr;
      start           : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_RegExpExec, "Tcl_RegExpExec");

   function Tcl_RegExpMatch (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      pattern         : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_RegExpMatch, "Tcl_RegExpMatch");

   procedure Tcl_RegExpRange (
      regexp          : Tcl_RegExp;
      index           : C.Int;
      startPtr        : Tash_CArgv.Chars_Ptr_Ptr;
      endPtr          : Tash_CArgv.Chars_Ptr_Ptr
   );
   pragma Import (C, Tcl_RegExpRange, "Tcl_RegExpRange");

   procedure Tcl_RegisterChannel (
      interp          : Tcl_Interp;
      chan            : Tcl_Channel
   );
   pragma Import (C, Tcl_RegisterChannel, "Tcl_RegisterChannel");

   procedure Tcl_RegisterObjType (
      typePtr         : Tcl_ObjType
   );
   pragma Import (C, Tcl_RegisterObjType, "Tcl_RegisterObjType");

   procedure Tcl_Release (
      data            : ClientData
   );
   pragma Import (C, Tcl_Release, "Tcl_Release");

   procedure Tcl_RestartIdleTimer;
   pragma Import (C, Tcl_RestartIdleTimer, "Tcl_RestartIdleTimer");

   procedure Tcl_ResetResult (
      interp          : Tcl_Interp
   );
   pragma Import (C, Tcl_ResetResult, "Tcl_ResetResult");

   -- Tcl_Return is not implemented because its name conflicts with
   -- an enumeration literal.  Use Tcl_SetResult instead.

   function Tcl_ScanCountedElement (
      strng           : C.Strings.Chars_Ptr;
      length          : C.Int;
      flagPtr         : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ScanCountedElement, "Tcl_ScanCountedElement");

   function Tcl_ScanElement (
      strng           : C.Strings.Chars_Ptr;
      flagPtr         : access C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ScanElement, "Tcl_ScanElement");

   function Tcl_Seek (
      chan            : Tcl_Channel;
      offset          : C.Int;
      mode            : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_Seek, "Tcl_Seek");

   function Tcl_ServiceAll return C.Int;
   pragma Import (C, Tcl_ServiceAll, "Tcl_ServiceAll");

   function Tcl_ServiceEvent (
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_ServiceEvent, "Tcl_ServiceEvent");

   procedure Tcl_SetAssocData (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      proc            : Tcl_InterpDeleteProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_SetAssocData, "Tcl_SetAssocData");

   procedure Tcl_SetBooleanObj (
      objPtr          : Tcl_Obj;
      boolValue       : C.Int
   );
   pragma Import (C, Tcl_SetBooleanObj, "Tcl_SetBooleanObj");

   procedure Tcl_SetChannelBufferSize (
      chan            : Tcl_Channel;
      sz              : C.Int
   );
   pragma Import (C, Tcl_SetChannelBufferSize, "Tcl_SetChannelBufferSize");

   function Tcl_SetChannelOption (
      interp          : Tcl_Interp;
      chan            : Tcl_Channel;
      optionName      : C.Strings.Chars_Ptr;
      newValue        : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_SetChannelOption, "Tcl_SetChannelOption");

   function Tcl_SetCommandInfo (
      interp          : Tcl_Interp;
      cmdName         : C.Strings.Chars_Ptr;
      infoPtr         : Tcl_CmdInfo
   ) return C.Int;
   pragma Import (C, Tcl_SetCommandInfo, "Tcl_SetCommandInfo");

   procedure Tcl_SetDoubleObj (
      objPtr          : Tcl_Obj;
      doubleValue     : C.Double
   );
   pragma Import (C, Tcl_SetDoubleObj, "Tcl_SetDoubleObj");

   procedure Tcl_SetErrno (
      err             : C.Int
   );
   pragma Import (C, Tcl_SetErrno, "Tcl_SetErrno");

   procedure Tcl_SetErrorCode (
      arg1            : Tcl_Interp;
      String1         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String2         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String3         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String4         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String5         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String6         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String7         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String8         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String9         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr
   );
   pragma Import (C, Tcl_SetErrorCode, "Tcl_SetErrorCode");

   procedure Tcl_SetIntObj (
      objPtr          : Tcl_Obj;
      intValue        : C.Int
   );
   pragma Import (C, Tcl_SetIntObj, "Tcl_SetIntObj");

   procedure Tcl_SetListObj (
      objPtr          : Tcl_Obj;
      objc            : C.Int;
      CONST_objv      : Tcl_Obj
   );
   pragma Import (C, Tcl_SetListObj, "Tcl_SetListObj");

   procedure Tcl_SetLongObj (
      objPtr          : Tcl_Obj;
      longValue       : C.Long
   );
   pragma Import (C, Tcl_SetLongObj, "Tcl_SetLongObj");

   procedure Tcl_SetMaxBlockTime (
      timePtr         : Tcl_Time
   );
   pragma Import (C, Tcl_SetMaxBlockTime, "Tcl_SetMaxBlockTime");

   procedure Tcl_SetObjErrorCode (
      interp          : Tcl_Interp;
      errorObjPtr     : Tcl_Obj
   );
   pragma Import (C, Tcl_SetObjErrorCode, "Tcl_SetObjErrorCode");

   procedure Tcl_SetObjLength (
      objPtr          : Tcl_Obj;
      length          : C.Int
   );
   pragma Import (C, Tcl_SetObjLength, "Tcl_SetObjLength");

   procedure Tcl_SetObjResult (
      interp          : Tcl_Interp;
      resultObjPtr    : Tcl_Obj
   );
   pragma Import (C, Tcl_SetObjResult, "Tcl_SetObjResult");

   type Tcl_PanicProc is access procedure (
      format          : C.Strings.Chars_Ptr;
      String1         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String2         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String3         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String4         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String5         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String6         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String7         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String8         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String9         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr
   );
   pragma Convention (C, Tcl_PanicProc);

   procedure Tcl_SetPanicProc (
      panicProc       : Tcl_PanicProc
   );
   pragma Import (C, Tcl_SetPanicProc, "Tcl_SetPanicProc");

   function Tcl_SetRecursionLimit (
      interp          : Tcl_Interp;
      depth           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_SetRecursionLimit, "Tcl_SetRecursionLimit");

   procedure Tcl_SetResult (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      freeProc        : Tcl_FreeProc
   );
   pragma Import (C, Tcl_SetResult, "Tcl_SetResult");

   procedure Tcl_SetResult (
      interp          : Tcl_Interp;
      strng           : C.Strings.Chars_Ptr;
      freeProc        : C.Int
   );
   pragma Import (C, Tcl_SetResult, "Tcl_SetResult");

   function Tcl_SetServiceMode (
      mode            : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_SetServiceMode, "Tcl_SetServiceMode");

   procedure Tcl_SetStdChannel (
      channel         : Tcl_Channel;
      typ             : C.Int
   );
   pragma Import (C, Tcl_SetStdChannel, "Tcl_SetStdChannel");

   procedure Tcl_SetStringObj (
      objPtr          : Tcl_Obj;
      bytes           : C.Strings.Chars_Ptr;
      length          : C.Int
   );
   pragma Import (C, Tcl_SetStringObj, "Tcl_SetStringObj");

   procedure Tcl_SetTimer (
      timePtr         : Tcl_Time
   );
   pragma Import (C, Tcl_SetTimer, "Tcl_SetTimer");

   function Tcl_SetVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr;
      newValue        : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_SetVar, "Tcl_SetVar");

   function Tcl_SetVar2 (
      interp          : Tcl_Interp;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      newValue        : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_SetVar2, "Tcl_SetVar2");

   function Tcl_SignalId (
      sig             : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_SignalId, "Tcl_SignalId");

   function Tcl_SignalMsg (
      sig             : C.Int
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_SignalMsg, "Tcl_SignalMsg");

   procedure Tcl_Sleep (
      ms              : C.Int
   );
   pragma Import (C, Tcl_Sleep, "Tcl_Sleep");

   procedure Tcl_SourceRCFile (
      interp          : Tcl_Interp
   );
   pragma Import (C, Tcl_SourceRCFile, "Tcl_SourceRCFile");

   function Tcl_SplitList (
      interp          : Tcl_Interp;
      list            : C.Strings.Chars_Ptr;
      argcPtr         : access C.Int;
      argvPtr         : access Tash_CArgv.Chars_Ptr_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_SplitList, "Tcl_SplitList");

   procedure Tcl_SplitPath (
      path            : C.Strings.Chars_Ptr;
      argcPtr         : access C.Int;
      argvPtr         : access Tash_CArgv.Chars_Ptr_Ptr
   );
   pragma Import (C, Tcl_SplitPath, "Tcl_SplitPath");

   procedure Tcl_StaticPackage (
      interp          : Tcl_Interp;
      pkgName         : C.Strings.Chars_Ptr;
      initProc        : Tcl_PackageInitProc;
      safeInitProc    : Tcl_PackageInitProc
   );
   pragma Import (C, Tcl_StaticPackage, "Tcl_StaticPackage");

   function Tcl_StringMatch (
      strng           : C.Strings.Chars_Ptr;
      pattern         : C.Strings.Chars_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_StringMatch, "Tcl_StringMatch");

   function Tcl_Tell (
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_Tell, "Tcl_Tell");

   function Tcl_TranslateFileName (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      bufferPtr       : Tcl_DString
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_TranslateFileName, "Tcl_TranslateFileName");

   function Tcl_TildeSubst (
      interp          : Tcl_Interp;
      name            : C.Strings.Chars_Ptr;
      bufferPtr       : Tcl_DString
   ) return C.Strings.Chars_Ptr renames Tcl_TranslateFileName;

   procedure Tcl_TraceVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr;
      flags           : C.Int;
      proc            : Tcl_VarTraceProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_TraceVar, "Tcl_TraceVar");

   procedure Tcl_TraceVar2 (
      interp          : Tcl_Interp;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      flags           : C.Int;
      proc            : Tcl_VarTraceProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_TraceVar2, "Tcl_TraceVar2");

   function Tcl_Ungets (
      chan            : Tcl_Channel;
      str             : C.Strings.Chars_Ptr;
      len             : C.Int;
      atHead          : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_Ungets, "Tcl_Ungets");

   procedure Tcl_UnlinkVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_UnlinkVar, "Tcl_UnlinkVar");

   function Tcl_UnregisterChannel (
      interp          : Tcl_Interp;
      chan            : Tcl_Channel
   ) return C.Int;
   pragma Import (C, Tcl_UnregisterChannel, "Tcl_UnregisterChannel");

   function Tcl_UnsetVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_UnsetVar, "Tcl_UnsetVar");

   function Tcl_UnsetVar2 (
      interp          : Tcl_Interp;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_UnsetVar2, "Tcl_UnsetVar2");

   procedure Tcl_UntraceVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr;
      flags           : C.Int;
      proc            : Tcl_VarTraceProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_UntraceVar, "Tcl_UntraceVar");

   procedure Tcl_UntraceVar2 (
      interp          : Tcl_Interp;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      flags           : C.Int;
      proc            : Tcl_VarTraceProc;
      data            : ClientData
   );
   pragma Import (C, Tcl_UntraceVar2, "Tcl_UntraceVar2");

   procedure Tcl_UpdateLinkedVar (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_UpdateLinkedVar, "Tcl_UpdateLinkedVar");

   function Tcl_UpVar (
      interp          : Tcl_Interp;
      frameName       : C.Strings.Chars_Ptr;
      varName         : C.Strings.Chars_Ptr;
      localName       : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_UpVar, "Tcl_UpVar");

   function Tcl_UpVar2 (
      interp          : Tcl_Interp;
      frameName       : C.Strings.Chars_Ptr;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      localName       : C.Strings.Chars_Ptr;
      flags           : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_UpVar2, "Tcl_UpVar2");

   function Tcl_VarEval (
      interp          : Tcl_Interp;
      String1         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String2         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String3         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String4         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String5         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String6         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String7         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String8         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr;
      String9         : C.Strings.Chars_Ptr := C.Strings.Null_Ptr
   ) return C.Int;
   pragma Import (C, Tcl_VarEval, "Tcl_VarEval");

   function Tcl_VarTraceInfo (
      interp          : Tcl_Interp;
      varName         : C.Strings.Chars_Ptr;
      flags           : C.Int;
      procPtr         : Tcl_VarTraceProc;
      prevclientdata  : ClientData
   ) return ClientData;
   pragma Import (C, Tcl_VarTraceInfo, "Tcl_VarTraceInfo");

   function Tcl_VarTraceInfo2 (
      interp          : Tcl_Interp;
      part1           : C.Strings.Chars_Ptr;
      part2           : C.Strings.Chars_Ptr;
      flags           : C.Int;
      procPtr         : Tcl_VarTraceProc;
      prevclientdata  : ClientData
   ) return ClientData;
   pragma Import (C, Tcl_VarTraceInfo2, "Tcl_VarTraceInfo2");

   function Tcl_WaitForEvent (
      timePtr         : Tcl_Time
   ) return C.Int;
   pragma Import (C, Tcl_WaitForEvent, "Tcl_WaitForEvent");

   function Tcl_WaitPid (
      pid             : Tcl_Pid;
      statPtr         : access C.Int;
      options         : C.Int
   ) return Tcl_Pid;
   pragma Import (C, Tcl_WaitPid, "Tcl_WaitPid");

   function Tcl_Write (
      chan            : Tcl_Channel;
      s               : C.Strings.Chars_Ptr;
      slen            : C.Int
   ) return C.Int;
   pragma Import (C, Tcl_Write, "Tcl_Write");

   procedure Tcl_WrongNumArgs (
      interp          : Tcl_Interp;
      objc            : C.Int;
      CONST_objv      : Tcl_Obj;
      message         : C.Strings.Chars_Ptr
   );
   pragma Import (C, Tcl_WrongNumArgs, "Tcl_WrongNumArgs");

   function Tcl_GetResult (
      interp          : Tcl_Interp
   ) return C.Strings.Chars_Ptr;
   pragma Import (C, Tcl_GetResult, "Tcl_GetResult");


private
   type Tcl_AsyncHandler_rec is null record;
   type Tcl_AsyncHandler is access all Tcl_AsyncHandler_rec;
   pragma Convention (C, Tcl_AsyncHandler);

   type Tcl_AsyncHandler_array is array (C.size_t range <>) of Tcl_AsyncHandler;

   type Tcl_CallFrame_rec is
      record
         nsPtr           : Tcl_Namespace;
         dummy1          : C.Int;
         dummy2          : C.Int;
         dummy3          : C.Strings.Chars_Ptr;
         dummy4          : C.Strings.Chars_Ptr;
         dummy5          : C.Strings.Chars_Ptr;
         dummy6          : C.Int;
         dummy7          : C.Strings.Chars_Ptr;
         dummy8          : C.Strings.Chars_Ptr;
         dummy9          : C.Int;
         dummy10         : C.Strings.Chars_Ptr;
      end record;

   type Tcl_Channel_rec is null record;
   type Tcl_Channel is access all Tcl_Channel_rec;
   pragma Convention (C, Tcl_Channel);

   type Tcl_Channel_array is array (C.size_t range <>) of Tcl_Channel;

   type Tcl_ChannelType_rec is
      record
         typeName        : C.Strings.Chars_Ptr;
         -- The name of the channel type in Tcl
         -- commands. This storage is owned by
         -- channel type.
         blockModeProc   : Tcl_DriverBlockModeProc;
         -- Set blocking mode for the
         -- raw channel. May be NULL.
         closeProc       : Tcl_DriverCloseProc;
         -- Procedure to call to close
         -- the channel.
         inputProc       : Tcl_DriverInputProc;
         -- Procedure to call for input
         -- on channel.
         outputProc      : Tcl_DriverOutputProc;
         -- Procedure to call for output
         -- on channel.
         seekProc        : Tcl_DriverSeekProc;
         -- Procedure to call to seek
         -- on the channel. May be NULL.
         setOptionProc   : Tcl_DriverSetOptionProc;
         -- Set an option on a channel.
         getOptionProc   : Tcl_DriverGetOptionProc;
         -- Get an option from a channel.
         watchProc       : Tcl_DriverWatchProc;
         -- Set up the notifier to watch
         -- for events on this channel.
         getHandleProc   : Tcl_DriverGetHandleProc;
         -- Get an OS handle from the channel
         -- or NULL if not supported.
      end record;

   type Tcl_CmdInfo_rec is
      record
         isNativeObjectProc : C.Int;
         -- 1 if objProc was registered by a call to
         -- Tcl_CreateObjCommand; 0 otherwise.
         -- Tcl_SetCmdInfo does not modify this
         -- field.
         objProc         : Tcl_ObjCmdProc;
         -- Command's object-based procedure.
         objclientdata   : ClientData;
         -- ClientData for object proc.
         proc            : Tcl_CmdProc;
         -- Command's string-based procedure.
         data            : ClientData;
         -- ClientData for string proc.
         deleteProc      : Tcl_CmdDeleteProc;
         -- Procedure to call when command is
         -- deleted.
         deletedata      : ClientData;
         -- Value to pass to deleteProc {usually
         -- the same as clientData}.
         namespacePtr    : Tcl_Namespace;
         -- Points to the namespace that contains
         -- this command. Note that Tcl_SetCmdInfo
         -- will not change a command's namespace;
         -- use Tcl_RenameCommand to do that.
      end record;

   type Tcl_Command_rec is null record;
   type Tcl_Command is access all Tcl_Command_rec;
   pragma Convention (C, Tcl_Command);

   type Tcl_Command_array is array (C.size_t range <>) of Tcl_Command;

   type Tcl_DString_rec is
      record
         strng           : C.Strings.Chars_Ptr;
         -- Points to beginning of string:  either
         -- staticSpace below or a malloced array.
         length          : C.Int;
         -- Number of non-NULL characters in the
         -- string.
         spaceAvl        : C.Int;
         -- Total number of bytes available for the
         -- string and its terminating NULL char.
         staticSpace     : C.Char_Array(0..TCL_DSTRING_STATIC_SIZE-1);
         -- Space to use in common case where string
         -- is small.
      end record;

   type Tcl_Event_rec is null record;
   type Tcl_Event is access all Tcl_Event_rec;
   pragma Convention (C, Tcl_Event);

   type Tcl_Event_array is array (C.size_t range <>) of Tcl_Event;

   type Tcl_HashEntry_rec is null record;
   type Tcl_HashEntry is access all Tcl_HashEntry_rec;
   pragma Convention (C, Tcl_HashEntry);

   type Tcl_HashEntry_array is array (C.size_t range <>) of Tcl_HashEntry;

   type Tcl_HashSearch_rec is
      record
         tablePtr        : Tcl_HashTable;
         -- Table being searched.
         nextIndex       : C.Int;
         -- Index of next bucket to be
         -- enumerated after present one.
         nextEntryPtr    : Tcl_HashEntry;
         -- Next entry to be enumerated in the
         -- the current bucket.
      end record;

   type Tcl_HashTable_rec is
      record
         buckets         : Tcl_HashEntry;
         -- Pointer to bucket array.  Each
         -- element points to first entry in
         -- bucket's hash chain, or NULL.
         staticBuckets   : Tcl_HashEntry_array(0..TCL_SMALL_HASH_TABLE-1);
         -- Bucket array used for small tables
         -- {to avoid mallocs and frees}.
         numBuckets      : C.Int;
         -- Total number of buckets allocated
         -- at **bucketPtr.
         numEntries      : C.Int;
         -- Total number of entries present
         -- in table.
         rebuildSize     : C.Int;
         -- Enlarge table when numEntries gets
         -- to be this large.
         downShift       : C.Int;
         -- Shift count used in hashing
         -- function.  Designed to use high-
         -- order bits of randomized keys.
         mask            : C.Int;
         -- Mask value used in hashing
         -- function.
         keyType         : C.Int;
         -- Type of keys used in this table.
         -- It's either TCL_STRING_KEYS,
         -- TCL_ONE_WORD_KEYS, or an integer
         -- giving the number of ints that
         -- is the size of the key.
         --
         findProc        : Tcl_HashEntry;
         createProc      : Tcl_HashEntry;
      end record;

   type Tcl_Interp_rec is null record;
   type Tcl_Interp is access all Tcl_Interp_rec;
   pragma Convention (C, Tcl_Interp);

   type Tcl_Interp_array is array (C.size_t range <>) of Tcl_Interp;

   type Tcl_Namespace_rec is
      record
         name            : C.Strings.Chars_Ptr;
         -- The namespace's name within its parent
         -- namespace. This contains no ::'s. The
         -- name of the global namespace is ""
         -- although "::" is an synonym.
         fullName        : C.Strings.Chars_Ptr;
         -- The namespace's fully qualified name.
         -- This starts with ::.
         data            : ClientData;
         -- Arbitrary value associated with this
         -- namespace.
         deleteProc      : Tcl_NamespaceDeleteProc;
         -- Procedure invoked when deleting the
         -- namespace to, e.g., free clientData.
         parentPtr       : Tcl_Namespace;
         -- Points to the namespace that contains
         -- this one. NULL if this is the global
         -- namespace.
      end record;

   type Tcl_Obj_rec is null record;
   type Tcl_Obj is access all Tcl_Obj_rec;
   pragma Convention (C, Tcl_Obj);

   type Tcl_Obj_array is array (C.size_t range <>) of Tcl_Obj;

   type Tcl_ObjType_rec is
      record
         name            : C.Strings.Chars_Ptr;
         -- Name of the type, e.g. "int".
         freeIntRepProc  : Tcl_FreeInternalRepProc;
         -- Called to free any storage for the type's
         -- internal rep. NULL if the internal rep
         -- does not need freeing.
         dupIntRepProc   : Tcl_DupInternalRepProc;
         -- Called to create a new object as a copy
         -- of an existing object.
         updateStringProc : Tcl_UpdateStringProc;
         -- Called to update the string rep from the
         -- type's internal representation.
         setFromAnyProc  : Tcl_SetFromAnyProc;
         -- Called to convert the object's internal
         -- rep to this type. Frees the internal rep
         -- of the old type. Returns TCL_ERROR on
         -- failure.
      end record;

   type Tcl_Pid_rec is null record;
   type Tcl_Pid is access all Tcl_Pid_rec;
   pragma Convention (C, Tcl_Pid);

   type Tcl_Pid_array is array (C.size_t range <>) of Tcl_Pid;

   type Tcl_RegExp_rec is null record;
   type Tcl_RegExp is access all Tcl_RegExp_rec;
   pragma Convention (C, Tcl_RegExp);

   type Tcl_RegExp_array is array (C.size_t range <>) of Tcl_RegExp;

   type Tcl_Time_rec is
      record
         sec             : C.Long;
         -- Seconds.
         usec            : C.Long;
         -- Microseconds.
      end record;

   type Tcl_TimerToken_rec is null record;
   type Tcl_TimerToken is access all Tcl_TimerToken_rec;
   pragma Convention (C, Tcl_TimerToken);

   type Tcl_TimerToken_array is array (C.size_t range <>) of Tcl_TimerToken;

   type Tcl_Trace_rec is null record;
   type Tcl_Trace is access all Tcl_Trace_rec;
   pragma Convention (C, Tcl_Trace);

   type Tcl_Trace_array is array (C.size_t range <>) of Tcl_Trace;

   type Tcl_Value_rec is
      record
         typ             : Tcl_ValueType;
         -- Indicates intValue or doubleValue is
         -- valid, or both.
         intValue        : C.Long;
         -- Integer value.
         doubleValue     : C.Double;
         -- Double-precision floating value.
      end record;

   type Tcl_Var_rec is null record;
   type Tcl_Var is access all Tcl_Var_rec;
   pragma Convention (C, Tcl_Var);

   type Tcl_Var_array is array (C.size_t range <>) of Tcl_Var;


end Tcl;
